@@ -1,4 +1,4 @@
-/* $Id: asapinstance.c 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: asapinstance.c 2263 2009-10-26 08:46:55Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -19,7 +19,7 @@
  * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software: you can redistribute it and/or modify
+ * This program is free software: you can redistribute it 1113and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
@@ -198,17 +198,26 @@ struct ASAPInstance* asapInstanceNew(struct Dispatcher*          dispatcher,
          }
          setNonBlocking(asapInstance->MainLoopPipe[0]);
          setNonBlocking(asapInstance->MainLoopPipe[1]);
-         if(pthread_create(&asapInstance->MainLoopThread, NULL, &asapInstanceMainLoop, asapInstance) != 0) {
-            logerror("Unable to create ASAP main loop thread");
-            asapInstanceDelete(asapInstance);
-            return(NULL);
-         }
+         /* The ASAP main loop thread cannot be started here, since the
+            static registrar entries are still missing. The can be added
+            later. After that, asapInstanceStartThread() has to be called. */
       }
    }
    return(asapInstance);
 }
 
 
+/* ###### Constructor #################################################### */
+bool asapInstanceStartThread(struct ASAPInstance* asapInstance)
+{
+   if(pthread_create(&asapInstance->MainLoopThread, NULL, &asapInstanceMainLoop, asapInstance) != 0) {
+      logerror("Unable to create ASAP main loop thread");
+      return(false);
+   }
+   return(true);
+}
+
+
 /* ###### Destructor ##################################################### */
 void asapInstanceDelete(struct ASAPInstance* asapInstance)
 {
@@ -216,7 +225,9 @@ void asapInstanceDelete(struct ASAPInstance* asapInstance)
 
    if(asapInstance) {
       if(asapInstance->MainLoopThread != 0) {
+         dispatcherLock(asapInstance->StateMachine);
          asapInstance->MainLoopShutdown = true;
+         dispatcherUnlock(asapInstance->StateMachine);
          asapInstanceNotifyMainLoop(asapInstance);
          CHECK(pthread_join(asapInstance->MainLoopThread, NULL) == 0);
          asapInstance->MainLoopThread = 0;
@@ -279,9 +290,9 @@ static void asapInstanceConfigure(struct ASAPInstance* asapInstance,
    /* ====== Show results =================================================== */
    LOG_VERBOSE3
    fputs("New ASAP instance's configuration:\n", stdlog);
-   fprintf(stdlog, "registrar.request.maxtrials   = %u\n",     (unsigned int)asapInstance->RegistrarRequestMaxTrials);
    fprintf(stdlog, "registrar.request.timeout     = %lluus\n", asapInstance->RegistrarRequestTimeout);
    fprintf(stdlog, "registrar.response.timeout    = %lluus\n", asapInstance->RegistrarResponseTimeout);
+   fprintf(stdlog, "registrar.request.maxtrials   = %u\n",     (unsigned int)asapInstance->RegistrarRequestMaxTrials);
    LOG_END
 }
 
@@ -290,8 +301,9 @@ static void asapInstanceConfigure(struct ASAPInstance* asapInstance,
 static bool asapInstanceConnectToRegistrar(struct ASAPInstance* asapInstance,
                                            int                  sd)
 {
+   RegistrarIdentifierType registrarIdentifier;
 #ifdef HAVE_SCTP_DELAYED_SACK
-   struct sctp_sack_info sctpSACKInfo;
+   struct sctp_sack_info   sctpSACKInfo;
 #endif
 
    if(asapInstance->RegistrarSocket < 0) {
@@ -304,8 +316,11 @@ static bool asapInstanceConnectToRegistrar(struct ASAPInstance* asapInstance,
             sd = registrarTableGetRegistrar(asapInstance->RegistrarSet,
                                             asapInstance->RegistrarHuntSocket,
                                             asapInstance->RegistrarHuntMessageBuffer,
-                                            &asapInstance->RegistrarIdentifier);
+                                            &registrarIdentifier);
          }
+         dispatcherLock(asapInstance->StateMachine);
+         asapInstance->RegistrarIdentifier = registrarIdentifier;
+         dispatcherUnlock(asapInstance->StateMachine);
          if(sd < 0) {
             LOG_ACTION
             fputs("Unable to connect to a registrar\n", stdlog);
@@ -354,8 +369,10 @@ static void asapInstanceDisconnectFromRegistrar(struct ASAPInstance* asapInstanc
                                                 bool                 sendAbort)
 {
    if(asapInstance->RegistrarSocket >= 0) {
+      dispatcherLock(asapInstance->StateMachine);
       timerStop(&asapInstance->RegistrarTimeoutTimer);
       fdCallbackDelete(&asapInstance->RegistrarFDCallback);
+      dispatcherUnlock(asapInstance->StateMachine);
       if(sendAbort) {
          sendabort(asapInstance->RegistrarSocket, 0);
       }
@@ -384,7 +401,9 @@ static unsigned int asapInstanceSendRequest(struct ASAPInstance*    asapInstance
       return(RSPERR_OUT_OF_MEMORY);
    }
 
+// ???   dispatcherLock(asapInstance->StateMachine);
    interThreadMessagePortEnqueue(&asapInstance->MainLoopPort, &aitm->Node, NULL);
+// ???   dispatcherUnlock(asapInstance->StateMachine);
    asapInstanceNotifyMainLoop(asapInstance);
    return(RSPERR_OKAY);
 }
@@ -402,7 +421,9 @@ static unsigned int asapInstanceBeginIO(struct ASAPInstance*           asapInsta
       return(RSPERR_OUT_OF_MEMORY);
    }
 
+// ???   dispatcherLock(asapInstance->StateMachine);
    interThreadMessagePortEnqueue(&asapInstance->MainLoopPort, &aitm->Node, itmPort);
+// ???   dispatcherUnlock(asapInstance->StateMachine);
    asapInstanceNotifyMainLoop(asapInstance);
 
    return(RSPERR_OKAY);
@@ -450,7 +471,8 @@ static unsigned int asapInstanceDoIO(struct ASAPInstance*     asapInstance,
 unsigned int asapInstanceRegister(struct ASAPInstance*              asapInstance,
                                   struct PoolHandle*                poolHandle,
                                   struct ST_CLASS(PoolElementNode)* poolElementNode,
-                                  const bool                        waitForResponse)
+                                  const bool                        waitForResponse,
+                                  const bool                        daemonMode)
 {
    struct RSerPoolMessage*           message;
    struct RSerPoolMessage*           response;
@@ -458,7 +480,6 @@ unsigned int asapInstanceRegister(struct ASAPInstance*              asapInstance
    struct ST_CLASS(PoolElementNode)* newPoolElementNode;
    struct TransportAddressBlock*     newUserTransport;
    unsigned int                      result;
-   unsigned int                      handlespaceMgtResult;
 
    LOG_VERBOSE
    fputs("Trying to register to pool ", stdlog);
@@ -487,7 +508,29 @@ unsigned int asapInstanceRegister(struct ASAPInstance*              asapInstance
                      oldPoolElementNode);
       }
       else {
-         result = RSPERR_OKAY;
+         result = ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
+                     &asapInstance->OwnPoolElements,
+                     poolHandle,
+                     message->PoolElementPtr->HomeRegistrarIdentifier,
+                     message->PoolElementPtr->Identifier,
+                     message->PoolElementPtr->RegistrationLife,
+                     &message->PoolElementPtr->PolicySettings,
+                     message->PoolElementPtr->UserTransport,
+                     NULL,
+                     -1, 0,
+                     getMicroTime(),
+                     &newPoolElementNode);
+         if(result == RSPERR_OKAY) {
+            newPoolElementNode->UserData = (void*)asapInstance;
+         }
+         else {
+            LOG_ERROR
+            fprintf(stdlog, "Unable to register pool element $%08x of pool ",
+                    poolElementNode->Identifier);
+            poolHandlePrint(poolHandle, stdlog);
+            fputs(" to OwnPoolElements\n", stdlog);
+            LOG_END_FATAL
+         }
       }
       dispatcherUnlock(asapInstance->StateMachine);
 
@@ -496,46 +539,22 @@ unsigned int asapInstanceRegister(struct ASAPInstance*              asapInstance
          if(waitForResponse) {
             result = asapInstanceDoIO(asapInstance, message, &response);
             if(result == RSPERR_OKAY) {
-               if( (response->Error == RSPERR_OKAY) &&
-                   (!(response->Flags & AHF_REGISTRATION_REJECT)) ) {
-                  dispatcherLock(asapInstance->StateMachine);
+               dispatcherLock(asapInstance->StateMachine);
 
-                  handlespaceMgtResult = ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
-                                          &asapInstance->OwnPoolElements,
-                                          poolHandle,
-                                          message->PoolElementPtr->HomeRegistrarIdentifier,
-                                          message->PoolElementPtr->Identifier,
-                                          message->PoolElementPtr->RegistrationLife,
-                                          &message->PoolElementPtr->PolicySettings,
-                                          message->PoolElementPtr->UserTransport,
-                                          NULL,
-                                          -1, 0,
-                                          getMicroTime(),
-                                          &newPoolElementNode);
-                  if(handlespaceMgtResult == RSPERR_OKAY) {
-                     newPoolElementNode->UserData = (void*)asapInstance;
-                     if(response->Identifier != poolElementNode->Identifier) {
-                        LOG_ERROR
-                        fprintf(stdlog, "Tried to register PE $%08x, got response for PE $%08x\n",
-                              poolElementNode->Identifier,
-                              response->Identifier);
-                        LOG_END_FATAL
-                     }
-                  }
-                  else {
+               if( (response->Error == RSPERR_OKAY) && (!(response->Flags & AHF_REGISTRATION_REJECT)) ) {
+                  if(response->Identifier != poolElementNode->Identifier) {
                      LOG_ERROR
-                     fprintf(stdlog, "Unable to register pool element $%08x of pool ",
-                           poolElementNode->Identifier);
-                     poolHandlePrint(poolHandle, stdlog);
-                     fputs(" to OwnPoolElements\n", stdlog);
-                     LOG_END_FATAL
+                     fprintf(stdlog, "Tried to register PE $%08x, got response for PE $%08x\n",
+                             poolElementNode->Identifier,
+                             response->Identifier);
+                     LOG_END
                   }
-
-                  dispatcherUnlock(asapInstance->StateMachine);
                }
                else {
                   result = (unsigned int)response->Error;
                }
+               dispatcherUnlock(asapInstance->StateMachine);
+               
                if(response) {
                   rserpoolMessageDelete(response);
                }
@@ -551,13 +570,13 @@ unsigned int asapInstanceRegister(struct ASAPInstance*              asapInstance
                newUserTransport = transportAddressBlockDuplicate(message->PoolElementPtr->UserTransport);
                if(newUserTransport != NULL) {
                   ST_CLASS(poolElementNodeNew)(newPoolElementNode,
-                                             message->PoolElementPtr->Identifier,
-                                             message->PoolElementPtr->HomeRegistrarIdentifier,
-                                             message->PoolElementPtr->RegistrationLife,
-                                             &message->PoolElementPtr->PolicySettings,
-                                             newUserTransport,
-                                             NULL,
-                                             -1, 0);
+                                               message->PoolElementPtr->Identifier,
+                                               message->PoolElementPtr->HomeRegistrarIdentifier,
+                                               message->PoolElementPtr->RegistrationLife,
+                                               &message->PoolElementPtr->PolicySettings,
+                                               newUserTransport,
+                                               NULL,
+                                               -1, 0);
                   message->PoolElementPtr           = newPoolElementNode;
                   message->PoolElementPtrAutoDelete = true;
                   result = asapInstanceSendRequest(asapInstance, message, true);
@@ -607,7 +626,6 @@ unsigned int asapInstanceDeregister(struct ASAPInstance*            asapInstance
    struct RSerPoolMessage* message;
    struct RSerPoolMessage* response;
    unsigned int            result;
-   unsigned int            handlespaceMgtResult;
 
    LOG_VERBOSE
    fprintf(stdlog, "Trying to deregister $%08x from pool ",identifier);
@@ -615,48 +633,50 @@ unsigned int asapInstanceDeregister(struct ASAPInstance*            asapInstance
    fputs("\n", stdlog);
    LOG_END
 
-   message = rserpoolMessageNew(NULL, ASAP_BUFFER_SIZE);
-   if(message != NULL) {
-      message->Type       = AHT_DEREGISTRATION;
-      message->Flags      = 0x00;
-      message->Handle     = *poolHandle;
-      message->Identifier = identifier;
-
-      if(waitForResponse) {
-         result = asapInstanceDoIO(asapInstance, message, &response);
-         if((result == RSPERR_OKAY) && (response->Error == RSPERR_OKAY)) {
-            if(identifier != response->Identifier) {
-               LOG_ERROR
-               fprintf(stdlog, "Tried to deregister PE $%08x, got response for PE $%08x\n",
-                       identifier,
-                       message->Identifier);
-               LOG_END_FATAL
-            }
-
-            handlespaceMgtResult = ST_CLASS(poolHandlespaceManagementDeregisterPoolElement)(
-                                      &asapInstance->OwnPoolElements,
-                                      poolHandle,
-                                      identifier);
-            if(handlespaceMgtResult != RSPERR_OKAY) {
-               LOG_ERROR
-               fprintf(stdlog, "Unable to deregister pool element $%08x of pool ",
-                       identifier);
-               poolHandlePrint(poolHandle, stdlog);
-               fputs(" from OwnPoolElements\n", stdlog);
-               LOG_END
-            }
-            if(response) {
-               rserpoolMessageDelete(response);
-            }
-         }
-         rserpoolMessageDelete(message);
-      }
-      else {
-         result = asapInstanceSendRequest(asapInstance, message, true);
-      }
+   result = ST_CLASS(poolHandlespaceManagementDeregisterPoolElement)(
+                     &asapInstance->OwnPoolElements,
+                     poolHandle,
+                     identifier);
+   if(result != RSPERR_OKAY) {
+      LOG_ERROR
+      fprintf(stdlog, "Unable to deregister pool element $%08x of pool ",
+              identifier);
+      poolHandlePrint(poolHandle, stdlog);
+      fputs(" from OwnPoolElements\n", stdlog);
+      LOG_END
    }
    else {
-      result = RSPERR_OUT_OF_MEMORY;
+      message = rserpoolMessageNew(NULL, ASAP_BUFFER_SIZE);
+      if(message != NULL) {
+         message->Type       = AHT_DEREGISTRATION;
+         message->Flags      = 0x00;
+         message->Handle     = *poolHandle;
+         message->Identifier = identifier;
+
+         if(waitForResponse) {
+            result = asapInstanceDoIO(asapInstance, message, &response);
+            if((result == RSPERR_OKAY) && (response->Error == RSPERR_OKAY)) {
+               if(identifier != response->Identifier) {
+                  LOG_ERROR
+                  fprintf(stdlog, "Tried to deregister PE $%08x, got response for PE $%08x\n",
+                          identifier,
+                          message->Identifier);
+                  LOG_END_FATAL
+               }
+
+               if(response) {
+                  rserpoolMessageDelete(response);
+               }
+            }
+            rserpoolMessageDelete(message);
+         }
+         else {
+            result = asapInstanceSendRequest(asapInstance, message, true);
+         }
+      }
+      else {
+         result = RSPERR_OUT_OF_MEMORY;
+      }
    }
 
    LOG_VERBOSE
@@ -1051,6 +1071,11 @@ static void asapInstanceHandleEndpointKeepAlive(
                                     0, 0, 0, 0,
                                     message);
       }
+      else {
+         LOG_WARNING
+         fprintf(stdlog, "KeepAlive for unknown pool element $%08x -> ignoring it\n",message->Identifier);
+         LOG_END
+      }
    }
    else {
       poolNode = ST_CLASS(poolHandlespaceNodeGetFirstPoolNode)(&asapInstance->OwnPoolElements.Handlespace);
@@ -1091,11 +1116,12 @@ static void asapInstanceHandleResponseFromRegistrar(
 
    aitm = (struct ASAPInterThreadMessage*)interThreadMessagePortDequeue(&asapInstance->MainLoopPort);
    if(aitm != NULL) {
+      /* No timeout occurred -> stop timeout timer */
+      timerStop(&asapInstance->RegistrarTimeoutTimer);
+
       if( ((response->Type == AHT_REGISTRATION_RESPONSE)      && (aitm->Request->Type == AHT_REGISTRATION))   ||
           ((response->Type == AHT_DEREGISTRATION_RESPONSE)    && (aitm->Request->Type == AHT_DEREGISTRATION)) ||
           ((response->Type == AHT_HANDLE_RESOLUTION_RESPONSE) && (aitm->Request->Type == AHT_HANDLE_RESOLUTION)) ) {
-         /* No timeout occurred -> stop timeout timer */
-         timerStop(&asapInstance->RegistrarTimeoutTimer);
 
          LOG_VERBOSE
          fprintf(stdlog, "Successfully got response ($%04x) for request ($%04x) from registrar\n"
@@ -1104,40 +1130,76 @@ static void asapInstanceHandleResponseFromRegistrar(
                  getMicroTime() - aitm->CreationTimeStamp,
                  aitm->TransmissionTimeStamp - aitm->CreationTimeStamp);
          LOG_END
-
-         aitm->Response = response;
-         if(asapInstance->LastAITM == aitm) {
-            /* No more responses are expected. */
-            asapInstance->LastAITM = NULL;
+         
+         /* Asynchronous message: print errors here! */
+         if(aitm->Node.ReplyPort == NULL) {
+            if( (response->Type == AHT_REGISTRATION_RESPONSE) &&
+                ((response->Error != RSPERR_OKAY) || (response->Flags & AHF_REGISTRATION_REJECT)) ) {
+               LOG_ERROR
+               fprintf(stdlog, "Unable to register pool element $%08x of pool ",
+                       response->Identifier);
+               poolHandlePrint(&response->Handle, stdlog);
+               fputs(": ", stdlog);
+               rserpoolErrorPrint(response->Error, stdlog);
+               fputs("\n", stdlog);
+               LOG_END
+               
+               /* If no addresses are usable, try to re-connect. May be, the
+                  host has got some usable addresses then. */
+               if( (response->Error == RSPERR_NO_USABLE_USER_ADDRESSES) ||
+                   (response->Error == RSPERR_NO_USABLE_ASAP_ADDRESSES) ) {
+                  LOG_NOTE
+                  fputs("Trying to re-connect to a registrar ...\n", stdlog);   
+                  LOG_END
+                  asapInstanceDisconnectFromRegistrar(asapInstance, true);
+               }
+            }
          }
-         if(aitm->Node.ReplyPort) {
-             /* Reply to requesting thread, if reply is desired. */
-            interThreadMessageReply(&aitm->Node);
-         }
-         else {
-            /* Sender is not interested in result, throw it away. */
-            asapInterThreadMessageDelete(aitm);
-         }
-
-         /* Schedule next response's timeout */
-         interThreadMessagePortLock(&asapInstance->MainLoopPort);
-         if(asapInstance->LastAITM != NULL) {
-            nextAITM = (struct ASAPInterThreadMessage*)interThreadMessagePortGetFirstMessage(
-                                                          &asapInstance->MainLoopPort);
-            CHECK(nextAITM != NULL);
-            timerStart(&asapInstance->RegistrarTimeoutTimer,
-                       nextAITM->ResponseTimeoutTimeStamp);
-         }
-         interThreadMessagePortUnlock(&asapInstance->MainLoopPort);
       }
       else {
-         LOG_ERROR
-         fprintf(stdlog, "Got unexpected response ($%04x) for request ($%04x) from registrar\n",
-                 response->Type, aitm->Request->Type);
-         LOG_END
+         if(response->Type == AHT_ERROR) {
+            LOG_ERROR
+            fprintf(stdlog, "Got ASAP_ERROR ($%04x) for request ($%04x) from registrar: ",
+                    response->OperationErrorCode, aitm->Request->Type);
+            rserpoolErrorPrint(response->OperationErrorCode, stdlog);
+            fputs("\n", stdlog);
+            LOG_END
+         }
+         else {
+            LOG_ERROR
+            fprintf(stdlog, "Got unexpected response ($%04x) for request ($%04x) from registrar\n",
+                    response->Type, aitm->Request->Type);
+            LOG_END
+         }
          asapInstanceDisconnectFromRegistrar(asapInstance, true);
-         rserpoolMessageDelete(response);
       }
+
+      aitm->Response = response;
+      if(asapInstance->LastAITM == aitm) {
+         /* No more responses are expected. */
+         asapInstance->LastAITM = NULL;
+      }
+      if(aitm->Node.ReplyPort) {
+          /* Reply to requesting thread, if reply is desired. */
+         interThreadMessageReply(&aitm->Node);
+      }
+      else {
+         /* Sender is not interested in result, throw it away. */
+         asapInterThreadMessageDelete(aitm);
+      }
+
+      /* Schedule next response's timeout */
+      dispatcherLock(asapInstance->StateMachine);
+      interThreadMessagePortLock(&asapInstance->MainLoopPort);
+      if(asapInstance->LastAITM != NULL) {
+         nextAITM = (struct ASAPInterThreadMessage*)interThreadMessagePortGetFirstMessage(
+                                                       &asapInstance->MainLoopPort);
+         CHECK(nextAITM != NULL);
+         timerStart(&asapInstance->RegistrarTimeoutTimer,
+                    nextAITM->ResponseTimeoutTimeStamp);
+      }
+      interThreadMessagePortUnlock(&asapInstance->MainLoopPort);
+      dispatcherUnlock(asapInstance->StateMachine);
    }
    else {
       LOG_ERROR
@@ -1157,13 +1219,13 @@ static void handleNotificationOnRegistrarSocket(struct ASAPInstance*           a
    if( (notification->sn_header.sn_type == SCTP_ASSOC_CHANGE) &&
        ((notification->sn_assoc_change.sac_state == SCTP_COMM_LOST) ||
         (notification->sn_assoc_change.sac_state == SCTP_SHUTDOWN_COMP)) ) {
-      LOG_WARNING
+      LOG_NOTE
       fputs("Registrar connection lost\n", stdlog);
       LOG_END
       asapInstanceDisconnectFromRegistrar(asapInstance, true);
    }
    else if(notification->sn_header.sn_type == SCTP_SHUTDOWN_EVENT) {
-      LOG_WARNING
+      LOG_NOTE
       fputs("Registrar connection is shutting down\n", stdlog);
       LOG_END
       asapInstanceDisconnectFromRegistrar(asapInstance, true);
@@ -1361,7 +1423,9 @@ static void asapInstanceHandleQueuedAITMs(struct ASAPInstance* asapInstance)
                LOG_WARNING
                logerror("Failed to send message to registrar");
                LOG_END
+               interThreadMessagePortUnlock(&asapInstance->MainLoopPort);
                asapInstanceDisconnectFromRegistrar(asapInstance, true);
+               interThreadMessagePortLock(&asapInstance->MainLoopPort);
                break;
             }
             aitm->TransmissionTimeStamp = getMicroTime();
@@ -1375,11 +1439,15 @@ static void asapInstanceHandleQueuedAITMs(struct ASAPInstance* asapInstance)
                asapInstance->LastAITM = aitm;
 
                /* Schedule timeout */
-               aitm->ResponseTimeoutTimeStamp = getMicroTime() + asapInstance->RegistrarResponseTimeout;
-               if(!timerIsRunning(&asapInstance->RegistrarTimeoutTimer)) {
-                  timerStart(&asapInstance->RegistrarTimeoutTimer,
-                             aitm->ResponseTimeoutTimeStamp);
-               }
+               aitm->ResponseTimeoutTimeStamp       = getMicroTime() + asapInstance->RegistrarResponseTimeout;
+               aitm->ResponseTimeoutNeedsScheduling = true;
+               // NOTE:
+               // The timer cannot be scheduled here, because a lock for
+               // asapInstance->StateMachine is required to do this.
+               // asapInstance->StateMachine cannot be locked here, since
+               // asapInstance->MainLoopPort is already locked. Trying to
+               // lock asapInstance->MainLoopPort would violate the locking
+               // order!
             }
          }
       }
@@ -1387,6 +1455,26 @@ static void asapInstanceHandleQueuedAITMs(struct ASAPInstance* asapInstance)
       aitm = nextAITM;
    }
    interThreadMessagePortUnlock(&asapInstance->MainLoopPort);
+
+
+   /* ====== Set timeout ================================================= */
+   dispatcherLock(asapInstance->StateMachine);
+   interThreadMessagePortLock(&asapInstance->MainLoopPort);
+   aitm = (struct ASAPInterThreadMessage*)interThreadMessagePortGetFirstMessage(&asapInstance->MainLoopPort);
+   while(aitm != NULL) {
+      if( (aitm->ResponseExpected) && (aitm->ResponseTimeoutNeedsScheduling) ) {
+         if(!timerIsRunning(&asapInstance->RegistrarTimeoutTimer)) {
+            // Now, the AITM's timer can be scheduled, since asapInstance->StateMachine
+            // and asapInstance->MainLoopPort are locked.
+            timerStart(&asapInstance->RegistrarTimeoutTimer,
+                       aitm->ResponseTimeoutTimeStamp);
+         }
+      }
+      aitm->ResponseTimeoutNeedsScheduling = false;         
+      aitm = (struct ASAPInterThreadMessage*)interThreadMessagePortGetNextMessage(&asapInstance->MainLoopPort, &aitm->Node);
+   }
+   interThreadMessagePortUnlock(&asapInstance->MainLoopPort);
+   dispatcherUnlock(asapInstance->StateMachine);
 }
 
 
@@ -1397,6 +1485,16 @@ static void asapInstanceNotifyMainLoop(struct ASAPInstance* asapInstance)
 }
 
 
+/* ###### Check whether ASAP instance is shutting down ################### */
+static bool asapInstanceIsShuttingDown(struct ASAPInstance* asapInstance)
+{
+   dispatcherLock(asapInstance->StateMachine);
+   const bool isShuttingDown = asapInstance->MainLoopShutdown;
+   dispatcherUnlock(asapInstance->StateMachine);
+   return(isShuttingDown);
+}
+
+
 /* ###### ASAP Instance main loop thread ################################# */
 static void* asapInstanceMainLoop(void* args)
 {
@@ -1411,17 +1509,19 @@ static void* asapInstanceMainLoop(void* args)
 
    asapInstanceConnectToRegistrar(asapInstance, -1);
 
-   while(!asapInstance->MainLoopShutdown) {
+   while(!asapInstanceIsShuttingDown(asapInstance)) {
       /* ====== Collect data for ext_select() call ======================= */
       dispatcherGetPollParameters(asapInstance->StateMachine,
-                                  (struct pollfd*)&ufds, &nfds, &timeout, &pollTimeStamp);
+                                  (struct pollfd*)&ufds, &nfds, &timeout,
+                                  &pollTimeStamp);
       pipeIndex = nfds;
       ufds[pipeIndex].fd      = asapInstance->MainLoopPipe[0];
       ufds[pipeIndex].events  = POLLIN;
       ufds[pipeIndex].revents = 0;
-      if((struct ASAPInterThreadMessage*)interThreadMessagePortGetFirstMessage(&asapInstance->MainLoopPort) !=
-            asapInstance->LastAITM) {
-         /* There are new AITM messages to be handled.
+      if(!interThreadMessagePortIsFirstMessage(&asapInstance->MainLoopPort,
+                                               &asapInstance->LastAITM->Node)) {
+         /* First message in AITM queue is not LastAITM:
+            There are new AITM messages to be handled.
             Do not block if there are no socket events! */
          timeout = 0;
       }
