@@ -1,4 +1,4 @@
-/* $Id: calcappservice.cc 1726 2008-02-14 09:56:55Z dreibh $
+/* $Id: calcappservice.cc 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 #include "netutilities.h"
 #include "timeutilities.h"
 #include "stringutilities.h"
+#include "randomizer.h"
 #include "loglevel.h"
 
 
@@ -49,7 +50,10 @@ CalcAppServer::CalcAppServer(const size_t             maxJobs,
                              const unsigned long long keepAliveTransmissionInterval,
                              const unsigned long long keepAliveTimeoutInterval,
                              const unsigned long long cookieMaxTime,
-                             const double             cookieMaxCalculations)
+                             const double             cookieMaxCalculations,
+                             const double             cleanShutdownProbability,
+                             CalcAppServerStatistics* statistics,
+                             const bool               resetStatistics)
 {
    FirstJob                      = NULL;
    ObjectName                    = objectName;
@@ -58,19 +62,21 @@ CalcAppServer::CalcAppServer(const size_t             maxJobs,
    VectorFH                      = NULL;
    ScalarFH                      = NULL;
    MaxJobs                       = maxJobs;
-   TotalUsedCalculations         = 0.0;
    Capacity                      = capacity;
    KeepAliveTimeoutInterval      = keepAliveTimeoutInterval;
    KeepAliveTransmissionInterval = keepAliveTransmissionInterval;
    CookieMaxTime                 = cookieMaxTime;
    CookieMaxCalculations         = cookieMaxCalculations;
-   StartupTimeStamp              = getMicroTime();
-
-   TotalJobsAccepted             = 0;
-   TotalJobsRejected             = 0;
-
-   VectorLine                    = 0;
+   CleanShutdownProbability      = cleanShutdownProbability;
    Jobs                          = 0;
+   Stats                         = statistics;
+   if(resetStatistics) {
+      Stats->StartupTimeStamp      = getMicroTime();
+      Stats->TotalUsedCalculations = 0.0;
+      Stats->TotalJobsAccepted     = 0;
+      Stats->TotalJobsRejected     = 0;
+      Stats->VectorLine            = 0;
+   }
 }
 
 
@@ -88,11 +94,12 @@ void CalcAppServer::printParameters()
    printf("   Vector File Name        = %s\n", VectorFileName.c_str());
    printf("   Scalar File Name        = %s\n", ScalarFileName.c_str());
    printf("   Max Jobs                = %u\n", (unsigned int)MaxJobs);
-   printf("   Capacity                = %1.1f [Calculations/s]\n",Capacity);
-   printf("   KA Transmission Interv. = %llu [us]\n", KeepAliveTransmissionInterval);
-   printf("   KA Timeout Interval     = %llu [us]\n", KeepAliveTimeoutInterval);
-   printf("   Cookie Max Time         = %llu [us]\n", CookieMaxTime);
-   printf("   Cookie Max Calculations = %1.0f [Calculations]\n", CookieMaxCalculations);
+   printf("   Capacity                = %1.1f Calculations/s\n", Capacity);
+   printf("   Clean Shutdown Prob.    = %1.2f%%\n", 100.0 * CleanShutdownProbability);
+   printf("   KA Transmission Interv. = %llums\n", KeepAliveTransmissionInterval / 1000);
+   printf("   KA Timeout Interval     = %llums\n", KeepAliveTimeoutInterval / 1000);
+   printf("   Cookie Max Time         = %1.3fs\n", CookieMaxTime / 1000000.0);
+   printf("   Cookie Max Calculations = %1.0f Calculations\n", CookieMaxCalculations);
 }
 
 
@@ -116,7 +123,7 @@ CalcAppServer::CalcAppServerJob* CalcAppServer::addJob(rserpool_session_t sessio
          job->JobCompleteTimeStamp           = ~0ULL;
          job->CookieTransmissionTimeStamp    = ~0ULL;
          job->LastUpdateAt                   = getMicroTime();
-         job->LastCookieAt                   = 0;
+         job->LastCookieAt                   = job->LastUpdateAt;
          job->LastCookieCompleted            = 0.0;
          FirstJob = job;
          Jobs++;
@@ -175,13 +182,23 @@ void CalcAppServer::removeJob(CalcAppServer::CalcAppServerJob* removedJob)
 // ###### Remove all jobs ###################################################
 void CalcAppServer::removeAllJobs()
 {
-   CalcAppServerJob* job = FirstJob;
-   while(job != NULL) {
-      CalcAppServerJob* next = job->Next;
-      sendCookie(job);   // Note, this may all removeJob()!
-      job = next;
+   // ====== Clean shutdown: send cookies to all PUs ========================
+   const double r = randomDouble();
+   if(r <= CleanShutdownProbability) {
+      // Update the jobs' progress to send correct cookies!
+      updateCalculations();
+      scheduleJobs();
+
+      // Send cookies
+      CalcAppServerJob* job = FirstJob;
+      while(job != NULL) {
+         CalcAppServerJob* next = job->Next;
+         sendCookie(job);   // Note: this function may call removeJob()!
+         job = next;
+      }
    }
 
+   // ====== Remove all jobs ================================================
    while(FirstJob) {
       removeJob(FirstJob);
    }
@@ -201,13 +218,13 @@ void CalcAppServer::updateCalculations()
          const unsigned long long elapsed   = now - job->LastUpdateAt;
          const double completedCalculations = elapsed * capacityPerJob;
          if(job->Completed + completedCalculations < job->JobSize) {
-            TotalUsedCalculations += completedCalculations;
-            job->Completed += completedCalculations;
+            Stats->TotalUsedCalculations += completedCalculations;
+            job->Completed               += completedCalculations;
          }
          else {
             CHECK(job->JobSize - job->Completed >= 0.0);
-            TotalUsedCalculations += job->JobSize - job->Completed;
-            job->Completed = job->JobSize;
+            Stats->TotalUsedCalculations += job->JobSize - job->Completed;
+            job->Completed                = job->JobSize;
          }
          job->LastUpdateAt = now;
 
@@ -227,15 +244,15 @@ void CalcAppServer::scheduleJobs()
       const double capacityPerJob = Capacity / ((double)Jobs * 1000000.0);
       while(job != NULL) {
          // When is the current session's job completed?
-         const double calculationsToGo         = job->JobSize - job->Completed;
-         const unsigned long long timeToGo     = (unsigned long long)ceil(calculationsToGo / capacityPerJob);
+         const double calculationsToGo     = job->JobSize - job->Completed;
+         const unsigned long long timeToGo = (unsigned long long)ceil(calculationsToGo / capacityPerJob);
 
          // job->JobCompleteTimeStamp = now + timeToGo;
          job->JobCompleteTimeStamp = job->LastUpdateAt + timeToGo;
 
          // When is the time to send the next cookie?
          const double calculationsSinceLastCookie = job->Completed - job->LastCookieCompleted;
-         const double timeSinceLastCookie         = now - job->LastCookieAt;
+         const double timeSinceLastCookie         = (double)now - (double)job->LastCookieAt;
          const unsigned long long nextByCalculations =
             (unsigned long long)rint(max(0.0, CookieMaxCalculations - calculationsSinceLastCookie) / capacityPerJob);
          const unsigned long long nextByTime =
@@ -268,9 +285,11 @@ void CalcAppServer::scheduleNextTimerEvent()
 
 
 // ###### Initialize ########################################################
-EventHandlingResult CalcAppServer::initializeService(int sd)
+EventHandlingResult CalcAppServer::initializeService(int sd, const uint32_t peIdentifier)
 {
-   VectorFH = fopen(VectorFileName.c_str(), "w");
+   Identifier = peIdentifier;
+
+   VectorFH = fopen(VectorFileName.c_str(), (Stats->VectorLine > 0) ? "a" : "w");
    if(VectorFH == NULL) {
       cout << " Unable to open output file " << VectorFileName << endl;
       return(EHR_Abort);
@@ -295,11 +314,11 @@ void CalcAppServer::finishService(int sd, EventHandlingResult initializeResult)
 
    if(ScalarFH) {
       unsigned long long       shutdownTimeStamp     = getMicroTime();
-      const unsigned long long serviceUptime         = shutdownTimeStamp - StartupTimeStamp;
+      const unsigned long long serviceUptime         = shutdownTimeStamp - Stats->StartupTimeStamp;
       const double             availableCalculations = serviceUptime * Capacity / 1000000.0;
-      const double             utilization           = TotalUsedCalculations / availableCalculations;
+      const double             utilization           = Stats->TotalUsedCalculations / availableCalculations;
 
-      const double             totalUsedCapacity     = TotalUsedCalculations;
+      const double             totalUsedCapacity     = Stats->TotalUsedCalculations;
       const double             totalWastedCapacity   = serviceUptime - totalUsedCapacity;
 
       fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPE Service Uptime\"        %1.6f\n", ObjectName.c_str(), serviceUptime / 1000000.0);
@@ -309,8 +328,8 @@ void CalcAppServer::finishService(int sd, EventHandlingResult initializeResult)
       fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPE Total Used Capacity\"   %1.0f\n", ObjectName.c_str(), totalUsedCapacity);
       fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPE Total Wasted Capacity\" %1.0f\n", ObjectName.c_str(), totalWastedCapacity);
 
-      fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPE Total Jobs Accepted\"   %llu\n",  ObjectName.c_str(), TotalJobsAccepted);
-      fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPE Total Jobs Rejected\"   %llu\n",  ObjectName.c_str(), TotalJobsRejected);
+      fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPE Total Jobs Accepted\"   %llu\n",  ObjectName.c_str(), Stats->TotalJobsAccepted);
+      fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPE Total Jobs Rejected\"   %llu\n",  ObjectName.c_str(), Stats->TotalJobsRejected);
 
       fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPE Utilization\"           %1.6f\n", ObjectName.c_str(), utilization);
 
@@ -375,7 +394,7 @@ EventHandlingResult CalcAppServer::handleCookieEcho(rserpool_session_t sessionID
 // ###### Send CalcAppAccept ################################################
 void CalcAppServer::sendCalcAppAccept(CalcAppServer::CalcAppServerJob* job)
 {
-   TotalJobsAccepted++;
+   Stats->TotalJobsAccepted++;
    struct CalcAppMessage message;
    memset(&message, 0, sizeof(message));
    message.Type   = CALCAPP_ACCEPT;
@@ -394,7 +413,7 @@ void CalcAppServer::sendCalcAppAccept(CalcAppServer::CalcAppServerJob* job)
 // ###### Send CalcAppReject ################################################
 void CalcAppServer::sendCalcAppReject(rserpool_session_t sessionID, uint32_t jobID)
 {
-   TotalJobsRejected++;
+   Stats->TotalJobsRejected++;
    struct CalcAppMessage message;
    memset(&message, 0, sizeof(message));
    message.Type   = CALCAPP_REJECT;
@@ -438,20 +457,21 @@ void CalcAppServer::sendCalcAppComplete(CalcAppServer::CalcAppServerJob* job)
    }
 
    unsigned long long       shutdownTimeStamp     = getMicroTime();
-   const unsigned long long serviceUptime         = shutdownTimeStamp - StartupTimeStamp;
+   const unsigned long long serviceUptime         = shutdownTimeStamp - Stats->StartupTimeStamp;
    const double             availableCalculations = serviceUptime * Capacity / 1000000.0;
-   const double             utilization           = TotalUsedCalculations / availableCalculations;
+   const double             utilization           = Stats->TotalUsedCalculations / availableCalculations;
    if(VectorFH) {
-      if(VectorLine == 0) {
-         fprintf(VectorFH, "ObjectName PID CurrentTimeStamp Runtime   AvailableCalculations UsedCalculations Utilization   CurrentJobs MaxJobs\n");
+      if(Stats->VectorLine == 0) {
+         fprintf(VectorFH, "ObjectName PID PoolElementIdentifier CurrentTimeStamp Runtime   AvailableCalculations UsedCalculations Utilization   CurrentJobs MaxJobs\n");
       }
-      fprintf(VectorFH,"%06u %s %u %1.6f %1.6f   %1.0f %1.0f %1.6f   %u %u\n",
-            ++VectorLine,
+      fprintf(VectorFH,"%06llu %s %u 0x%08x %1.6f   %1.6f   %1.0f %1.0f %1.6f   %u %u\n",
+            ++Stats->VectorLine,
             ObjectName.c_str(),
             (unsigned int)getpid(),
+            Identifier,
             getMicroTime() / 1000000.0,
             serviceUptime / 1000000.0,
-            availableCalculations, TotalUsedCalculations, utilization,
+            availableCalculations, Stats->TotalUsedCalculations, utilization,
             (unsigned int)Jobs, (unsigned int)MaxJobs);
       // fflush(VectorFH);
    }
