@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: calcappservice.h 2076 2008-12-28 15:52:34Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: calcappservice.h 1391 2007-05-23 07:27:45Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef CALCAPPSERVICE_H
@@ -35,6 +39,15 @@
 
 class CalcAppServer : public UDPLikeServer
 {
+   public:
+   struct CalcAppServerStatistics {
+      unsigned long long StartupTimeStamp;
+      double             TotalUsedCalculations;
+      unsigned long long TotalJobsAccepted;
+      unsigned long long TotalJobsRejected;
+      unsigned long long VectorLine;
+   };
+
    public:
    CalcAppServer(const size_t             maxJobs,
                  const char*              objectName,
@@ -44,11 +57,14 @@ class CalcAppServer : public UDPLikeServer
                  const unsigned long long keepAliveTransmissionInterval,
                  const unsigned long long keepAliveTimeoutInterval,
                  const unsigned long long cookieMaxTime,
-                 const double             cookieMaxCalculations);
+                 const double             cookieMaxCalculations,
+                 const double             cleanShutdownProbability,
+                 CalcAppServerStatistics* statistics,
+                 const bool               resetStatistics);
    virtual ~CalcAppServer();
 
    protected:
-   virtual EventHandlingResult initializeService(int sd);
+   virtual EventHandlingResult initializeService(int sd, const uint32_t peIdentifier);
    virtual void finishService(int sd, EventHandlingResult result);
    virtual void printParameters();
    virtual EventHandlingResult handleMessage(rserpool_session_t sessionID,
@@ -114,27 +130,23 @@ class CalcAppServer : public UDPLikeServer
    void handleCookieTransmissionTimer(CalcAppServerJob* job);
 
 
-   std::string        ObjectName;
-   std::string        VectorFileName;
-   std::string        ScalarFileName;
-   FILE*              VectorFH;
-   FILE*              ScalarFH;
-   unsigned int       VectorLine;
+   std::string              ObjectName;
+   std::string              VectorFileName;
+   std::string              ScalarFileName;
+   FILE*                    VectorFH;
+   FILE*                    ScalarFH;
+   CalcAppServerJob*        FirstJob;
+   CalcAppServerStatistics* Stats;
 
-   size_t             MaxJobs;
-   double             Capacity;
-   unsigned long long KeepAliveTimeoutInterval;
-   unsigned long long KeepAliveTransmissionInterval;
-   unsigned long long CookieMaxTime;
-   double             CookieMaxCalculations;
-
-   double             TotalUsedCalculations;
-   unsigned long long TotalJobsAccepted;
-   unsigned long long TotalJobsRejected;
-
-   unsigned long long StartupTimeStamp;
-   size_t             Jobs;
-   CalcAppServerJob*  FirstJob;
+   size_t                   MaxJobs;
+   double                   Capacity;
+   unsigned long long       KeepAliveTimeoutInterval;
+   unsigned long long       KeepAliveTransmissionInterval;
+   unsigned long long       CookieMaxTime;
+   double                   CookieMaxCalculations;
+   double                   CleanShutdownProbability;
+   size_t                   Jobs;
+   uint32_t                 Identifier;
 };
 
 #endif
