@@ -1,4 +1,4 @@
-/* $Id: componentstatuspackets.h 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: componentstatuspackets.h 2112 2009-03-14 12:01:35Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -30,24 +30,14 @@
 #ifndef COMPONENTSTATUSPACKETS_H
 #define COMPONENTSTATUSPACKETS_H
 
+#include "rserpool-csp.h"
 #include "tdtypes.h"
 
 
-#define CSP_VERSION        0x0200
+#define CSP_VERSION 0x0200
+#define CSPT_REPORT   0x01
 
 
-#define CID_GROUP(id)  (((uint64_t)id >> 56) & (0xffffULL))
-#define CID_OBJECT(id) ((uint64_t)id & 0xffffffffffffffULL)
-
-#define CID_GROUP_REGISTRAR   0x0001
-#define CID_GROUP_POOLELEMENT 0x0002
-#define CID_GROUP_POOLUSER    0x0003
-
-#define CID_COMPOUND(group, object)  ((((uint64_t)(group & 0xffff)) << 56) | CID_OBJECT((uint64_t)object))
-
-
-#define CSPT_REPORT           0x01
-
 struct ComponentStatusCommonHeader
 {
    uint8_t  Type;
