@@ -1,4 +1,4 @@
-/* $Id: componentstatusreporter.h 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: componentstatusreporter.h 2172 2009-07-31 21:52:24Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -60,6 +60,7 @@ struct CSPReporter
    union sockaddr_union CSPReportAddress;
    unsigned int         CSPReportInterval;
    struct Timer         CSPReportTimer;
+   const char*          StatusTextOverride;
 
    size_t               (*CSPGetReportFunction)(
                            void*                         userData,
@@ -87,6 +88,7 @@ void cspReporterNew(struct CSPReporter*    cspReporter,
                     void*                  cspGetReportFunctionUserData);
 void cspReporterDelete(struct CSPReporter* cspReporter);
 
+void cspReporterHandleTimerDuringRegistrarSearch(struct CSPReporter* cspReporter);
 
 #ifdef __cplusplus
 }
