@@ -1,4 +1,4 @@
-/* $Id: exectime.cc 1726 2008-02-14 09:56:55Z dreibh $
+/* $Id: exectime.cc 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,9 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <unistd.h>
+#include <string.h>
 #include <errno.h>
 #include <thread.h>
 #include <iostream>
