@@ -1,4 +1,4 @@
-/* $Id: fractalgeneratorservice.h 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: fractalgeneratorservice.h 2154 2009-07-11 12:39:46Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -80,7 +80,7 @@ class FractalGeneratorServer : public TCPLikeServer
 
    struct FGPData                 Data;
    size_t                         DataPackets;
-   int                            Alert;
+   bool                           Alert;
 
    FractalGeneratorStatus         Status;
    FractalGeneratorServerSettings Settings;
