@@ -1,4 +1,4 @@
-/* $Id: fractalpooluser.cc 2124 2009-05-20 11:30:26Z dreibh $
+/* $Id: fractalpooluser.cc 2272 2009-11-15 11:36:26Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -40,11 +40,14 @@
 #include <QStringList>
 #include <QPaintEvent>
 #include <QDomDocument>
-#include <QMutex>
+#include <QTranslator>
+#include <QLocale>
 #include <iostream>
 #include <complex>
 
 #include "fractalpooluser.h"
+#include "fractalpooluser_moc.cc"
+
 
 #ifdef FRACTALPOOLUSER_USE_KDE
 #include <kcmdlineargs.h>
@@ -62,8 +65,6 @@
 #include "loglevel.h"
 #include "debug.h"
 
-#include "fractalpooluser_moc.cc"
-
 
 
 #define DEFAULT_FPU_WIDTH             400
@@ -73,13 +74,14 @@
 #define DEFAULT_FPU_INTER_IMAGE_TIME    5
 #define DEFAULT_FPU_THREADS             1
 
-#define DEFAULT_FPU_CAPTION     NULL
-#define DEFAULT_FPU_POOLHANDLE  "FractalGeneratorPool"
+#define DEFAULT_FPU_CAPTION          NULL
+#define DEFAULT_FPU_POOLHANDLE       "FractalGeneratorPool"
 #ifndef DEFAULT_FPU_CONFIGDIR
-#define DEFAULT_FPU_CONFIGDIR   "fgpconfig"
+#define DEFAULT_FPU_CONFIGDIR        "fgpconfig"
 #endif
-#define DEFAULT_FPU_IMAGEPREFIX ""
-#define DEFAULT_FPU_COLORMARKS  TRUE
+#define DEFAULT_FPU_IMAGEPREFIX      ""
+#define DEFAULT_FPU_SHOWCOLORMARKS   TRUE
+#define DEFAULT_FPU_SHOWSESSIONS     TRUE
 
 
 
@@ -92,7 +94,8 @@ FractalPU::FractalPU(const size_t       width,
                      const unsigned int recvTimeout,
                      const unsigned int interImageTime,
                      const char*        imageStoragePrefix,
-                     const bool         colorMarks,
+                     const bool         showFailoverMarks,
+                     const bool         showSessions,
                      const size_t       threads,
                      QWidget*           parent)
 #ifdef FRACTALPOOLUSER_USE_KDE
@@ -112,7 +115,8 @@ FractalPU::FractalPU(const size_t       width,
    SendTimeout        = recvTimeout;
    InterImageTime     = interImageTime;
    ImageStoragePrefix = QString(imageStoragePrefix);
-   ColorMarks         = colorMarks;
+   ShowFailoverMarks     = showFailoverMarks;
+   ShowSessions       = showSessions;
    ConfiguredThreads  = threads;
    FileNumber         = 0;
    Run                = 0;
@@ -133,7 +137,7 @@ FractalPU::FractalPU(const size_t       width,
 
    // ====== Initialize widget and PU thread ================================
    setWindowTitle("Fractal Pool User");
-   statusBar()->showMessage("Welcome to Fractal PU!", 3000);
+   statusBar()->showMessage(tr("Welcome to FractalPoolUser!"), 3000);
    show();
    QTimer::singleShot(100, this, SLOT(startNextJob()));
 }
@@ -142,7 +146,7 @@ FractalPU::FractalPU(const size_t       width,
 /* ###### Destructor ##################################################### */
 FractalPU::~FractalPU()
 {
-   Status = FPU_Shutdown;
+   setStatus(FPU_Shutdown);
    if(CalculationThreadArray) {
       for(size_t i = 0;i < CurrentThreads;i++) {
          CalculationThreadArray[i]->wait();
@@ -168,7 +172,7 @@ void FractalPU::closeEvent(QCloseEvent* closeEvent)
 /* ###### Handle Resize event ############################################ */
 void FractalPU::resizeEvent(QResizeEvent* resizeEvent)
 {
-   Status = FPU_CalcAborted;
+   setStatus(FPU_CalcAborted);
 }
 
 
@@ -242,7 +246,7 @@ void FractalPU::getNextParameters()
 /* ###### Start next fractal PU job ###################################### */
 void FractalPU::startNextJob()
 {
-   CHECK(Status != FPU_CalcInProgress);
+   CHECK(getStatus() != FPU_CalcInProgress);
 
    // ====== Initialize image object and timeout timer ======================
    Run++;
@@ -260,11 +264,11 @@ void FractalPU::startNextJob()
 
 
    // ====== Start job distribution =========================================
-   Status = FPU_CalcInProgress;
+   setStatus(FPU_CalcInProgress);
 
    // std::cout << "Starting job distribution ..." << std::endl;
    Display->setCursor(Qt::WaitCursor);
-   statusBar()->showMessage("Starting job distribution ...");
+   statusBar()->showMessage(tr("Starting job distribution ..."));
 
    CurrentThreads = ConfiguredThreads;
    CalculationThreadArray = new FractalCalculationThread*[CurrentThreads];
@@ -275,6 +279,7 @@ void FractalPU::startNextJob()
 
 
    // ====== Initialize configured number of sessions =======================
+   Display->lock();
    QColor       color;
    const size_t yCount = (size_t)floor(sqrt((double)CurrentThreads));
    const size_t yStep  = (size_t)rint((double)Parameter.Height / yCount);
@@ -290,12 +295,12 @@ void FractalPU::startNextJob()
          // ====== Initialize rectange for next session =====================
          const QPen numberPen(Qt::red);
          QPainter   painter;
-         painter.begin(Display->Image);
+         painter.begin(Display->getImage());
          painter.setPen(numberPen);
          for(size_t xPosition = 0;xPosition < xCount;xPosition++) {
             // ====== Mark rectange for next session ========================
             if(CurrentThreads > 1) {
-               if(ColorMarks) {
+               if(ShowFailoverMarks) {
                   color.setHsv((((5 * number) % 72) * 5) % 256, 100, 255);
                   Display->fillRect(xPosition * xStep, yPosition * yStep,
                                     xStep, yStep, color.rgb());
@@ -303,7 +308,8 @@ void FractalPU::startNextJob()
                char str[64];
                snprintf((char*)&str, sizeof(str), "%u", (unsigned int)number + 1);
                const QString numberString(str);
-               painter.drawText(xPosition * xStep, yPosition * yStep, xStep, yStep, Qt::AlignCenter, numberString);
+               painter.drawText(xPosition * xStep, yPosition * yStep, xStep, yStep,
+                                Qt::AlignCenter, numberString);
             }
 
             // ====== Start new session =====================================
@@ -330,6 +336,7 @@ void FractalPU::startNextJob()
 
    // ====== Make everything visible ========================================
    Display->update();
+   Display->unlock();
 
    // ====== Wait for job completion ========================================
    // std::cout << "Waiting for job completion ..." << std::endl;
@@ -346,7 +353,8 @@ void FractalPU::handleCompletedSession()
       // ====== Count active sessions =======================================
       size_t active = 0;
       for(size_t i = 0;i < CurrentThreads;i++) {
-         if( (CalculationThreadArray[i] != NULL) && (!CalculationThreadArray[i]->isFinished()) ) {
+         if( (CalculationThreadArray[i] != NULL) &&
+             (!CalculationThreadArray[i]->isFinished()) ) {
             active++;
          }
       }
@@ -368,13 +376,13 @@ void FractalPU::handleCompletedSession()
          CalculationThreadArray = NULL;
 
          // ====== Pause before next image ==================================
-         if(Status == FPU_CalcInProgress) {
-            Status = FPU_Completed;
+         if(getStatus() == FPU_CalcInProgress) {
+            setStatus(FPU_Completed);
             CountDown = InterImageTime;
             QTimer::singleShot(0, this, SLOT(countDown()));
          }
-         else if(Status == FPU_CalcAborted) {
-            Status = FPU_Completed;
+         else if(getStatus() == FPU_CalcAborted) {
+            setStatus(FPU_Completed);
             statusBar()->showMessage("Restarting ...");
             QTimer::singleShot(500, this, SLOT(startNextJob()));
          }
@@ -386,13 +394,10 @@ void FractalPU::handleCompletedSession()
 /* ###### Wait until start of next job ################################### */
 void FractalPU::countDown()
 {
-   char str[128];
-
-   snprintf((char*)&str, sizeof(str),
-            "%s - Waiting for %u seconds ...",
-            (Success == true) ? "Image completed" : "Image calculation failed",
-            CountDown);
-   statusBar()->showMessage(str);
+   const QString status =
+      ((Success == true) ? tr("Image completed") : tr("Image calculation failed")) +
+      " - " + tr("Waiting for ") + QString::number(CountDown) + tr(" seconds ...");
+   statusBar()->showMessage(status);
    if(CountDown > 0) {
       CountDown--;
       QTimer::singleShot(1000, this, SLOT(countDown()));
@@ -425,25 +430,29 @@ void FractalPU::contextMenuEvent(QContextMenuEvent* event)
 
    static unsigned int sessionsEntries[] = {1,
                                             0,
-                                            2, 3, 4, 6, 8, 10,
+                                            2, 3, 4, 6, 8,
                                             0,
                                             9, 16, 25, 36, 49, 64, 81, 100,
                                             0,
                                             144, 225, 256, 400};
 
-   QAction* showFailovers = menu.addAction("Show Failovers");
+   QAction* showFailovers = menu.addAction(tr("Show &Failovers"));
    Q_CHECK_PTR(showFailovers);
    showFailovers->setCheckable(true);
-   showFailovers->setChecked(ColorMarks);
-   connect(showFailovers, SIGNAL(toggled(bool)), this, SLOT(changeColorMarks(bool)));
+   showFailovers->setChecked(ShowFailoverMarks);
+   connect(showFailovers, SIGNAL(toggled(bool)), this, SLOT(changeShowFailoverMarks(bool)));
 
-   QMenu* sessionsMenu = menu.addMenu("Sessions");
+   QAction* showSessions = menu.addAction(tr("Show &Sessions"));
+   Q_CHECK_PTR(showSessions);
+   showSessions->setCheckable(true);
+   showSessions->setChecked(ShowSessions);
+   connect(showSessions, SIGNAL(toggled(bool)), this, SLOT(changeShowSessions(bool)));
+   
+   QMenu* sessionsMenu = menu.addMenu(tr("Sessions"));
    Q_CHECK_PTR(sessionsMenu);
    for(size_t i = 0;i < sizeof(sessionsEntries) / sizeof(sessionsEntries[0]);i++) {
       if(sessionsEntries[i] > 0) {
-         char str[64];
-         snprintf((char*)&str, sizeof(str), "%u", sessionsEntries[i]);
-         QAction* action = sessionsMenu->addAction(str);
+         QAction* action = sessionsMenu->addAction(QString::number(sessionsEntries[i]));
          Q_CHECK_PTR(action);
          action->setCheckable(true);
          if(sessionsEntries[i] == ConfiguredThreads) {
@@ -458,18 +467,25 @@ void FractalPU::contextMenuEvent(QContextMenuEvent* event)
    connect(sessionsMenu, SIGNAL(triggered(QAction*)), this, SLOT(changeThreads(QAction*)));
 
    menu.addSeparator();
-   menu.addAction("About", this, SLOT(about()));
+   menu.addAction(tr("&About"), this, SLOT(about()));
    menu.addSeparator();
-   menu.addAction("Quit", this, SLOT(quit()));
+   menu.addAction(tr("&Quit"), this, SLOT(quit()));
 
    menu.exec(event->globalPos());
 }
 
 
-/* ###### Change ColorMarks setting ###################################### */
-void FractalPU::changeColorMarks(bool checked)
+/* ###### Change FailoverMarks setting ################################### */
+void FractalPU::changeShowFailoverMarks(bool checked)
 {
-   ColorMarks = checked;
+   ShowFailoverMarks = checked;
+}
+
+
+/* ###### Change FailoverMarks setting ################################### */
+void FractalPU::changeShowSessions(bool checked)
+{
+   ShowSessions = checked;
 }
 
 
@@ -479,7 +495,7 @@ void FractalPU::changeThreads(QAction* action)
    const unsigned int newThreads = action->data().toInt();
    if(newThreads >= 1) {
       ConfiguredThreads = newThreads;
-      Status = FPU_CalcAborted;
+      setStatus(FPU_CalcAborted);
    }
 }
 
@@ -487,15 +503,18 @@ void FractalPU::changeThreads(QAction* action)
 /* ###### About ########################################################## */
 void FractalPU::about()
 {
-   QMessageBox::about(this, "About Fractal Pool User",
-      "<center><b>Fractal Pool User</b><br>Copyright (C) 2002-2009 by Thomas Dreibholz</center>");
+   QMessageBox::about(this, tr("About Fractal Pool User"),
+      "<center>"
+         "<b>Fractal Pool User</b><br>"
+         "Copyright (C) 2002-2009 by Thomas Dreibholz"
+      "</center>");
 }
 
 
 /* ###### Quit ########################################################### */
 void FractalPU::quit()
 {
-   Status = FPU_Shutdown;
+   setStatus(FPU_Shutdown);
    qApp->exit(0);
 }
 
@@ -529,33 +548,34 @@ bool FractalCalculationThread::sendParameterMessage()
    FGPParameter parameter;
    ssize_t      sent;
 
-   const double c1real = Master->Parameter.C1Real + (double)ViewX * (Master->Parameter.C2Real - Master->Parameter.C1Real) / (double)Master->Parameter.Width;
-   const double c1imag = Master->Parameter.C1Imag + (double)ViewY * (Master->Parameter.C2Imag - Master->Parameter.C1Imag) / (double)Master->Parameter.Height;
-   const double c2real = Master->Parameter.C1Real + (double)(ViewX + ViewWidth) * (Master->Parameter.C2Real - Master->Parameter.C1Real) / (double)Master->Parameter.Width;
-   const double c2imag = Master->Parameter.C1Imag + (double)(ViewY + ViewHeight) * (Master->Parameter.C2Imag - Master->Parameter.C1Imag) / (double)Master->Parameter.Height;
+   const double c1real = Master->getParameters().C1Real + (double)ViewX * (Master->getParameters().C2Real - Master->getParameters().C1Real) / (double)Master->getParameters().Width;
+   const double c1imag = Master->getParameters().C1Imag + (double)ViewY * (Master->getParameters().C2Imag - Master->getParameters().C1Imag) / (double)Master->getParameters().Height;
+   const double c2real = Master->getParameters().C1Real + (double)(ViewX + ViewWidth) * (Master->getParameters().C2Real - Master->getParameters().C1Real) / (double)Master->getParameters().Width;
+   const double c2imag = Master->getParameters().C1Imag + (double)(ViewY + ViewHeight) * (Master->getParameters().C2Imag - Master->getParameters().C1Imag) / (double)Master->getParameters().Height;
 
    parameter.Header.Type   = FGPT_PARAMETER;
    parameter.Header.Flags  = 0x00;
    parameter.Header.Length = htons(sizeof(parameter));
    parameter.Width         = htonl(ViewWidth);
    parameter.Height        = htonl(ViewHeight);
-   parameter.AlgorithmID   = htonl(Master->Parameter.AlgorithmID);
-   parameter.MaxIterations = htonl(Master->Parameter.MaxIterations);
+   parameter.AlgorithmID   = htonl(Master->getParameters().AlgorithmID);
+   parameter.MaxIterations = htonl(Master->getParameters().MaxIterations);
    parameter.C1Real        = doubleToNetwork(c1real);
    parameter.C1Imag        = doubleToNetwork(c1imag);
    parameter.C2Real        = doubleToNetwork(c2real);
    parameter.C2Imag        = doubleToNetwork(c2imag);
-   parameter.N             = doubleToNetwork(Master->Parameter.N);
+   parameter.N             = doubleToNetwork(Master->getParameters().N);
 
    sent = rsp_sendmsg(Session, (char*)&parameter, sizeof(parameter), 0,
-                      0, htonl(PPID_FGP), 0, 0, 0, Master->SendTimeout);
+                      0, htonl(PPID_FGP), 0, 0, 0, Master->getSendTimeout());
    return(sent > 0);
 }
 
 
 /* ###### Handle Data message ############################################ */
-FractalCalculationThread::DataStatus FractalCalculationThread::handleDataMessage(const FGPData* data,
-                                                                                 const size_t   size)
+FractalCalculationThread::DataStatus FractalCalculationThread::handleDataMessage(
+                                                                  const FGPData* data,
+                                                                  const size_t   size)
 {
    if(size < getFGPDataSize(0)) {
       return(Invalid);
@@ -580,7 +600,7 @@ FractalCalculationThread::DataStatus FractalCalculationThread::handleDataMessage
       return(Invalid);
    }
 
-   Master->Display->lock();
+   Master->getDisplay()->lock();
    QColor color;
    while(y < ViewHeight) {
       while(x < ViewWidth) {
@@ -588,14 +608,15 @@ FractalCalculationThread::DataStatus FractalCalculationThread::handleDataMessage
             goto finished;
          }
          uint32_t point = ntohl(data->Buffer[p]);
-         if(Master->ColorMarks) {
-            point = ((point + (2 * Master->Run) + (3 * ThreadID) + (5 * PoolElementUsages) % 72) * 5) % 256;
+         if(Master->getShowFailoverMarks()) {
+            point += (2 * Master->getRun()) + (5 * PoolElementUsages);
          }
-         else {
-            point = ((point % 72) * 5 % 256);
+         if(Master->getShowSessions()) {
+            point += (3 * ThreadID);
          }
+         point = ((point % 72) * 5) % 360;
          color.setHsv(point, 255, 255);
-         Master->Display->setPixel(x + ViewX, y + ViewY, color.rgb());
+         Master->getDisplay()->setPixel(x + ViewX, y + ViewY, color.rgb());
          p++;
 
          x++;
@@ -605,7 +626,7 @@ FractalCalculationThread::DataStatus FractalCalculationThread::handleDataMessage
    }
 
 finished:
-   Master->Display->unlock();
+   Master->getDisplay()->unlock();
    emit updateImage(ntohl(data->StartY) + ViewY, y + ViewY);
    return(Okay);
 }
@@ -614,20 +635,19 @@ finished:
 /* ###### Fractal PU thread implementation ############################### */
 void FractalCalculationThread::run()
 {
-   char statusText[128];
-
    PoolElementUsages = 0;
 
    Session = rsp_socket(0, SOCK_SEQPACKET, IPPROTO_SCTP);
    if(Session >= 0) {
-      if(rsp_connect(Session, Master->PoolHandle, Master->PoolHandleSize, 0) == 0) {
+      if(rsp_connect(Session, Master->getPoolHandle(), Master->getPoolHandleSize(), 0) == 0) {
 
          // ====== Begin image calculation ==================================
          do {
             if(ShowStatus) {
-               rsp_csp_setstatus(Session, 0, "Sending parameter command ...");
+               const QString status = tr("Requesting calculation ...");
+               rsp_csp_setstatus(Session, 0, status.toLocal8Bit().data());
                // cout << "Sending parameter command ..." << endl;
-               emit updateStatus("Sending parameter command ...");
+               emit updateStatus(status);
             }
 
             if(sendParameterMessage()) {
@@ -641,7 +661,7 @@ void FractalCalculationThread::run()
                   ssize_t received;
 
                   received = rsp_recvfullmsg(Session, (char*)&data, sizeof(data),
-                                             &rinfo, &flags, Master->RecvTimeout);
+                                             &rinfo, &flags, Master->getRecvTimeout());
                   while(received > 0) {
                      // ====== Handle notification ==========================
                      if(flags & MSG_RSERPOOL_NOTIFICATION) {
@@ -678,13 +698,14 @@ void FractalCalculationThread::run()
                               default:
                                  packets++;
                                  if(ShowStatus) {
+                                    char statusText[128];
                                     snprintf((char*)&statusText, sizeof(statusText),
-                                             "Processed packet #%03u; PE is $%08x",
-                                             (unsigned int)packets, rinfo.rinfo_pe_id);
+                                             "%s%u %s $%08x",
+                                             tr("Processed packet #").toLocal8Bit().data(),
+                                             (unsigned int)packets,
+                                             tr("of PE").toLocal8Bit().data(),
+                                             rinfo.rinfo_pe_id);
                                     rsp_csp_setstatus(Session, 0, statusText);
-                                    snprintf((char*)&statusText, sizeof(statusText),
-                                             "Processed packet #%03u from PE $%08x ...",
-                                             (unsigned int)packets, rinfo.rinfo_pe_id);
                                     emit updateStatus(QString(statusText));
                                  }
                               break;
@@ -692,17 +713,18 @@ void FractalCalculationThread::run()
                         }
                      }
 
-                     if(Master->Status != FractalPU::FPU_CalcInProgress) {
+                     if(Master->getStatus() != FractalPU::FPU_CalcInProgress) {
                         goto finish;
                      }
 
                      flags = 0;
                      received = rsp_recvfullmsg(Session, (char*)&data, sizeof(data),
-                                                &rinfo, &flags, Master->RecvTimeout);
+                                                &rinfo, &flags, Master->getRecvTimeout());
                   }
                   if(Success == false) {
                      printTimeStamp(stdout);
-                     printf("FAILOVER (cookie=%s)...\n", (rsp_has_cookie(Session)) ? "yes" : "NO!");
+                     printf("FAILOVER (cookie=%s)...\n",
+                            (rsp_has_cookie(Session)) ? "yes" : "NO!");
                      rsp_forcefailover(Session, FFF_NONE, 0);
                   }
 
@@ -710,15 +732,17 @@ void FractalCalculationThread::run()
             }
             else {
                printTimeStamp(stdout);
-               printf("FAILOVER AFTER FAILED sendParameterMessage() (cookie=%s)...\n", (rsp_has_cookie(Session)) ? "yes" : "NO!");
+               printf("FAILOVER AFTER FAILED sendParameterMessage() (cookie=%s)...\n",
+                      (rsp_has_cookie(Session)) ? "yes" : "NO!");
                rsp_forcefailover(Session, FFF_UNREACHABLE, 0);
             }
          } while(Success == false);
 
          // ====== Image calculation completed ==============================
          if(ShowStatus) {
-            const char* statusText = (Success == true) ? "Image completed!" : "Image calculation failed!";
-            rsp_csp_setstatus(Session, 0, statusText);
+            const QString status = (Success == true) ? tr("Image completed!") :
+                                                       tr("Image calculation failed!");
+            rsp_csp_setstatus(Session, 0, status.toLocal8Bit().data());
          }
       }
       else {
@@ -806,7 +830,8 @@ int main(int argc, char** argv)
    unsigned int    recvTimeout        = DEFAULT_FPU_RECV_TIMEOUT;
    unsigned int    interImageTime     = DEFAULT_FPU_INTER_IMAGE_TIME;
    const char*     imageStoragePrefix = DEFAULT_FPU_IMAGEPREFIX;
-   bool            colorMarks         = DEFAULT_FPU_COLORMARKS;
+   bool            showFailoverMarks  = DEFAULT_FPU_SHOWCOLORMARKS;
+   bool            showSessions       = DEFAULT_FPU_SHOWSESSIONS;
    unsigned int    identifier;
    int             i;
 
@@ -859,11 +884,17 @@ int main(int argc, char** argv)
       else if(!(strncmp(argv[i], "-imagestorageprefix=" ,20))) {
          imageStoragePrefix = (const char*)&argv[i][20];
       }
+      else if(!(strcmp(argv[i], "-sessions"))) {
+         showSessions = TRUE;
+      }
+      else if(!(strcmp(argv[i], "-nosessions"))) {
+         showSessions = TRUE;
+      }
       else if(!(strcmp(argv[i], "-colormarks"))) {
-         colorMarks = TRUE;
+         showFailoverMarks = TRUE;
       }
       else if(!(strcmp(argv[i], "-nocolormarks"))) {
-         colorMarks = FALSE;
+         showFailoverMarks = FALSE;
       }
       else {
          printf("ERROR: Bad argument %s\n", argv[i]);
@@ -905,7 +936,8 @@ int main(int argc, char** argv)
    printf("Receive Timeout      = %u [ms]\n", recvTimeout);
    printf("Inter Image Time     = %u [s]\n", interImageTime);
    printf("Image Storage Prefix = %s\n", imageStoragePrefix);
-   printf("Color Marks          = %s\n", (colorMarks == TRUE) ? "on" : "off");
+   printf("Show Color Marks     = %s\n", (showFailoverMarks == TRUE) ? "on" : "off");
+   printf("Show Sessions        = %s\n", (showSessions == TRUE) ? "on" : "off");
    printf("Threads              = %u\n\n", (unsigned int)threads);
 
 
@@ -926,10 +958,15 @@ int main(int argc, char** argv)
    KApplication application;
 #else
    QApplication application(argc, argv);
+   QTranslator applicationTranslator;
+   applicationTranslator.load("fractalpooluser_" + QLocale::system().name(),
+                              "/usr/share/fractalpooluser");
+   application.installTranslator(&applicationTranslator);
 #endif
    FractalPU* fractalPU = new FractalPU(width, height, poolHandle, configDirName,
                                         sendTimeout, recvTimeout, interImageTime,
-                                        imageStoragePrefix, colorMarks, threads);
+                                        imageStoragePrefix,
+                                        showFailoverMarks, showSessions, threads);
    Q_CHECK_PTR(fractalPU);
    if(caption) {
       fractalPU->setWindowTitle(caption);
