@@ -1,4 +1,4 @@
-/* $Id: fractalpooluser.h 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: fractalpooluser.h 2178 2009-08-04 07:29:58Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -55,6 +55,7 @@ class ImageDisplay : public QWidget
 {
    Q_OBJECT
 
+   // ====== Public methods =================================================
    public:
    ImageDisplay(QWidget* parent = NULL);
    ~ImageDisplay();
@@ -88,15 +89,21 @@ class ImageDisplay : public QWidget
    inline void lock() {
       ImageMutex.lock();
    }
-
    inline void unlock() {
       ImageMutex.unlock();
    }
+   inline QImage* getImage() {
+      return(Image);
+   }
 
+
+   // ====== Protected methods ==============================================
    protected:
    void paintEvent(QPaintEvent* paintEvent);
 
-   public:
+   
+   // ====== Private data ===================================================
+   private:
    QImage* Image;
    QMutex  ImageMutex;
 };
@@ -106,6 +113,7 @@ class FractalCalculationThread : public QThread
 {
    Q_OBJECT
 
+   // ====== Public methods =================================================
    public:
    FractalCalculationThread(FractalPU*         fractalPU,
                             const unsigned int threadID,
@@ -120,10 +128,12 @@ class FractalCalculationThread : public QThread
       return(Success);
    }
 
+   // ====== Signals ========================================================
    signals:
    void updateImage(int start, int end);
    void updateStatus(QString statusText);
 
+   // ====== Private methods and data =======================================
    private:
    enum DataStatus {
       Okay      = 0,
@@ -157,41 +167,14 @@ class FractalPU
 {
    Q_OBJECT
 
-   friend class FractalCalculationThread;
-
+   // ====== Definitions ====================================================
    public:
-   FractalPU(const size_t       width,
-             const size_t       height,
-             const char*        poolHandle,
-             const char*        configDirName,
-             const unsigned int sendTimeout,
-             const unsigned int recvTimeout,
-             const unsigned int interImageTime,
-             const char*        imageStoragePrefix,
-             const bool         colorMarks,
-             const size_t       threads,
-             QWidget*           parent = NULL);
-   ~FractalPU();
-
-
-   protected:
-   void closeEvent(QCloseEvent* closeEvent);
-   void resizeEvent(QResizeEvent* resizeEvent);
-   void contextMenuEvent(QContextMenuEvent* event);
-
-   public slots:
-   void countDown();
-   void startNextJob();
-   void handleCompletedSession();
-   void redrawImage(int start, int end);
-   void changeStatus(QString statusText);
-   void changeColorMarks(bool checked);
-   void changeThreads(QAction* action);
-   void about();
-   void quit();
-
-   private:
-   void getNextParameters();
+   enum FractalGeneratorStatus {
+      FPU_Shutdown       = 0,
+      FPU_CalcAborted    = 1,
+      FPU_CalcInProgress = 2,
+      FPU_Completed      = 3
+   };
 
    struct FractalParameter
    {
@@ -206,16 +189,92 @@ class FractalPU
       double       N;
    };
 
-   enum FractalGeneratorStatus {
-      FPU_Shutdown       = 0,
-      FPU_CalcAborted    = 1,
-      FPU_CalcInProgress = 2,
-      FPU_Completed      = 3
-   };
+   
+   // ====== Public methods =================================================
+   public:
+   FractalPU(const size_t       width,
+             const size_t       height,
+             const char*        poolHandle,
+             const char*        configDirName,
+             const unsigned int sendTimeout,
+             const unsigned int recvTimeout,
+             const unsigned int interImageTime,
+             const char*        imageStoragePrefix,
+             const bool         showFailoverMarks,
+             const bool         showSessions,
+             const size_t       threads,
+             QWidget*           parent = NULL);
+   ~FractalPU();
+
+   inline const FractalParameter& getParameters() const {
+      return(Parameter);
+   }
+   inline void setStatus(const FractalGeneratorStatus status) {
+      StatusMutex.lock();
+      Status = status;
+      StatusMutex.unlock();
+   }
+   inline const FractalGeneratorStatus getStatus() {
+      StatusMutex.lock();
+      const FractalGeneratorStatus status = Status;
+      StatusMutex.unlock();
+      return(status);
+   }
+   inline ImageDisplay* getDisplay() {
+      return(Display);
+   }
+   inline bool getShowFailoverMarks() const {
+      return(ShowFailoverMarks);
+   }
+   inline bool getShowSessions() const {
+      return(ShowSessions);
+   }
+   inline size_t getRun() const {
+      return(Run);
+   }
+   inline const unsigned char* getPoolHandle() const {
+      return(PoolHandle);
+   }
+   inline size_t getPoolHandleSize() const {
+      return(PoolHandleSize);
+   }
+   inline unsigned int getSendTimeout() const {
+      return(SendTimeout);
+   }
+   inline unsigned int getRecvTimeout() const {
+      return(RecvTimeout);
+   }
+   
+
+   // ====== Protected methods ==============================================
+   protected:
+   void closeEvent(QCloseEvent* closeEvent);
+   void resizeEvent(QResizeEvent* resizeEvent);
+   void contextMenuEvent(QContextMenuEvent* event);
+
+   
+   // ====== Slots ==========================================================
+   public slots:
+   void countDown();
+   void startNextJob();
+   void handleCompletedSession();
+   void redrawImage(int start, int end);
+   void changeStatus(QString statusText);
+   void changeShowFailoverMarks(bool checked);
+   void changeShowSessions(bool checked);
+   void changeThreads(QAction* action);
+   void about();
+   void quit();
+
+   
+   // ====== Private methods and data =======================================
+   private:
+   void getNextParameters();
 
    FractalCalculationThread** CalculationThreadArray;
    FractalParameter           Parameter;
    FractalGeneratorStatus     Status;
+   QMutex                     StatusMutex;
    ImageDisplay*              Display;
    size_t                     Run;
 
@@ -224,7 +283,8 @@ class FractalPU
    unsigned int               SendTimeout;
    unsigned int               RecvTimeout;
    unsigned int               InterImageTime;
-   bool                       ColorMarks;
+   bool                       ShowFailoverMarks;
+   bool                       ShowSessions;
    size_t                     ConfiguredThreads;
    size_t                     CurrentThreads;
 
