@@ -1,32 +1,38 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: gettimestamp.c 2076 2008-12-28 15:52:34Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: gettimestamp.c 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "tdtypes.h"
 #include "timeutilities.h"
 
+#include <stdlib.h>
 #include <time.h>
+#include <string.h>
 
 
 /* ###### Main program ################################################### */
@@ -53,14 +59,14 @@ int main(int argc, char** argv)
 
    timeStamp = getMicroTime() + offset;
    if(!textOutput) {
-      printf("%Lu\n", timeStamp);
+      printf("%llu\n", timeStamp);
    }
    else {
       const time_t     timeValue = timeStamp / 1000000;
       const struct tm* timePtr   = localtime(&timeValue);
       strftime((char*)&str, sizeof(str), "%d-%b-%Y %H:%M:%S", timePtr);
-      printf(str);
-      printf(".%06Lu, offset is %Luus\n", timeStamp % 1000000ULL, offset);
+      fputs(str, stdout);
+      printf(".%06Lu, offset is %lluus\n", timeStamp % 1000000ULL, offset);
    }
 
    return(0);
