@@ -1,4 +1,4 @@
-/* $Id: interthreadmessageport.c 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: interthreadmessageport.c 2141 2009-07-08 12:50:10Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -62,6 +62,20 @@ void interThreadMessagePortUnlock(struct InterThreadMessagePort* itmPort)
 }
 
 
+/* ###### Get first message ############################################## */
+bool interThreadMessagePortIsFirstMessage(struct InterThreadMessagePort* itmPort,
+                                          struct InterThreadMessageNode* message)
+{
+   bool result;
+
+   threadSignalLock(&itmPort->Signal);
+   result = (interThreadMessagePortGetFirstMessage(itmPort) == message);
+   threadSignalUnlock(&itmPort->Signal);
+
+   return(result);
+}
+
+
 /* ###### Get first message ############################################## */
 struct InterThreadMessageNode* interThreadMessagePortGetFirstMessage(struct InterThreadMessagePort* itmPort)
 {
@@ -76,7 +90,7 @@ struct InterThreadMessageNode* interThreadMessagePortGetFirstMessage(struct Inte
 
 /* ###### Get next message ############################################### */
 struct InterThreadMessageNode* interThreadMessagePortGetNextMessage(struct InterThreadMessagePort* itmPort,
-                                                                    struct InterThreadMessageNode*     message)
+                                                                    struct InterThreadMessageNode* message)
 {
    struct DoubleLinkedRingListNode* node;
    node = message->Node.Next;
