@@ -1,4 +1,4 @@
-/* $Id: interthreadmessageport.h 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: interthreadmessageport.h 2141 2009-07-08 12:50:10Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -59,6 +59,8 @@ void interThreadMessagePortNew(struct InterThreadMessagePort* itmPort);
 void interThreadMessagePortDelete(struct InterThreadMessagePort* itmPort);
 void interThreadMessagePortLock(struct InterThreadMessagePort* itmPort);
 void interThreadMessagePortUnlock(struct InterThreadMessagePort* itmPort);
+bool interThreadMessagePortIsFirstMessage(struct InterThreadMessagePort* itmPort,
+                                          struct InterThreadMessageNode* message);
 struct InterThreadMessageNode* interThreadMessagePortGetFirstMessage(struct InterThreadMessagePort* itmPort);
 struct InterThreadMessageNode* interThreadMessagePortGetNextMessage(struct InterThreadMessagePort* itmPort,
                                                                     struct InterThreadMessageNode* message);
