@@ -1,4 +1,4 @@
-/* $Id: messagebuffer.c 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: messagebuffer.c 2264 2009-10-27 18:25:43Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -29,9 +29,12 @@
 
 #include "tdtypes.h"
 #include "messagebuffer.h"
+#include "timeutilities.h"
 #include "netutilities.h"
 #include "loglevel.h"
 
+#include <string.h>
+
 
 /* ###### Constructor #################################################### */
 struct MessageBuffer* messageBufferNew(size_t bufferSize, const bool useEOR)
@@ -76,7 +79,6 @@ ssize_t messageBufferRead(struct MessageBuffer*    messageBuffer,
                           const unsigned long long timeout)
 {
    ssize_t result;
-   /* ssize_t i; */
 
    LOG_VERBOSE4
    fprintf(stdlog, "Reading into message buffer from socket %d: offset=%u, max=%u\n",
