@@ -1,4 +1,4 @@
-/* $Id: netutilities.c 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: netutilities.c 2259 2009-10-25 14:56:13Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -54,6 +54,8 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <netdb.h>
+#include <math.h>
+#include <assert.h>
 #ifdef SOLARIS
 #include <sys/sockio.h>
 #define CMSG_SPACE(len) (_CMSG_HDR_ALIGN(sizeof(struct cmsghdr)) + _CMSG_DATA_ALIGN(len))
@@ -157,11 +159,11 @@ ssize_t sctp_sendx(int                           sd,
 int ext_poll(struct pollfd* fdlist, long unsigned int count, int time)
 {
    // ====== Prepare timeout setting ========================================
-   struct timeval  timeout;
-   struct timeval* to;
-   int    fdcount = 0;
-   int tsize;
-   int       result;
+   struct       timeval  timeout;
+   struct       timeval* to;
+   int          fdcount = 0;
+   int          n;
+   int          result;
    unsigned int i;
 
    if(time < 0)
@@ -179,29 +181,27 @@ int ext_poll(struct pollfd* fdlist, long unsigned int count, int time)
    FD_ZERO(&readfdset);
    FD_ZERO(&writefdset);
    FD_ZERO(&exceptfdset);
+   n = 0;
    for(i = 0; i < count; i++) {
       if(fdlist[i].fd < 0) {
          continue;
       }
       if(fdlist[i].events & POLLIN) {
-         FD_SET(fdlist[i].fd,&readfdset);
+         FD_SET(fdlist[i].fd, &readfdset);
       }
       if(fdlist[i].events & POLLOUT) {
-         FD_SET(fdlist[i].fd,&writefdset);
+         FD_SET(fdlist[i].fd, &writefdset);
       }
-      FD_SET(fdlist[i].fd,&exceptfdset);
+      FD_SET(fdlist[i].fd, &exceptfdset);
+      n = max(n, fdlist[i].fd);
       fdcount++;
    }
-   if(fdcount == 0) {
-      return(0);
-   }
    for(i = 0;i < count;i++) {
       fdlist[i].revents = 0;
    }
 
    // ====== Do ext_select() ================================================
-   tsize  = getdtablesize();
-   result = ext_select(tsize,&readfdset,&writefdset,&exceptfdset,to);
+   result = ext_select(n + 1, &readfdset, &writefdset ,&exceptfdset, to);
    if(result < 0) {
       return(result);
    }
@@ -463,8 +463,8 @@ static bool obtainLocalAddresses(union sockaddr_union** addressArray,
 
 
 /* ###### Unpack sockaddr blocks to sockaddr_union array ################## */
-union sockaddr_union* unpack_sockaddr(struct sockaddr* addrArray,
-                                      const size_t     addrs)
+union sockaddr_union* unpack_sockaddr(const struct sockaddr* addrArray,
+                                      const size_t            addrs)
 {
    union sockaddr_union* newArray;
    size_t                i;
@@ -628,6 +628,26 @@ void deleteAddressArray(union sockaddr_union* addressArray)
 }
 
 
+/* ###### Filter address array by scope ################################## */
+size_t filterAddressesByScope(union sockaddr_union* addressArray,
+                              const size_t          inputAddresses,
+                              const unsigned int    minScope)
+{
+   size_t outputAddresses = 0;
+   size_t i;
+   
+   for(i = 0;i < inputAddresses;i++) {
+      if(getScope(&addressArray[i].sa) >= minScope) {
+         if(outputAddresses != i) {
+            memcpy((void*)&addressArray[outputAddresses], (void*)&addressArray[i], sizeof(union sockaddr_union));
+         }
+         outputAddresses++;
+      }
+   }
+   return(outputAddresses);
+}
+
+
 /* ###### Duplicate address array ######################################## */
 union sockaddr_union* duplicateAddressArray(const union sockaddr_union* addressArray,
                                             const size_t                addresses)
@@ -1324,8 +1344,8 @@ int sendtoplus(int                      sockfd,
    unsigned long long     remainingTimeout;
 
    LOG_VERBOSE4
-   fprintf(stdlog, "sendmsg(%d/A%u, %u bytes) PPID=$%08x streamID=%u flags=$%x toaddrs=%p toaddrcnt=%u...\n",
-           sockfd, (unsigned int)assocID, (unsigned int)length, ppid, streamID, flags,toaddrs, (unsigned int)toaddrcnt);
+   fprintf(stdlog, "sendmsg(%d/A%u, %u bytes) PPID=$%08x streamID=%u flags=$%x sctpFlags=$%x toaddrs=%p toaddrcnt=%u...\n",
+           sockfd, (unsigned int)assocID, (unsigned int)length, ppid, streamID, flags, sctpFlags, toaddrs, (unsigned int)toaddrcnt);
    LOG_END
 
    setNonBlocking(sockfd);
@@ -1402,11 +1422,11 @@ int sendtoplus(int                      sockfd,
          pfd.fd      = sockfd;
          pfd.events  = POLLOUT;
          pfd.revents = 0;
-         result = ext_poll((struct pollfd*)&pfd, 1, (int)(remainingTimeout / 1000));
-         if( (result > 0) && (pfd.revents = POLLOUT) ) {
+         result = ext_poll((struct pollfd*)&pfd, 1, (int)ceil((double)remainingTimeout / 1000.0));
+         if( (result > 0) && (pfd.revents & POLLOUT) ) {
             LOG_VERBOSE4
             fprintf(stdlog, "retrying sendmsg(%d/A%u, %u bytes)...\n",
-                  sockfd, (unsigned int)assocID, (unsigned int)length);
+                    sockfd, (unsigned int)assocID, (unsigned int)length);
             LOG_END
             if((assocID != 0) || (ppid != 0) || (streamID != 0)) {
                if(toaddrs) {
@@ -1414,8 +1434,8 @@ int sendtoplus(int                      sockfd,
                   fputs("Calling sctp_sendx() with addresses...\n", stdlog);
                   LOG_END
                   result = sctp_sendx(sockfd, buffer, length,
-                                    (struct sockaddr*)&addressArray, addresses,
-                                    &sri, flags);
+                                      (struct sockaddr*)&addressArray, addresses,
+                                      &sri, flags);
                }
                else {
                   LOG_VERBOSE5
@@ -1429,7 +1449,7 @@ int sendtoplus(int                      sockfd,
                fputs("Calling sctp_sendto()...\n", stdlog);
                LOG_END
                result = ext_sendto(sockfd, buffer, length, flags,
-                                 (struct sockaddr*)toaddrs, (toaddrs != NULL) ? getSocklen((struct sockaddr*)toaddrs) : 0);
+                                   (struct sockaddr*)toaddrs, (toaddrs != NULL) ? getSocklen((struct sockaddr*)toaddrs) : 0);
             }
          }
          if( (result >= 0) || (errno != EWOULDBLOCK) ) {
@@ -1502,7 +1522,7 @@ int recvfromplus(int                      sockfd,
 
    setNonBlocking(sockfd);
    cc = ext_recvmsg(sockfd, &msg, *flags);
-   if((timeout > 0) && ((cc < 0) && (errno == EWOULDBLOCK))) {
+   if((cc < 0) && (errno == EWOULDBLOCK) && (timeout > 0)) {
       LOG_VERBOSE5
       fprintf(stdlog, "recvmsg(%d) would block, waiting with timeout %lld [us]...\n",
               sockfd, timeout);
@@ -1511,8 +1531,8 @@ int recvfromplus(int                      sockfd,
       pfd.fd      = sockfd;
       pfd.events  = POLLIN;
       pfd.revents = 0;
-      result = ext_poll((struct pollfd*)&pfd, 1, (int)(timeout / 1000));
-      if( (result > 0) && (pfd.revents = POLLIN) ) {
+      result = ext_poll((struct pollfd*)&pfd, 1, (int)ceil((double)timeout / 1000.0));
+      if( (result > 0) && (pfd.revents & POLLIN) ) {
          LOG_VERBOSE5
          fprintf(stdlog, "retrying recvmsg(%d, %u bytes)...\n",
                  sockfd, (unsigned int)iov.iov_len);
@@ -1532,13 +1552,19 @@ int recvfromplus(int                      sockfd,
          msg.msg_flags      = *flags;
          cc = ext_recvmsg(sockfd, &msg, *flags);
       }
-      else {
+      else if(result == 0) {   /* Timeout */
          LOG_VERBOSE5
          fprintf(stdlog, "recvmsg(%d) timed out\n", sockfd);
          LOG_END
          cc    = -1;
          errno = EWOULDBLOCK;
       }
+      else {   /* Error */
+         LOG_ERROR
+         fprintf(stdlog, "poll(%d) failed: %s\n", sockfd, strerror(errno));
+         LOG_END
+         cc = -1;
+      }
    }
 
    LOG_VERBOSE4
@@ -1791,176 +1817,6 @@ bool setReusable(int sd, int on)
 }
 
 
-/* ###### Tune SCTP parameters ############################################ */
-bool tuneSCTP(int sockfd, sctp_assoc_t assocID, struct TagItem* tags)
-{
-   struct sctp_rtoinfo     rtoinfo;
-   struct sctp_paddrparams peerParams;
-   struct sctp_assocparams assocParams;
-   union sockaddr_union*   addrs = NULL;
-   socklen_t               size;
-   int                     i, n;
-   bool                    result = true;
-
-   size = (socklen_t)sizeof(rtoinfo);
-   rtoinfo.srto_assoc_id = assocID;
-   if(ext_getsockopt(sockfd, IPPROTO_SCTP, SCTP_RTOINFO, &rtoinfo, &size) == 0) {
-      LOG_VERBOSE3
-      fprintf(stdlog, "  Current RTO info of socket %d, assoc %u:\n", sockfd, (unsigned int)rtoinfo.srto_assoc_id);
-      fprintf(stdlog, "  Initial SRTO: %u\n", rtoinfo.srto_initial);
-      fprintf(stdlog, "  Min SRTO:     %u\n", rtoinfo.srto_min);
-      fprintf(stdlog, "  Max SRTO:     %u\n", rtoinfo.srto_max);
-      LOG_END
-
-      rtoinfo.srto_min     = tagListGetData(tags, TAG_TuneSCTP_MinRTO,     rtoinfo.srto_min);
-      rtoinfo.srto_max     = tagListGetData(tags, TAG_TuneSCTP_MaxRTO,     rtoinfo.srto_max);
-      rtoinfo.srto_initial = tagListGetData(tags, TAG_TuneSCTP_InitialRTO, rtoinfo.srto_initial);
-
-      if(ext_setsockopt(sockfd, IPPROTO_SCTP, SCTP_RTOINFO, &rtoinfo, (socklen_t)sizeof(rtoinfo)) < 0) {
-         result = false;
-         LOG_WARNING
-         logerror("SCTP_RTOINFO");
-         fprintf(stdlog, "Unable to update RTO info of socket %d, assoc %u\n", sockfd, (unsigned int)assocID);
-         LOG_END
-      }
-      else {
-         LOG_VERBOSE2
-         fprintf(stdlog, "  New RTO info of socket %d, assoc %u:\n", sockfd, (unsigned int)rtoinfo.srto_assoc_id);
-         fprintf(stdlog, "  Initial SRTO: %u\n", rtoinfo.srto_initial);
-         fprintf(stdlog, "  Min SRTO:     %u\n", rtoinfo.srto_min);
-         fprintf(stdlog, "  Max SRTO:     %u\n", rtoinfo.srto_max);
-         LOG_END
-      }
-   }
-   else {
-      LOG_VERBOSE2
-      logerror("Cannot get RTO info -> Skipping RTO info configuration");
-      LOG_END
-   }
-
-   size = (socklen_t)sizeof(assocParams);
-   assocParams.sasoc_assoc_id = assocID;
-   if(ext_getsockopt(sockfd, IPPROTO_SCTP, SCTP_ASSOCINFO, &assocParams, &size) == 0) {
-      LOG_VERBOSE3
-      fprintf(stdlog, "Current Assoc info of socket %d, assoc %u:\n", sockfd, (unsigned int)assocParams.sasoc_assoc_id);
-      fprintf(stdlog, "  AssocMaxRXT:       %u\n", assocParams.sasoc_asocmaxrxt);
-      fprintf(stdlog, "  Peer Destinations: %u\n", assocParams.sasoc_number_peer_destinations);
-      fprintf(stdlog, "  Peer RWND:         %u\n", assocParams.sasoc_peer_rwnd);
-      fprintf(stdlog, "  Local RWND:        %u\n", assocParams.sasoc_local_rwnd);
-      fprintf(stdlog, "  Cookie Life:       %u\n", assocParams.sasoc_cookie_life);
-      LOG_END
-
-      assocParams.sasoc_asocmaxrxt  = tagListGetData(tags, TAG_TuneSCTP_AssocMaxRXT, assocParams.sasoc_asocmaxrxt);
-      assocParams.sasoc_local_rwnd  = tagListGetData(tags, TAG_TuneSCTP_LocalRWND,   assocParams.sasoc_local_rwnd);
-      assocParams.sasoc_cookie_life = tagListGetData(tags, TAG_TuneSCTP_CookieLife,  assocParams.sasoc_cookie_life);
-
-      if(ext_setsockopt(sockfd, IPPROTO_SCTP, SCTP_ASSOCINFO, &assocParams, (socklen_t)sizeof(assocParams)) < 0) {
-         result = false;
-         LOG_WARNING
-         logerror("SCTP_ASSOCINFO");
-         fprintf(stdlog, "Unable to update Assoc info of socket %d, assoc %u\n", sockfd, (unsigned int)assocID);
-         LOG_END
-      }
-      else {
-         LOG_VERBOSE2
-         fprintf(stdlog, "New Assoc info of socket %d, assoc %u:\n", sockfd, (unsigned int)assocParams.sasoc_assoc_id);
-         fprintf(stdlog, "  AssocMaxRXT:       %u\n", assocParams.sasoc_asocmaxrxt);
-         fprintf(stdlog, "  Peer Destinations: %u\n", assocParams.sasoc_number_peer_destinations);
-         fprintf(stdlog, "  Peer RWND:         %u\n", assocParams.sasoc_peer_rwnd);
-         fprintf(stdlog, "  Local RWND:        %u\n", assocParams.sasoc_local_rwnd);
-         fprintf(stdlog, "  Cookie Life:       %u\n", assocParams.sasoc_cookie_life);
-         LOG_END
-      }
-   }
-   else {
-      LOG_VERBOSE2
-      logerror("Cannot get Assoc info -> Skipping Assoc info configuration");
-      LOG_END
-   }
-
-   n = getpaddrsplus(sockfd, assocID, &addrs);
-   if(n >= 0) {
-      for(i = 0;i < n;i++) {
-         memset(&peerParams, 0, sizeof(peerParams));
-         peerParams.spp_assoc_id = assocID;
-         memcpy((void*)&(peerParams.spp_address),
-                (const void*)&addrs[i], sizeof(union sockaddr_union));
-         size = (socklen_t)sizeof(peerParams);
-
-         if(sctp_opt_info(sockfd, assocID, SCTP_PEER_ADDR_PARAMS,
-                          (void*)&peerParams, &size) == 0) {
-            LOG_VERBOSE3
-            fputs("Old peer parameters for address ", stdlog);
-            fputaddress((struct sockaddr*)&(peerParams.spp_address), false, stdlog);
-#ifdef HAVE_SPP_FLAGS
-            fprintf(stdlog, " on socket %d, assoc %u: hb=%d maxrxt=%d flags=$%x\n",
-                    sockfd, (unsigned int)assocID,
-                    peerParams.spp_hbinterval, peerParams.spp_pathmaxrxt, peerParams.spp_flags);
-#else
-            fprintf(stdlog, " on socket %d, assoc %u: hb=%d maxrxt=%d\n",
-                    sockfd, (unsigned int)assocID,
-                    peerParams.spp_hbinterval, peerParams.spp_pathmaxrxt);
-#endif
-            LOG_END
-
-            peerParams.spp_hbinterval = tagListGetData(tags, TAG_TuneSCTP_Heartbeat,  peerParams.spp_hbinterval);
-#ifdef HAVE_SPP_FLAGS
-            if(peerParams.spp_hbinterval > 0) {
-               peerParams.spp_flags |= SPP_HB_ENABLE;
-            }
-            else {
-               peerParams.spp_flags |= SPP_HB_DISABLE;
-            }
-#endif
-            peerParams.spp_pathmaxrxt = tagListGetData(tags, TAG_TuneSCTP_PathMaxRXT, peerParams.spp_pathmaxrxt);;
-
-            if(sctp_opt_info(sockfd, 0, SCTP_PEER_ADDR_PARAMS,
-                             (void *)&peerParams, &size) < 0) {
-               LOG_VERBOSE2
-               fputs("Unable to set peer parameters for address ", stdlog);
-               fputaddress((struct sockaddr*)&(peerParams.spp_address), false, stdlog);
-               fprintf(stdlog, " on socket %d, assoc %u\n",
-                       sockfd, (unsigned int)assocID);
-               LOG_END
-            }
-            else {
-               LOG_VERBOSE2
-               fputs("New peer parameters for address ", stdlog);
-               fputaddress((struct sockaddr*)&(peerParams.spp_address), false, stdlog);
-#ifdef HAVE_SPP_FLAGS
-                fprintf(stdlog, " on socket %d, assoc %u: hb=%d maxrxt=%d flags=$%x\n",
-                        sockfd, (unsigned int)assocID,
-                        peerParams.spp_hbinterval, peerParams.spp_pathmaxrxt, peerParams.spp_flags);
-#else
-                fprintf(stdlog, " on socket %d, assoc %u: hb=%d maxrxt=%d\n",
-                        sockfd, (unsigned int)assocID,
-                        peerParams.spp_hbinterval, peerParams.spp_pathmaxrxt);
-#endif
-               LOG_END
-            }
-         }
-         else {
-            LOG_VERBOSE2
-            fputs("Unable to get peer parameters for address ", stdlog);
-            fputaddress((struct sockaddr*)&(peerParams.spp_address), false, stdlog);
-            fprintf(stdlog, " on socket %d, assoc %u\n",
-                    sockfd, (unsigned int)assocID);
-            LOG_END
-         }
-      }
-      free(addrs);
-   }
-   else {
-      result = false;
-      LOG_VERBOSE2
-      fputs("No peer addresses -> skipping peer parameters\n", stdlog);
-      LOG_END
-   }
-
-   return(result);
-}
-
-
 /* ###### Establish connection ########################################### */
 int connectplus(int                   sockfd,
                 union sockaddr_union* addressArray,
@@ -2161,6 +2017,13 @@ size_t getladdrsplus(const int              fd,
       LOG_END
       return((size_t)addrs);
    }
+   else {
+      if(packedAddresses) {
+         /* LK-SCTP may allocate memory even when the number of addresses is 0.
+            => free it here if packedAddresses is set. */
+         sctp_freeladdrs(packedAddresses);
+      }
+   }
    return(0);
 }
 
