@@ -1,28 +1,32 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: peerlist-template_impl.h 2086 2009-01-07 10:29:41Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: peerlist-template_impl.h 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
-
 /* ###### Initialize ##################################################### */
 void ST_CLASS(peerListNew)(struct ST_CLASS(PeerList)*    peerList,
                            const RegistrarIdentifierType ownIdentifier)
@@ -65,7 +69,7 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListGetFirstPeerListNodeFromIndexSto
       return(ST_CLASS(getPeerListNodeFromPeerListIndexStorageNode)(node));
    }
    return(NULL);
-};
+}
 
 
 /* ###### Get last PeerListNode from Index ############################### */
@@ -77,7 +81,7 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListGetLastPeerListNodeFromIndexStor
       return(ST_CLASS(getPeerListNodeFromPeerListIndexStorageNode)(node));
    }
    return(NULL);
-};
+}
 
 
 /* ###### Get next PeerListNode from Index ############################### */
@@ -117,7 +121,7 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListGetFirstPeerListNodeFromTimerSto
       return(ST_CLASS(getPeerListNodeFromPeerListTimerStorageNode)(node));
    }
    return(NULL);
-};
+}
 
 
 /* ###### Get last PeerListNode from Timer ############################### */
@@ -129,7 +133,7 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListGetLastPeerListNodeFromTimerStor
       return(ST_CLASS(getPeerListNodeFromPeerListTimerStorageNode)(node));
    }
    return(NULL);
-};
+}
 
 
 /* ###### Get next PeerListNode from Timer ############################### */
@@ -274,8 +278,7 @@ unsigned int ST_CLASS(peerListCheckPeerListNodeCompatibility)(
                 const struct ST_CLASS(PeerList)*     peerList,
                 const struct ST_CLASS(PeerListNode)* peerListNode)
 {
-   /* It does not make sense to insert its one's own ID into the
-      peer list! */
+   /* It does not make sense to insert its own ID into the peer list! */
    if((peerList->OwnIdentifier != UNDEFINED_REGISTRAR_IDENTIFIER) &&
       (peerListNode->Identifier == peerList->OwnIdentifier)) {
       return(RSPERR_OWN_ID);
@@ -336,6 +339,7 @@ void ST_CLASS(peerListUpdatePeerListNode)(
          CHECK(result == &peerListNode->PeerListIndexStorageNode);
       }
       ST_CLASS(peerListNodeUpdate)(peerListNode, source);
+      peerListNode->Flags &= ~PLNF_NEW;
    }
 }
 
@@ -357,12 +361,12 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListAddOrUpdatePeerListNode)(
    struct ST_CLASS(PeerListNode)* newPeerListNode;
 
    newPeerListNode = ST_CLASS(peerListAddPeerListNode)(peerList, *peerListNode, errorCode);
-
    if(newPeerListNode != NULL) {
       if(newPeerListNode != *peerListNode) {
          ST_CLASS(peerListUpdatePeerListNode)(peerList, newPeerListNode, *peerListNode, errorCode);
       }
       else {
+         newPeerListNode->Flags |= PLNF_NEW;
          *peerListNode = NULL;
       }
    }
@@ -465,7 +469,7 @@ struct ST_CLASS(PeerListNode)* ST_CLASS(peerListRemovePeerListNode)(
 
 
 /* ###### Get random PeerListNode from Index ############################# */
-struct ST_CLASS(PeerListNode)* ST_CLASS(peerListGetRandomPeerNode)(
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListGetRandomPeerListNode)(
                                   struct ST_CLASS(PeerList)* peerList)
 {
    unsigned long long       value;
