@@ -1,4 +1,4 @@
-/* $Id: peerlistmanagement-template_impl.h 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: peerlistmanagement-template_impl.h 2086 2009-01-07 10:29:41Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //=====  //   //      //
@@ -9,7 +9,7 @@
  *
  * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
  *
- * Copyright (C) 2003-2008 by Thomas Dreibholz
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -594,3 +594,28 @@ void ST_CLASS(peerListManagementVerifyChecksumsInHandlespace)(
       peerListNode = ST_CLASS(peerListGetNextPeerListNodeFromIndexStorage)(&peerListManagement->List, peerListNode);
    }
 }
+
+
+/* ###### Get better peer for PE ######################################### */
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetUsefulPeerForPE)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement,
+                                  const PoolElementIdentifierType      peIdentifier)
+{
+   struct ST_CLASS(PeerListNode)* bestPeerListNode = NULL;
+   struct ST_CLASS(PeerListNode)* peerListNode     = ST_CLASS(peerListGetFirstPeerListNodeFromIndexStorage)(&peerListManagement->List);
+   PoolElementIdentifierType      bestMetric       = peerListManagement->List.OwnIdentifier ^ peIdentifier;
+   PoolElementIdentifierType      metric;
+
+   /* Current best metric is the own node. Only return other entry if it has a
+      better metric! */
+
+   while(peerListNode != NULL) {
+      metric = peerListNode->Identifier ^ peIdentifier;
+      if( (peerListNode->Identifier != UNDEFINED_REGISTRAR_IDENTIFIER) && (metric > bestMetric) ) {
+         bestPeerListNode = peerListNode;
+         bestMetric       = metric;
+      }
+      peerListNode = ST_CLASS(peerListGetNextPeerListNodeFromIndexStorage)(&peerListManagement->List, peerListNode);
+   }
+   return(bestPeerListNode);
+}
