@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: poolhandlespacemanagement.h 2086 2009-01-07 10:29:41Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: poolhandlespacemanagement.h 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef POOLNAMESPACEMANAGEMENT_H
@@ -36,6 +41,8 @@
 #include "transportaddressblock.h"
 #include "stringutilities.h"
 
+#include <math.h>
+
 
 #ifdef INCLUDE_LINEARLIST
 #include "linearlist.h"
@@ -78,6 +85,8 @@
 #include "peerlistnode-template.h"
 #include "peerlist-template.h"
 #include "peerlistmanagement-template.h"
+#include "poolusernode-template.h"
+#include "pooluserlist-template.h"
 
 #ifdef INTERNAL_POOLTEMPLATE_IMPLEMENT_IT
 #include "poolpolicy-template_impl.h"
@@ -88,6 +97,8 @@
 #include "peerlistnode-template_impl.h"
 #include "peerlist-template_impl.h"
 #include "peerlistmanagement-template_impl.h"
+#include "poolusernode-template_impl.h"
+#include "pooluserlist-template_impl.h"
 #endif
 
 #undef STN_CLASSNAME
@@ -114,6 +125,8 @@
 #include "peerlistnode-template.h"
 #include "peerlist-template.h"
 #include "peerlistmanagement-template.h"
+#include "poolusernode-template.h"
+#include "pooluserlist-template.h"
 
 #ifdef INTERNAL_POOLTEMPLATE_IMPLEMENT_IT
 #include "poolpolicy-template_impl.h"
@@ -124,6 +137,8 @@
 #include "peerlistnode-template_impl.h"
 #include "peerlist-template_impl.h"
 #include "peerlistmanagement-template_impl.h"
+#include "poolusernode-template_impl.h"
+#include "pooluserlist-template_impl.h"
 #endif
 
 #undef STN_CLASSNAME
@@ -151,6 +166,8 @@
 #include "peerlistnode-template.h"
 #include "peerlist-template.h"
 #include "peerlistmanagement-template.h"
+#include "poolusernode-template.h"
+#include "pooluserlist-template.h"
 
 #ifdef INTERNAL_POOLTEMPLATE_IMPLEMENT_IT
 #include "poolpolicy-template_impl.h"
@@ -161,6 +178,8 @@
 #include "peerlistnode-template_impl.h"
 #include "peerlist-template_impl.h"
 #include "peerlistmanagement-template_impl.h"
+#include "poolusernode-template_impl.h"
+#include "pooluserlist-template_impl.h"
 #endif
 
 #undef STN_CLASSNAME
@@ -188,6 +207,8 @@
 #include "peerlistnode-template.h"
 #include "peerlist-template.h"
 #include "peerlistmanagement-template.h"
+#include "poolusernode-template.h"
+#include "pooluserlist-template.h"
 
 #ifdef INTERNAL_POOLTEMPLATE_IMPLEMENT_IT
 #include "poolpolicy-template_impl.h"
@@ -198,6 +219,8 @@
 #include "peerlistnode-template_impl.h"
 #include "peerlist-template_impl.h"
 #include "peerlistmanagement-template_impl.h"
+#include "poolusernode-template_impl.h"
+#include "pooluserlist-template_impl.h"
 #endif
 
 #undef STN_CLASSNAME
@@ -225,6 +248,8 @@
 #include "peerlistnode-template.h"
 #include "peerlist-template.h"
 #include "peerlistmanagement-template.h"
+#include "poolusernode-template.h"
+#include "pooluserlist-template.h"
 
 #ifdef INTERNAL_POOLTEMPLATE_IMPLEMENT_IT
 #include "poolpolicy-template_impl.h"
@@ -235,6 +260,8 @@
 #include "peerlistnode-template_impl.h"
 #include "peerlist-template_impl.h"
 #include "peerlistmanagement-template_impl.h"
+#include "poolusernode-template_impl.h"
+#include "pooluserlist-template_impl.h"
 #endif
 
 #undef STN_CLASSNAME
@@ -262,6 +289,8 @@
 #include "peerlistnode-template.h"
 #include "peerlist-template.h"
 #include "peerlistmanagement-template.h"
+#include "poolusernode-template.h"
+#include "pooluserlist-template.h"
 
 #ifdef INTERNAL_POOLTEMPLATE_IMPLEMENT_IT
 #include "poolpolicy-template_impl.h"
@@ -272,6 +301,8 @@
 #include "peerlistnode-template_impl.h"
 #include "peerlist-template_impl.h"
 #include "peerlistmanagement-template_impl.h"
+#include "poolusernode-template_impl.h"
+#include "pooluserlist-template_impl.h"
 #endif
 
 #undef STN_CLASSNAME
@@ -299,6 +330,8 @@
 #include "peerlistnode-template.h"
 #include "peerlist-template.h"
 #include "peerlistmanagement-template.h"
+#include "poolusernode-template.h"
+#include "pooluserlist-template.h"
 
 #ifdef INTERNAL_POOLTEMPLATE_IMPLEMENT_IT
 #include "poolpolicy-template_impl.h"
@@ -309,6 +342,8 @@
 #include "peerlistnode-template_impl.h"
 #include "peerlist-template_impl.h"
 #include "peerlistmanagement-template_impl.h"
+#include "poolusernode-template_impl.h"
+#include "pooluserlist-template_impl.h"
 #endif
 
 #undef STN_CLASSNAME
