@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: poolpolicy-template_impl.h 2086 2009-01-07 10:29:41Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: poolpolicy-template_impl.h 1436 2007-06-06 12:30:19Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "randomizer.h"
@@ -189,14 +194,17 @@ size_t ST_CLASS(poolPolicySelectPoolElementNodesByValueTree)(
             poolNode->GlobalSeqNumber++;
          poolElementNodeArray[poolElementNodes]->SelectionCounter++;
 
-         /* Policy-specifc pool element node updates (e.g. counter changes) */
-         if(poolNode->Policy->UpdatePoolElementNodeFunction) {
-            poolNode->Policy->UpdatePoolElementNodeFunction(poolElementNodeArray[poolElementNodes]);
+         /* Update PE entries with respect to maxIncrement setting. */
+         if(poolElementNodes < maxIncrement) {
+            /* Policy-specifc pool element node updates (e.g. counter changes) */
+            if(poolNode->Policy->UpdatePoolElementNodeFunction) {
+               poolNode->Policy->UpdatePoolElementNodeFunction(poolElementNodeArray[poolElementNodes]);
+            }
          }
 
-         if(poolElementNodes < maxIncrement) {
-            ST_CLASS(poolNodeUnlinkPoolElementNodeFromSelection)(poolNode, poolElementNodeArray[poolElementNodes]);
-         }
+         /* Unlinking *must* be done for all PEs
+               -> otherwise, multiple selections of the same PE possible! */
+         ST_CLASS(poolNodeUnlinkPoolElementNodeFromSelection)(poolNode, poolElementNodeArray[poolElementNodes]);
          poolElementNodes++;
       }
       else {
@@ -204,7 +212,8 @@ size_t ST_CLASS(poolPolicySelectPoolElementNodesByValueTree)(
       }
    }
 
-   for(i = 0;i < min(poolElementNodes, maxIncrement);i++) {
+   /* Re-linking of all previously unlinked nodes */
+   for(i = 0;i < poolElementNodes;i++) {
       ST_CLASS(poolNodeLinkPoolElementNodeToSelection)(poolNode, poolElementNodeArray[i]);
    }
 
@@ -228,23 +237,6 @@ static int ST_CLASS(roundRobinComparison)(
 }
 
 
-/*
-   #######################################################################
-   #### Priority Policy                                               ####
-   #######################################################################
-*/
-
-/* ###### Sorting Order ################################################## */
-static int ST_CLASS(priorityComparison)(
-   const struct ST_CLASS(PoolElementNode)* poolElementNode1,
-   const struct ST_CLASS(PoolElementNode)* poolElementNode2)
-{
-   COMPARE_KEY_DESCENDING(poolElementNode1->PolicySettings.Weight, poolElementNode2->PolicySettings.Weight);
-   COMPARE_KEY_ASCENDING(poolElementNode1->SeqNumber, poolElementNode2->SeqNumber);
-   return(0);
-}
-
-
 /*
    #######################################################################
    #### Weighted Round Robin Policy                                   ####
@@ -413,6 +405,23 @@ static void ST_CLASS(weightedRandomDPFUpdatePoolElementNode)(
 }
 
 
+/*
+   #######################################################################
+   #### Priority Policy                                               ####
+   #######################################################################
+*/
+
+/* ###### Sorting Order ################################################## */
+static int ST_CLASS(priorityComparison)(
+   const struct ST_CLASS(PoolElementNode)* poolElementNode1,
+   const struct ST_CLASS(PoolElementNode)* poolElementNode2)
+{
+   COMPARE_KEY_DESCENDING(poolElementNode1->PolicySettings.Weight, poolElementNode2->PolicySettings.Weight);
+   COMPARE_KEY_ASCENDING(poolElementNode1->SeqNumber, poolElementNode2->SeqNumber);
+   return(0);
+}
+
+
 /*
    #######################################################################
    #### Least Used Policy                                             ####
