@@ -1,4 +1,4 @@
-/* $Id: registrar-takeover.c 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: registrar-takeover.c 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,6 +56,10 @@ void registrarHandleENRPInitTakeover(struct Registrar*       registrar,
            message->SenderID,
            message->RegistrarIdentifier);
    LOG_END
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   registrarWriteActionLog(registrar, "Recv", "ENRP", "InitTakeover", "", 0, 0, 0,
+                           NULL, 0, message->SenderID, message->ReceiverID, message->RegistrarIdentifier, 0);
+#endif
 
    /* ====== We are the takeover target -> try to stop it by Peer Presence */
    if(message->RegistrarIdentifier == registrar->ServerID) {
@@ -156,6 +160,11 @@ void registrarSendENRPInitTakeoverToAllPeers(struct Registrar*             regis
       message->ReceiverID   = 0;
       message->RegistrarIdentifier = targetID;
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Send", "ENRP", "InitTakeover", "", 0, 0, 0,
+                              NULL, 0, message->SenderID, message->ReceiverID, message->RegistrarIdentifier, 0);
+#endif
+
       peerListNode = ST_CLASS(peerListManagementGetFirstPeerListNodeFromIndexStorage)(&registrar->Peers);
       while(peerListNode != NULL) {
          message->ReceiverID   = peerListNode->Identifier;
@@ -185,6 +194,7 @@ void registrarHandleENRPInitTakeoverAck(struct Registrar*       registrar,
                                         struct RSerPoolMessage* message)
 {
    struct ST_CLASS(PeerListNode)* peerListNode;
+   size_t                         acksToGo;
 
    if(message->SenderID == registrar->ServerID) {
       /* This is our own message -> skip it! */
@@ -203,9 +213,16 @@ void registrarHandleENRPInitTakeoverAck(struct Registrar*       registrar,
             message->RegistrarIdentifier);
       LOG_END
 
-      if(takeoverProcessAcknowledge(peerListNode->TakeoverProcess,
-                                    message->RegistrarIdentifier,
-                                    message->SenderID) > 0) {
+      acksToGo = takeoverProcessAcknowledge(peerListNode->TakeoverProcess,
+                                            message->RegistrarIdentifier,
+                                            message->SenderID);
+
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Recv", "ENRP", "InitTakeoverAck", "", 0, acksToGo, 0,
+                              NULL, 0, message->SenderID, message->ReceiverID, message->RegistrarIdentifier, 0);
+#endif
+
+      if(acksToGo > 0) {
          LOG_ACTION
          fprintf(stdlog, "Peer $%08x acknowledges takeover of target $%08x. %u acknowledges to go.\n",
                message->SenderID,
@@ -260,6 +277,11 @@ void registrarSendENRPInitTakeoverAck(struct Registrar*             registrar,
               message->RegistrarIdentifier,
               message->ReceiverID);
       LOG_END
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Send", "ENRP", "InitTakeoverAck", "", 0, 0, 0,
+                              NULL, 0, message->SenderID, message->ReceiverID, message->RegistrarIdentifier, 0);
+#endif
+
       if(rserpoolMessageSend(IPPROTO_SCTP,
                              sd, assocID, 0, 0, 0, message) == false) {
          LOG_WARNING
@@ -360,6 +382,12 @@ void registrarHandleENRPTakeoverServer(struct Registrar*       registrar,
            message->RegistrarIdentifier);
    LOG_END
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   registrarWriteActionLog(registrar, "Recv", "ENRP", "TakeoverServer", "", 0, 0, 0,
+                           NULL, 0, message->SenderID, message->ReceiverID, message->RegistrarIdentifier, 0);
+#endif
+
+
    /* ====== Update PEs' home PR identifier ============================== */
    poolElementNode = ST_CLASS(poolHandlespaceNodeGetFirstPoolElementOwnershipNodeForIdentifier)(
                         &registrar->Handlespace.Handlespace, message->RegistrarIdentifier);
@@ -409,6 +437,11 @@ static void registrarSendENRPTakeoverServer(struct Registrar*             regist
       message->ReceiverID          = receiverID;
       message->RegistrarIdentifier = targetID;
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Send", "ENRP", "TakeoverServer", "", 0, 0, 0,
+                              NULL, 0, message->SenderID, message->ReceiverID, message->RegistrarIdentifier, 0);
+#endif
+
       if(rserpoolMessageSend(IPPROTO_SCTP,
                              sd, assocID, msgSendFlags, 0, 0, message) == false) {
          LOG_WARNING
