@@ -1,4 +1,4 @@
-/* $Id: registrar.h 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: registrar.h 2245 2009-10-23 05:46:42Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -49,7 +49,6 @@
 #include <ext_socket.h>
 #include <net/if.h>
 #include <sys/ioctl.h>
-#define ENABLE_REGISTRAR_STATISTICS
 #ifdef ENABLE_REGISTRAR_STATISTICS
 #include <bzlib.h>
 #endif
@@ -396,4 +395,10 @@ bool registrarPoolUserHasPermissionFor(struct Registrar*               registrar
                                        const struct PoolHandle*        poolHandle,
                                        const PoolElementIdentifierType peIdentifier);
 
+/* ###### Miscellaneous ################################################## */
+void registrarRegistrationHook(struct Registrar*                 registrar,
+                               struct ST_CLASS(PoolElementNode)* poolElementNode);
+void registrarDeregistrationHook(struct Registrar*                 registrar,
+                                 struct ST_CLASS(PoolElementNode)* poolElementNode);
+
 #endif
