@@ -1,4 +1,4 @@
-/* $Id: registrartable.c 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: registrartable.c 2190 2009-08-05 07:43:39Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -44,10 +44,16 @@
 #include "netutilities.h"
 #include "randomizer.h"
 #include "loglevel.h"
+#ifdef ENABLE_CSP
+#include "componentstatusreporter.h"
+#endif
 
 #include <ext_socket.h>
 #include <sys/time.h>
 
+#ifdef ENABLE_CSP
+extern struct CSPReporter* gCSPReporter;
+#endif
 
 
 /* Maximum number of simultaneous registrar connection trials */
@@ -209,6 +215,12 @@ struct RegistrarTable* registrarTableNew(struct Dispatcher*          dispatcher,
                                                                                    ASAP_DEFAULT_REGISTRAR_CONNECT_TIMEOUT);
       registrarTable->RegistrarAnnounceTimeout = (unsigned long long)tagListGetData(tags, TAG_RspLib_RegistrarAnnounceTimeout,
                                                                                     ASAP_DEFAULT_REGISTRAR_ANNOUNCE_TIMEOUT);
+      LOG_VERBOSE3
+      fputs("New ASAP registrar table's configuration:\n", stdlog);
+      fprintf(stdlog, "registrartable.announce.timeout    = %lluus\n", registrarTable->RegistrarAnnounceTimeout);
+      fprintf(stdlog, "registrartable.connect.timeout     = %lluus\n", registrarTable->RegistrarConnectTimeout);
+      fprintf(stdlog, "registrartable.connect.maxtrials   = %u\n",     registrarTable->RegistrarConnectMaxTrials);
+      LOG_END
 
       if(enableAutoConfig) {
          if(registrarAnnounceAddress == NULL) {
@@ -316,7 +328,8 @@ static void addRegistrarAssocID(struct RegistrarTable* registrarTable,
                                 int                    registrarHuntFD,
                                 sctp_assoc_t           assocID)
 {
-   struct RegistrarAssocIDNode* node = (struct RegistrarAssocIDNode*)malloc(sizeof(struct RegistrarAssocIDNode));
+   struct RegistrarAssocIDNode* node =
+      (struct RegistrarAssocIDNode*)malloc(sizeof(struct RegistrarAssocIDNode));
    if(node != NULL) {
       simpleRedBlackTreeNodeNew(&node->Node);
       node->Node.Value = 1;
@@ -409,7 +422,8 @@ static int selectRegistrar(struct RegistrarTable*   registrarTable,
          LOG_END
 
          /* ====== Get registrar's entry ================================= */
-         struct TransportAddressBlock* registrarAddressBlock = (struct TransportAddressBlock*)malloc(transportAddressBlockGetSize(n));
+         struct TransportAddressBlock* registrarAddressBlock =
+            (struct TransportAddressBlock*)malloc(transportAddressBlockGetSize(n));
          if(registrarAddressBlock) {
             transportAddressBlockNew(registrarAddressBlock,
                                      IPPROTO_SCTP,
@@ -458,7 +472,7 @@ int registrarTablePeelOffRegistrarAssocID(struct RegistrarTable* registrarTable,
       LOG_END
    }
    else {
-      LOG_ERROR
+      LOG_VERBOSE
       fprintf(stdlog, "Assoc %u peel-off failed: %s\n", assocID, strerror(errno));
       LOG_END
       sendabort(registrarHuntFD, assocID);
@@ -470,10 +484,11 @@ int registrarTablePeelOffRegistrarAssocID(struct RegistrarTable* registrarTable,
 
 
 /* ###### Handle notification on registrar hunt socket ################### */
-void registrarTableHandleNotificationOnRegistrarHuntSocket(struct RegistrarTable*         registrarTable,
-                                                           int                            registrarHuntFD,
-                                                           struct MessageBuffer*          registrarHuntMessageBuffer,
-                                                           const union sctp_notification* notification)
+void registrarTableHandleNotificationOnRegistrarHuntSocket(
+        struct RegistrarTable*         registrarTable,
+        int                            registrarHuntFD,
+        struct MessageBuffer*          registrarHuntMessageBuffer,
+        const union sctp_notification* notification)
 {
    union sockaddr_union* peerAddressArray;
    size_t                n;
@@ -567,6 +582,18 @@ static void tryNextBlock(struct RegistrarTable*         registrarTable,
    int                            result;
    size_t                         i;
 
+
+   if( (registrarTable->AnnounceAddress.sa.sa_family != 0) &&
+       (!multicastGroupControl(registrarTable->AnnounceSocket,
+                               &registrarTable->AnnounceAddress,
+                               true)) ) {
+      LOG_WARNING
+      fputs("Joining multicast group ",  stdlog);
+      fputaddress(&registrarTable->AnnounceAddress.sa, true, stdlog);
+      fputs(" failed. Check routing (is default route set?) and firewall settings!\n",  stdlog);
+      LOG_END
+   }
+
    i = ST_CLASS(peerListManagementPurgeExpiredPeerListNodes)(
          &registrarTable->RegistrarList,
          getMicroTime());
@@ -574,13 +601,12 @@ static void tryNextBlock(struct RegistrarTable*         registrarTable,
    fprintf(stdlog, "Purged %u out-of-date peer list nodes. Peer List:\n",  (unsigned int)i);
    ST_CLASS(peerListManagementPrint)(&registrarTable->RegistrarList, stdlog, PLPO_PEERS_INDEX|PLNPO_TRANSPORT);
    LOG_END
-
+   
    for(i = 0;i < MAX_SIMULTANEOUS_REQUESTS;i++) {
       peerListNode = ST_CLASS(peerListManagementFindNearestNextPeerListNode)(
                         &registrarTable->RegistrarList,
                         *lastRegistrarIdentifier,
                         *lastTransportAddressBlock);
-
       if(peerListNode != NULL) {
          *lastRegistrarIdentifier = peerListNode->Identifier;
          if(*lastTransportAddressBlock) {
@@ -606,8 +632,7 @@ static void tryNextBlock(struct RegistrarTable*         registrarTable,
             result = connectplus(registrarHuntFD,
                                  transportAddressBlock->AddressArray,
                                  transportAddressBlock->Addresses);
-            if((result < 0) &&
-               (errno != EINPROGRESS) && (errno != EISCONN)) {
+            if((result < 0) && (errno != EINPROGRESS) && (errno != EISCONN)) {
                LOG_WARNING
                fputs("Connection to registrar ",  stdlog);
                transportAddressBlockPrint(transportAddressBlock, stdlog);
@@ -637,17 +662,18 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
                                struct MessageBuffer*    registrarHuntMessageBuffer,
                                RegistrarIdentifierType* registrarIdentifier)
 {
-   struct timeval                 selectTimeout;
    const union sctp_notification* notification;
    RegistrarIdentifierType        lastRegistrarIdentifier;
    struct TransportAddressBlock*  lastTransportAddressBlock;
    struct ST_CLASS(PeerListNode)* peerListNode;
    sctp_assoc_t                   assocID;
    ssize_t                        received;
+   unsigned long long             now;
    unsigned long long             start;
+   unsigned long long             stop;
    unsigned long long             nextTimeout;
    unsigned long long             lastTrialTimeStamp;
-   fd_set                         rfdset;
+   struct pollfd                  pollFDs[2];
    unsigned int                   trials;
    int                            result;
    int                            flags;
@@ -688,6 +714,10 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
       ST_CLASS(peerListNodePrint)(peerListNode, stdlog, PLPO_FULL);
       fputs("\n", stdlog);
       LOG_END
+      
+      /* Subtract one, in order to select this node, if it is the first one!
+         => Increased connection establishment speed. */
+      lastRegistrarIdentifier--;
    }
 
    lastTrialTimeStamp = 0;
@@ -700,7 +730,7 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
             - A new registrar announce has been added to the list AND
                  there are no outstanding connects
             - The current trial has been running for at least
-               registrarTable->RegistrarConnectTimeout
+                 registrarTable->RegistrarConnectTimeout
          */
          if( (start == 0) ||
              ((registrarTable->LastAnnounceHeard >= start) && (registrarTable->OutstandingConnects == 0)) ||
@@ -719,20 +749,9 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
 
             lastTrialTimeStamp = 0;
             start = getMicroTime();
-            LOG_VERBOSE2
+            LOG_VERBOSE3
             fprintf(stdlog, "Trial #%u...\n", trials);
             LOG_END
-
-            if( (registrarTable->AnnounceAddress.sa.sa_family != 0) &&
-                (!multicastGroupControl(registrarTable->AnnounceSocket,
-                                        &registrarTable->AnnounceAddress,
-                                        true)) ) {
-               LOG_WARNING
-               fputs("Joining multicast group ",  stdlog);
-               fputaddress(&registrarTable->AnnounceAddress.sa, true, stdlog);
-               fputs(" failed. Check routing (is default route set?) and firewall settings!\n",  stdlog);
-               LOG_END
-            }
          }
       }
 
@@ -748,26 +767,47 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
       }
 
       /* ====== Wait for event =========================================== */
-      FD_ZERO(&rfdset);
-      FD_SET(registrarHuntFD, &rfdset);
-      n = registrarHuntFD;
-      if(registrarTable->AnnounceSocket >= 0) {
-         FD_SET(registrarTable->AnnounceSocket, &rfdset);
-         n = max(n, registrarTable->AnnounceSocket);
-      }
-      nextTimeout = registrarTable->RegistrarConnectTimeout;
-      selectTimeout.tv_sec  = nextTimeout / (unsigned long long)1000000;
-      selectTimeout.tv_usec = nextTimeout % (unsigned long long)1000000;
+      stop = getMicroTime() + registrarTable->RegistrarConnectTimeout;
+      do {
+         pollFDs[0].fd      = registrarHuntFD;
+         pollFDs[0].events  = POLLIN;
+         pollFDs[0].revents = 0;
+         if(registrarTable->AnnounceSocket >= 0) {
+            pollFDs[1].fd      = registrarTable->AnnounceSocket;
+            pollFDs[1].events  = POLLIN;
+            pollFDs[1].revents = 0;
+            n = 2;
+         }
+         else {
+            n = 1;
+         }
+         now         = getMicroTime();
+         nextTimeout = (now <= stop) ? (stop - now) : 0;
+#ifdef ENABLE_CSP
+         if(gCSPReporter != NULL) {
+            now = getMicroTime();
+            if(gCSPReporter->CSPReportTimer.TimeStamp >= now) {
+               nextTimeout = min(nextTimeout, gCSPReporter->CSPReportTimer.TimeStamp - now);
+            }
+            else {
+               nextTimeout = 0;
+            }
+         }
+#endif
 
-      LOG_VERBOSE4
-      fprintf(stdlog, "select() with timeout %lld\n", nextTimeout);
-      LOG_END
-      result = ext_select(n + 1,
-                          &rfdset, NULL, NULL,
-                          &selectTimeout);
-      LOG_VERBOSE4
-      fprintf(stdlog, "select() result=%d\n", result);
-      LOG_END
+         LOG_VERBOSE4
+         fprintf(stdlog, "poll() with timeout %d\n", (int)ceil((double)nextTimeout / 1000.0));
+         LOG_END
+         result = ext_poll((struct pollfd*)&pollFDs, n, (int)ceil((double)nextTimeout / 1000.0));
+#ifdef ENABLE_CSP
+         if(gCSPReporter != NULL) {
+            cspReporterHandleTimerDuringRegistrarSearch(gCSPReporter);
+         }
+#endif
+         LOG_VERBOSE4
+         fprintf(stdlog, "poll() result=%d\n", result);
+         LOG_END
+      } while( (result == 0) && (getMicroTime() < stop));
 
       if((result < 0) && (errno == EINTR)) {
          LOG_VERBOSE3
@@ -782,11 +822,10 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
 
       /* ====== Handle events ============================================ */
       if(result != 0) {
-         if( (registrarTable->AnnounceSocket > 0) &&
-             (FD_ISSET(registrarTable->AnnounceSocket, &rfdset)) ) {
+         if((registrarTable->AnnounceSocket > 0) && (pollFDs[1].revents & POLLIN)) {
             handleRegistrarAnnounceCallback(registrarTable, registrarTable->AnnounceSocket);
          }
-         if(FD_ISSET(registrarHuntFD, &rfdset)) {
+         if(pollFDs[0].revents & POLLIN) {
             flags    = MSG_PEEK;
             received = messageBufferRead(registrarHuntMessageBuffer, registrarHuntFD, &flags,
                                          NULL, 0,
@@ -813,7 +852,7 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
                         }
                         LOG_VERBOSE3
                         fprintf(stdlog, "Failed to establish registrar connection, outstanding=%u\n",
-                                 (unsigned int)registrarTable->OutstandingConnects);
+                                (unsigned int)registrarTable->OutstandingConnects);
                         LOG_END
                      }
                   }
@@ -842,8 +881,8 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
                           (unsigned int)assocID);
                   LOG_END
 
-                  sd = registrarTablePeelOffRegistrarAssocID(registrarTable, registrarHuntFD, registrarHuntMessageBuffer,
-                                                             assocID);
+                  sd = registrarTablePeelOffRegistrarAssocID(
+                          registrarTable, registrarHuntFD, registrarHuntMessageBuffer, assocID);
                   if(sd >= 0) {
                      LOG_ACTION
                      fprintf(stdlog, "Taking assoc %u as new registrar => socket %d\n", (unsigned int)assocID, sd);
