@@ -1,4 +1,4 @@
-/* $Id: rserpool-internals.h 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: rserpool-internals.h 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,17 +56,17 @@ struct TagItem;
 #define TAG_RspLib_RegistrarResponseTimeout          (TAG_USER + 4007)
 
 
-unsigned int rsp_pe_registration_tags(const unsigned char* poolHandle,
-                                      const size_t         poolHandleSize,
-                                      struct rsp_addrinfo* rspAddrInfo,
-                                      struct rsp_loadinfo* rspLoadInfo,
-                                      unsigned int         registrationLife,
-                                      int                  flags,
-                                      struct TagItem*      tags);
+unsigned int rsp_pe_registration_tags(const unsigned char*       poolHandle,
+                                      const size_t               poolHandleSize,
+                                      struct rsp_addrinfo*       rspAddrInfo,
+                                      const struct rsp_loadinfo* rspLoadInfo,
+                                      const unsigned int         registrationLife,
+                                      const int                  flags,
+                                      struct TagItem*            tags);
 unsigned int rsp_pe_deregistration_tags(const unsigned char* poolHandle,
                                         const size_t         poolHandleSize,
                                         const uint32_t       identifier,
-                                        int                  flags,
+                                        const int            flags,
                                         struct TagItem*      tags);
 unsigned int rsp_pe_failure_tags(const unsigned char* poolHandle,
                                  const size_t         poolHandleSize,
@@ -85,8 +85,8 @@ int rsp_register_tags(int                        sd,
                       const unsigned char*       poolHandle,
                       const size_t               poolHandleSize,
                       const struct rsp_loadinfo* loadinfo,
-                      unsigned int               reregistrationInterval,
-                      int                        flags,
+                      const unsigned int         reregistrationInterval,
+                      const int                  flags,
                       struct TagItem*            tags);
 int rsp_deregister_tags(int            sd,
                        int             flags,
