@@ -1,4 +1,4 @@
-/* $Id: rserpool.h 2112 2009-03-14 12:01:35Z dreibh $
+/* $Id: rserpool.h 2207 2009-09-11 09:48:04Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -85,6 +85,13 @@ struct rsp_info
    int                        ri_disable_autoconfig;
    struct sockaddr*           ri_registrar_announce;
 
+   unsigned int               ri_registrar_announce_timeout;
+   unsigned int               ri_registrar_connect_timeout;
+   unsigned int               ri_registrar_connect_max_trials;
+   unsigned int               ri_registrar_request_timeout;
+   unsigned int               ri_registrar_response_timeout;
+   unsigned int               ri_registrar_request_max_trials;
+   
    uint64_t                   ri_csp_identifier;
    struct sockaddr*           ri_csp_server;
    unsigned int               ri_csp_interval;
@@ -102,6 +109,7 @@ struct rsp_loadinfo
 
 #define REGF_DONTWAIT         (1 << 0)   /* Do not wait for Registration Response   */
 #define REGF_CONTROLCHANNEL   (1 << 1)   /* Pool Element has Control Channel        */
+#define REGF_DAEMONMODE       (1 << 2)   /* Daemon mode: don not stop on errors     */
 
 #define DEREGF_DONTWAIT       (1 << 0)   /* Do not wait for Deregistration Response */
 
@@ -314,7 +322,7 @@ int rsp_socket(int domain, int type, int protocol);
   * @param addrcnt Number of addresses in address array.
   * @return 0 in case of success; -1 in case of an error.
   */
-int rsp_bind(int sd, struct sockaddr* addrs, int addrcnt);
+int rsp_bind(int sd, const struct sockaddr* addrs, int addrcnt);
 
 /**
   * Put RSerPool socket (PE only) into listening mode.
