@@ -1,4 +1,4 @@
-/* $Id: rserpoolmessageparser.c 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: rserpoolmessageparser.c 2176 2009-08-03 16:22:49Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -500,7 +500,7 @@ static bool scanTransportParameter(struct RSerPoolMessage*       message,
    }
    if(addresses < 1) {
       LOG_WARNING
-      fputs("No addresses given!\n", stdlog);
+      fputs("No (usable) transport addresses given!\n", stdlog);
       LOG_END
       message->Error = RSPERR_INVALID_VALUE;
       return(false);
