@@ -1,4 +1,4 @@
-/* $Id: rspenhancedmode.c 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: rspenhancedmode.c 2258 2009-10-25 14:54:45Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -251,6 +251,7 @@ int rsp_close(int sd)
       rserpoolSocket->MsgBuffer = NULL;
    }
    threadSafetyDelete(&rserpoolSocket->SessionSetMutex);
+   threadSafetyUnlock(&rserpoolSocket->Mutex);
    threadSafetyDelete(&rserpoolSocket->Mutex);
    free(rserpoolSocket);
    return(0);
@@ -443,7 +444,7 @@ int rsp_select(int n, fd_set* readfds, fd_set* writefds, fd_set* exceptfds,
    }
 
    /* ====== Call poll() and propagate results to fdsets ================= */
-   waitingTime = (1000 * timeout->tv_sec) + (timeout->tv_usec / 1000);
+   waitingTime = (1000 * timeout->tv_sec) + (int)ceil((double)timeout->tv_usec / 1000.0);
    result = rsp_poll((struct pollfd*)&ufds, nfds, waitingTime);
    if(result > 0) {
       for(i = 0;i < nfds;i++) {
@@ -464,27 +465,35 @@ int rsp_select(int n, fd_set* readfds, fd_set* writefds, fd_set* exceptfds,
 
 
 /* ###### Bind RSerPool socket ########################################### */
-int rsp_bind(int sd, struct sockaddr* addrs, int addrcnt)
+int rsp_bind(int sd, const struct sockaddr* addrs, int addrcnt)
 {
    struct RSerPoolSocket* rserpoolSocket;
+   union sockaddr_union*  unpackedAddresses;
    union sockaddr_union   localAddress;
+   int                    result;
 
    GET_RSERPOOL_SOCKET(rserpoolSocket, sd);
 
    /* ====== Bind socket ================================================= */
-   if(addrs == NULL) {
+   if(addrcnt < 1) {
       addrs   = (struct sockaddr*)&localAddress;
       addrcnt = 1;
       memset((void*)&localAddress, 0, sizeof(localAddress));
       ((struct sockaddr*)&localAddress)->sa_family = rserpoolSocket->SocketDomain;
    }
-   if(bindplus(rserpoolSocket->Socket, (union sockaddr_union*)addrs, addrcnt) == false) {
+   unpackedAddresses = unpack_sockaddr(addrs, addrcnt);
+   if(unpackedAddresses == NULL) {
+      errno = ENOMEM;
+      return(-1);
+   }
+   result = bindplus(rserpoolSocket->Socket, unpackedAddresses, addrcnt);
+   free(unpackedAddresses);
+   if(result == false) {
       LOG_ERROR
       logerror("Unable to bind socket for new pool element");
       LOG_END
       return(-1);
    }
-
    return(0);
 }
 
@@ -584,16 +593,19 @@ int rsp_register_tags(int                        sd,
       rserpoolSocket->PoolElement->ReregistrationInterval = reregistrationInterval;
       rserpoolSocket->PoolElement->RegistrationLife       = 3 * rserpoolSocket->PoolElement->ReregistrationInterval;
       rserpoolSocket->PoolElement->HasControlChannel      = true;
+      rserpoolSocket->PoolElement->InDaemonMode           = (flags & REGF_DAEMONMODE);
 
       /* ====== Do registration ============================================= */
       if(doRegistration(rserpoolSocket, true) == false) {
          LOG_ERROR
          fputs("Unable to obtain registration information -> Creating pool element not possible\n", stdlog);
          LOG_END
-         deletePoolElement(rserpoolSocket->PoolElement, flags, tags);
-         rserpoolSocket->PoolElement = NULL;
-         threadSafetyUnlock(&rserpoolSocket->Mutex);
-         return(-1);
+         if(!(flags & REGF_DAEMONMODE)) {
+            deletePoolElement(rserpoolSocket->PoolElement, flags, tags);
+            rserpoolSocket->PoolElement = NULL;
+            threadSafetyUnlock(&rserpoolSocket->Mutex);
+            return(-1);
+         }
       }
 
       /* ====== start reregistration timer ================================== */
@@ -1155,7 +1167,6 @@ ssize_t rsp_recvmsg(int                    sd,
       }
    }
 
-
    /* ====== Read from socket ========================================= */
    startTimeStamp = getMicroTime();
    do {
@@ -1226,7 +1237,7 @@ ssize_t rsp_recvmsg(int                    sd,
                rserpoolSocket->ConnectedSession->IsFailed = true;
             }
          }
-         threadSafetyLock(&rserpoolSocket->Mutex);
+         threadSafetyUnlock(&rserpoolSocket->Mutex);
       }
 
       /* ====== Handle result =============================================== */
@@ -1443,14 +1454,14 @@ ssize_t rsp_send_cookie(int                  sd,
          message->Type       = AHT_COOKIE;
          message->CookiePtr  = (char*)cookie;
          message->CookieSize = (size_t)cookieSize;
-         threadSafetyLock(&rserpoolSocket->Mutex);
+         threadSafetyUnlock(&rserpoolSocket->Mutex);
          result = rserpoolMessageSend(IPPROTO_SCTP,
                                       rserpoolSocket->Socket,
                                       session->AssocID,
                                       0, 0,
                                       (1000ULL * timeout),
                                       message);
-         threadSafetyUnlock(&rserpoolSocket->Mutex);
+         threadSafetyLock(&rserpoolSocket->Mutex);
          rserpoolMessageDelete(message);
       }
    }
