@@ -1,4 +1,4 @@
-/* $Id: scriptingclient.c 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: scriptingclient.c 2263 2009-10-26 08:46:55Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -41,6 +41,8 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <ctype.h>
+#include <math.h>
 #include <errno.h>
 
 
@@ -48,35 +50,53 @@
 #define SSCR_COMPLETE    1
 #define SSCR_FAILOVER    2
 
-#define SSCS_WAIT_READY  1
-#define SSCS_PROCESSING  2
-#define SSCS_DOWNLOAD    3
+#define SSCS_WAIT_SERVER_READY     1
+#define SSCS_WAIT_START_PROCESSING 2
+#define SSCS_PROCESSING            3
+#define SSCS_DOWNLOAD              4
 
 
-static unsigned int       Status            = SSCS_WAIT_READY;
+static unsigned int       State             = SSCS_WAIT_SERVER_READY;
 static const char*        InputName         = NULL;
 static const char*        OutputName        = NULL;
 static FILE*              OutputFile        = NULL;
-static bool               Quiet             =  false;
+static bool               Quiet             = false;
 static unsigned int       MaxRetry          = 3;
-static unsigned long long TransmitTimeout   = 60000000;
-static unsigned long long KeepAliveInterval =  5000000;
-static unsigned long long KeepAliveTimeout  =  5000000;
+static unsigned long long TransmitTimeout   = 59500000;
+static unsigned long long KeepAliveInterval = 10000000;
+static unsigned long long KeepAliveTimeout  = 10000000;
 static unsigned long long RetryDelay        = 10000000;
-
 static const char*        RunID             = NULL;
 static unsigned int       Trial             = 1;
 static uint32_t           ExitStatus        = 0;
+static bool               HasAssignedPE     = false;
+static char               InfoString[SR_MAX_INFOSIZE + 1] = "";
 static bool               KeepAliveTransmitted;
 static unsigned long long LastKeepAlive;
 
 
+/* ###### Randomize waiting time ######################################### */
+static unsigned long long randomizeWaitingTime(const unsigned long long interval)
+{
+   const double originalInterval = (double)interval;
+   const double variation    = 0.250 * originalInterval;
+   const double nextInterval = originalInterval - (variation / 2.0) +
+                                  variation * ((double)rand() / (double)RAND_MAX);
+   return((unsigned long long)nextInterval);
+}
+
+
 /* ###### Write new log line ############################################# */
 static void newLogLine(FILE* fh)
 {
    printTimeStamp(fh);
    if(RunID) {
-      fprintf(fh, "[%s] ", RunID);
+      if(InfoString[0] != 0x00) {
+         fprintf(fh, "[%s@%s] ", RunID, InfoString);
+      }
+      else {
+         fprintf(fh, "[%s] ", RunID);
+      }
    }
 }
 
@@ -106,12 +126,13 @@ static unsigned int performUpload(int sd)
          upload.Header.Length = htons(dataLength + sizeof(struct ScriptingCommonHeader));
          sent = rsp_sendmsg(sd, (const char*)&upload, dataLength + sizeof(struct ScriptingCommonHeader), 0,
                             0, htonl(PPID_SP), 0, 0, 0, (int)(TransmitTimeout / 1000));
-         if(sent <= 0) {
+         if(sent != (ssize_t)dataLength + (ssize_t)sizeof(struct ScriptingCommonHeader)) {
             newLogLine(stdout);
             printf("Upload error: %s\n", strerror(errno));
             fflush(stdout);
             return(SSCR_FAILOVER);
          }
+         
          if(dataLength == 0) {
             break;
          }
@@ -175,10 +196,11 @@ static unsigned int sendKeepAlive(int sd)
    keepAlive.Header.Length = htons(sizeof(keepAlive));
    sent = rsp_sendmsg(sd, (const char*)&keepAlive, sizeof(keepAlive), 0,
                       0, htonl(PPID_SP), 0, 0, 0, 0);
-   if(sent <= 0) {
-      if(Status != SSCS_WAIT_READY) {
+   if(sent != sizeof(keepAlive)) {
+      if((State != SSCS_WAIT_SERVER_READY) &&
+         (State != SSCS_WAIT_START_PROCESSING)) {
          newLogLine(stdout);
-         printf("Keep-Alive transmission error in state #%u: %s\n", Status, strerror(errno));
+         printf("Keep-Alive transmission error in state #%u: %s\n", State, strerror(errno));
          fflush(stdout);
       }
       else {
@@ -194,6 +216,115 @@ static unsigned int sendKeepAlive(int sd)
 }
 
 
+/* ###### Handle KeepAliveAck message #################################### */
+static unsigned int handleKeepAliveAck()
+{
+   KeepAliveTransmitted = false;
+   LastKeepAlive        = getMicroTime();
+   return(SSCR_OKAY);
+}
+
+
+/* ###### Send KeepAliveAck message ###################################### */
+static unsigned int sendKeepAliveAck(int sd)
+{
+   struct KeepAlive keepAliveAck;
+   ssize_t          sent;
+
+   keepAliveAck.Header.Type   = SPT_KEEPALIVE_ACK;
+   keepAliveAck.Header.Flags  = 0x00;
+   keepAliveAck.Header.Length = htons(sizeof(keepAliveAck));
+   sent = rsp_sendmsg(sd, (const char*)&keepAliveAck, sizeof(keepAliveAck), 0,
+                      0, htonl(PPID_SP), 0, 0, 0, 0);
+   return( (sent == sizeof(keepAliveAck)) ? SSCR_OKAY : SSCR_FAILOVER );
+}
+
+
+/* ###### Handle Ready message ########################################### */
+static void handleNotReady(const int              sd,
+                           const struct NotReady* notReady,
+                           const size_t           length)
+{
+   if(length >= sizeof(struct NotReady)) {
+      /* ====== Get info string ========================================== */
+      size_t i;
+      for(i = 0;i < (length - sizeof(struct NotReady));i++) {
+         if(i == SR_MAX_INFOSIZE) {
+            break;
+         }
+         InfoString[i] = isprint(notReady->Info[i]) ? notReady->Info[i] : '.';
+      }
+      InfoString[i] = 0x00;
+   }
+   else {
+      InfoString[0] = 0x00;
+   }
+   newLogLine(stdout);
+   printf("Server %s is not ready yet => trying another one.\n", InfoString);
+   fflush(stdout);
+}
+
+
+/* ###### Handle Ready message ########################################### */
+static unsigned int handleReady(const int           sd,
+                                const struct Ready* ready,
+                                const size_t        length)
+{
+   if(length < sizeof(struct Ready)) {
+      newLogLine(stdout);
+      puts("Received invalid Ready message!");
+      fflush(stdout);
+      return(SSCR_FAILOVER);
+   }
+   
+   /* ====== Get info string ============================================= */
+   size_t i;
+   for(i = 0;i < (length - sizeof(struct Ready));i++) {
+      if(i == SR_MAX_INFOSIZE) {
+         break;
+      }
+      InfoString[i] = isprint(ready->Info[i]) ? ready->Info[i] : '.';
+   }
+   InfoString[i] = 0x00;
+   HasAssignedPE = true;
+
+   /* ====== Proceed with uploading the work package ===================== */
+   State = SSCS_WAIT_START_PROCESSING;
+   newLogLine(stdout);
+   printf("Server %s is ready!\n", InfoString);
+   fflush(stdout);
+   return(performUpload(sd));
+}
+
+
+/* ###### Handle Status message ########################################## */
+static unsigned int serverStartsProcessing(const int            sd,
+                                           const struct Status* status,
+                                           const size_t         length)
+{
+   if(length < sizeof(struct Status)) {
+      newLogLine(stdout);
+      puts("Received invalid Status message!");
+      fflush(stdout);
+      return(SSCR_FAILOVER);
+   }
+   
+   if(ntohl(status->Status) == 0) {
+      State = SSCS_PROCESSING;
+      newLogLine(stdout);
+      printf("Server %s is processing ...\n", InfoString);
+      fflush(stdout);
+   }
+   else {
+      State = SSCR_FAILOVER;
+      newLogLine(stdout);
+      printf("Server did not accept upload; status was %u\n", ntohl(status->Status));
+      fflush(stdout);
+   }
+   return(SSCR_OKAY);
+}
+
+
 /* ###### Handle Download message ######################################## */
 static unsigned int handleDownload(const struct Download* download,
                                    const size_t           length)
@@ -245,41 +376,63 @@ static unsigned int handleMessage(int                                 sd,
                                   const uint32_t                      ppid)
 {
    /* ====== Check message header ======================================== */
-   if( (ppid != PPID_SP) ||
-       (length < sizeof(struct ScriptingCommonHeader)) ) {
+   if(ppid != PPID_SP) {
       newLogLine(stdout);
-      puts("Received invalid message!");
+      printf("Received message has wrong PPID $%08x!\n", ppid);
+      fflush(stdout);
+      return(SSCR_FAILOVER);
+   }
+   if(length < sizeof(struct ScriptingCommonHeader)) {
+      newLogLine(stdout);
+      printf("Received message of %u bytes does not even contain header!\n",
+             (unsigned int)length);
+      fflush(stdout);
+      return(SSCR_FAILOVER);
+   }
+   if(ntohs(header->Length) != length) {
+      newLogLine(stdout);
+      printf("Received message has %u bytes but %u bytes are expected!\n",
+             (unsigned int)length, ntohs(header->Length));
       fflush(stdout);
       return(SSCR_FAILOVER);
    }
 
-
    /* ====== Handle message ============================================== */
-   switch(Status) {
+   switch(State) {
 
-      /* ====== Wait for Ready message =================================== */
-      case SSCS_WAIT_READY:
+      /* ====== Wait for Ready message to start upload =================== */
+      case SSCS_WAIT_SERVER_READY:
          switch(header->Type) {
             case SPT_READY:
-               Status = SSCS_PROCESSING;
-               newLogLine(stdout);
-               puts("Server is ready");
-               fflush(stdout);
-               return(performUpload(sd));
+               return(handleReady(sd, (const struct Ready*)header, length));
+             break;
+            case SPT_NOTREADY:
+               handleNotReady(sd, (const struct NotReady*)header, length);
+               return(SSCR_FAILOVER);
              break;
          }
         break;
 
+      /* ====== Wait for Ready message to start upload =================== */
+      case SSCS_WAIT_START_PROCESSING:
+         switch(header->Type) {
+            case SPT_STATUS:
+               return(serverStartsProcessing(sd, (const struct Status*)header, length));
+             break;
+         }
+        break;
+        
       /* ====== Processing =============================================== */
       case SSCS_PROCESSING:
          switch(header->Type) {
             case SPT_KEEPALIVE_ACK:
-               KeepAliveTransmitted = false;
-               LastKeepAlive        = getMicroTime();
-               return(SSCR_OKAY);
+             return(handleKeepAliveAck());
+             break;
+            case SPT_KEEPALIVE:
+               return(sendKeepAliveAck(sd));
              break;
             case SPT_STATUS:
-               Status = SSCS_DOWNLOAD;
+               State = SSCS_DOWNLOAD;
                return(handleStatus((const struct Status*)header, length));
              break;
          }
@@ -292,9 +445,7 @@ static unsigned int handleMessage(int                                 sd,
                return(handleDownload((const struct Download*)header, length));
              break;
             case SPT_KEEPALIVE_ACK:
-               KeepAliveTransmitted = false;
-               LastKeepAlive        = getMicroTime();
-               return(SSCR_OKAY);
+               return(handleKeepAliveAck());
              break;
          }
        break;
@@ -304,7 +455,7 @@ static unsigned int handleMessage(int                                 sd,
    /* ====== Unexpected message ========================================== */
    newLogLine(stdout);
    printf("Unexpected message $%02x in state #%u (length=%u, ppid=$%08x)\n",
-          header->Type, Status, (unsigned int)length, ppid);
+          header->Type, State, (unsigned int)length, ppid);
    fflush(stdout);
    return(SSCR_FAILOVER);
 }
@@ -413,7 +564,7 @@ int main(int argc, char** argv)
    }
 
    newLogLine(stdout);
-   puts("Connecting to pool ...");
+   printf("Connecting to pool %s ...\n", poolHandle);
    fflush(stdout);
 
    if(rsp_connect(sd, (unsigned char*)poolHandle, strlen(poolHandle), 0) < 0) {
@@ -424,10 +575,10 @@ int main(int argc, char** argv)
 
    installBreakDetector();
    result = SSCR_OKAY;
+   KeepAliveTransmitted = false;
+   LastKeepAlive        = getMicroTime();
    while( (!breakDetected()) && (result != SSCR_COMPLETE) ) {
-      KeepAliveTransmitted = false;
-      LastKeepAlive        = getMicroTime();
-      result               = SSCR_OKAY;
+      result = SSCR_OKAY;
 
       /* ====== Wait for results and regularly send keep-alives ======= */
       nextTimer   = LastKeepAlive + (KeepAliveTransmitted ? KeepAliveTimeout : KeepAliveInterval);
@@ -435,7 +586,7 @@ int main(int argc, char** argv)
       ufds.events = POLLIN;
       now         = getMicroTime();
       events = rsp_poll(&ufds, 1,
-                        (nextTimer <= now) ? 0 : (int)((nextTimer - now) / 1000));
+                        (nextTimer <= now) ? 0 : (int)ceil((double)(nextTimer - now) / 1000.0));
 
       /* ====== Handle results ===================================== */
       if( (events > 0) && (ufds.revents & POLLIN) ) {
@@ -445,13 +596,15 @@ int main(int argc, char** argv)
          if(received > 0) {
 
             /* ====== Notification ================================= */
-            if(flags & MSG_RSERPOOL_NOTIFICATION) {
+            if(flags & MSG_RSERPOOL_NOTIFICATION) {             
                notification = (union rserpool_notification*)&buffer;
-               newLogLine(stdout);
-               printf("\x1b[39;47mNotification: ");
-               rsp_print_notification(notification, stdout);
-               puts("\x1b[0m");
-               fflush(stdout);
+               if(HasAssignedPE == true) {   /* Skip printing if no PE has accepted the request yet. */
+                  newLogLine(stdout);
+                  printf("\x1b[39;47mNotification: ");
+                  rsp_print_notification(notification, stdout);
+                  puts("\x1b[0m");
+                  fflush(stdout);
+               }
                if((notification->rn_header.rn_type == RSERPOOL_FAILOVER) &&
                   (notification->rn_failover.rf_state == RSERPOOL_FAILOVER_NECESSARY)) {
                   result = SSCR_FAILOVER;
@@ -468,7 +621,7 @@ int main(int argc, char** argv)
       }
 
       /* ====== Handle timers ====================================== */
-      if(nextTimer <= getMicroTime()) {
+      if( (result == SSCR_OKAY) && (nextTimer <= getMicroTime()) ) {
          /* ====== KeepAlive transmission ========================== */
          if(!KeepAliveTransmitted) {
             result = sendKeepAlive(sd);
@@ -489,16 +642,21 @@ int main(int argc, char** argv)
             fclose(OutputFile);
             OutputFile = NULL;
          }
-         newLogLine(stdout);
-         puts("FAILOVER ...");
-         fflush(stdout);
-         nextTimer = getMicroTime() + RetryDelay;
+         unlink(OutputName);
+         if(HasAssignedPE == true) {   /* Skip printing if no PE has accepted the request yet. */
+            newLogLine(stdout);
+            puts("FAILOVER ...");
+            fflush(stdout);
+         }
+         InfoString[0] = 0x00;
+         nextTimer = getMicroTime() + randomizeWaitingTime(RetryDelay);
          do {
             usleep(500000);
          } while( (getMicroTime() < nextTimer) && (!breakDetected()) );
-         unlink(OutputName);
-         Status = SSCS_WAIT_READY;
+         State = SSCS_WAIT_SERVER_READY;
          rsp_forcefailover(sd, FFF_NONE, 0);
+         KeepAliveTransmitted = false;
+         LastKeepAlive        = getMicroTime();
       }
    }
 
