@@ -1,4 +1,4 @@
-/* $Id: scriptingservice.h 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: scriptingservice.h 2260 2009-10-25 14:57:10Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -44,7 +44,10 @@ class ScriptingServer : public TCPLikeServer
    struct ScriptingServerSettings
    {
       unsigned int TransmitTimeout;
+      unsigned int KeepAliveInterval;
+      unsigned int KeepAliveTimeout;
       bool         KeepTempDirs;
+      bool         VerboseMode;
    };
 
    ScriptingServer(int                      rserpoolSocketDescriptor,
@@ -53,6 +56,7 @@ class ScriptingServer : public TCPLikeServer
 
    static TCPLikeServer* scriptingServerFactory(int sd, void* userData, const uint32_t peIdentifier);
    static void scriptingPrintParameters(const void* userData);
+   static void rejectNewSession(int sd);
 
    protected:
    virtual bool start();
@@ -71,6 +75,8 @@ class ScriptingServer : public TCPLikeServer
    EventHandlingResult performDownload();
    EventHandlingResult handleUploadMessage(const char* buffer,
                                            size_t      bufferSize);
+   EventHandlingResult sendKeepAliveMessage();
+   EventHandlingResult handleKeepAliveAckMessage();
    EventHandlingResult handleKeepAliveMessage();
 
    enum ScriptingState {
@@ -87,6 +93,8 @@ class ScriptingServer : public TCPLikeServer
    char                    StatusName[256];
    FILE*                   UploadFile;
    pid_t                   ChildProcess;
+   bool                    WaitingForKeepAliveAck;
+   unsigned long long      LastKeepAliveTimeStamp;
 };
 
 
