@@ -1,4 +1,4 @@
-/* $Id: server.cc 2122 2009-05-04 07:50:14Z dreibh $
+/* $Id: server.cc 2260 2009-10-25 14:57:10Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -31,6 +31,7 @@
 #include "breakdetector.h"
 #include "tagitem.h"
 #include "netutilities.h"
+#include "stringutilities.h"
 #include "randomizer.h"
 #include "standardservices.h"
 #include "fractalgeneratorservice.h"
@@ -57,45 +58,35 @@
 
 
 /* ###### Count the number of CPUs/cores ################################# */
-size_t getNumberOfCPUs()
+static size_t getNumberOfCPUs()
 {
    const size_t numberOfCPUs = sysconf(_SC_NPROCESSORS_ONLN);
-/*
-#ifdef __linux__
-   char         buffer[258];
-   unsigned int id;
-   FILE*        fh = fopen("/proc/cpuinfo", "r");
-   if(fh != NULL) {
-
-      while( fgets((char*)&buffer, sizeof(buffer) - 1, fh) != NULL ) {
-         if(sscanf(buffer, "processor\t: %u", &id) == 1) {
-            numberOfCPUs++;
-         }
-      }
-   }
-#endif
-*/
    return( (numberOfCPUs > 0) ? numberOfCPUs : 1 );
 }
 
 
+#define MAX_PE_TRANSPORTADDRESSES 64
+
 /* ###### Main program ################################################### */
 int main(int argc, char** argv)
 {
    struct rsp_info     info;
    struct rsp_loadinfo loadInfo;
    struct TagItem      tags[8];
+   char                localAddressBuffer[sizeof(union sockaddr_union) * MAX_PE_TRANSPORTADDRESSES];
+   struct sockaddr*    localAddressSet = (struct sockaddr*)&localAddressBuffer;
+   size_t              localAddresses  = 0;
+   unsigned int        reregInterval   = 30000;
+   unsigned int        runtimeLimit    = 0;
+   unsigned int        service         = SERVICE_ECHO;
+   const char*         poolHandle      = NULL;
+   const char*         daemonPIDFile   = NULL;
+   bool                policyChanged   = false;
+   bool                quiet           = false;
+   double              uptime          = 0.0;
+   double              downtime        = 0.0;
    double              degradation;
    unsigned int        identifier;
-   unsigned int        reregInterval = 30000;
-   unsigned int        runtimeLimit  = 0;
-   unsigned int        service       = SERVICE_ECHO;
-   const char*         poolHandle    = NULL;
-   const char*         daemonPIDFile = NULL;
-   bool                policyChanged = false;
-   bool                quiet         = false;
-   double              uptime        = 0.0;
-   double              downtime      = 0.0;
 
 start:
    /* ====== Read parameters ============================================= */
@@ -104,7 +95,7 @@ start:
    tags[0].Data = 0;
    tags[1].Tag  = TAG_DONE;
 #ifdef ENABLE_CSP
-   info.ri_csp_identifier = CID_COMPOUND(CID_GROUP_POOLELEMENT, 0);
+   info.ri_csp_identifier = CID_COMPOUND(CID_GROUP_POOLELEMENT, random32());
 #endif
    memset(&loadInfo, 0, sizeof(loadInfo));
    loadInfo.rli_policy = PPT_ROUNDROBIN;
@@ -124,6 +115,30 @@ start:
          info.ri_csp_identifier = CID_COMPOUND(CID_GROUP_POOLELEMENT, tags[0].Data);
 #endif
       }
+      else if(!(strncmp(argv[i], "-local=" ,7))) {
+         localAddresses = 0;
+         sockaddr* addressPtr    = localAddressSet;
+         char*     addressString = (char*)&argv[i][7];
+         while(localAddresses < MAX_PE_TRANSPORTADDRESSES) {
+            char* idx = strindex(addressString, ',');
+            if(idx) {
+               *idx = 0x00;
+            }
+            if(!string2address(addressString, (union sockaddr_union*)addressPtr)) {
+               fprintf(stderr, "ERROR: Bad local address %s! Use format <address:port>.\n", addressString);
+               exit(1);
+            }
+            localAddresses++;
+            if(idx) {
+               addressString = idx;
+               addressString++;
+            }
+            else {
+               break;
+            }
+            addressPtr = (struct sockaddr*)((long)addressPtr + (long)getSocklen(addressPtr));
+         }
+      }
       else if(!(strncmp(argv[i], "-poolhandle=" ,12))) {
          poolHandle = (const char*)&argv[i][12];
       }
@@ -305,6 +320,9 @@ start:
          }
          exit(0);
       }
+      else {
+         signal(SIGHUP, SIG_IGN);
+      }
    }
 
 
@@ -314,7 +332,8 @@ start:
       echoServer.poolElement("Echo Server - Version 1.0",
                              (poolHandle != NULL) ? poolHandle : "EchoPool",
                              &info, &loadInfo,
-                             reregInterval, runtimeLimit, quiet,
+                             localAddressSet, localAddresses,
+                             reregInterval, runtimeLimit, quiet, (daemonPIDFile != NULL),
                              (struct TagItem*)&tags);
    }
    else if(service == SERVICE_DISCARD) {
@@ -322,7 +341,8 @@ start:
       discardServer.poolElement("Discard Server - Version 1.0",
                                 (poolHandle != NULL) ? poolHandle : "DiscardPool",
                                 &info, &loadInfo,
-                                reregInterval, runtimeLimit, quiet,
+                                localAddressSet, localAddresses,
+                                reregInterval, runtimeLimit, quiet, (daemonPIDFile != NULL),
                                 (struct TagItem*)&tags);
    }
    else if(service == SERVICE_DAYTIME) {
@@ -330,7 +350,8 @@ start:
       daytimeServer.poolElement("Daytime Server - Version 1.0",
                                 (poolHandle != NULL) ? poolHandle : "DaytimePool",
                                 &info, &loadInfo,
-                                reregInterval, runtimeLimit, quiet,
+                                localAddressSet, localAddresses,
+                                reregInterval, runtimeLimit, quiet, (daemonPIDFile != NULL),
                                 (struct TagItem*)&tags);
    }
    else if(service == SERVICE_CHARGEN) {
@@ -346,8 +367,9 @@ start:
                                   &info, &loadInfo,
                                   maxThreads,
                                   CharGenServer::charGenServerFactory,
-                                  NULL, NULL, NULL, NULL, NULL,
-                                  reregInterval, runtimeLimit, quiet,
+                                  NULL, NULL, NULL, NULL, NULL, NULL,
+                                  localAddressSet, localAddresses,
+                                  reregInterval, runtimeLimit, quiet, (daemonPIDFile != NULL),
                                   (struct TagItem*)&tags);
    }
    else if(service == SERVICE_PINGPONG) {
@@ -368,9 +390,10 @@ start:
                                  &info, &loadInfo,
                                  maxThreads,
                                  PingPongServer::pingPongServerFactory,
-                                 NULL, NULL, NULL, NULL,
+                                 NULL, NULL, NULL, NULL, NULL,
                                  (void*)&settings,
-                                 reregInterval, runtimeLimit, quiet,
+                                 localAddressSet, localAddresses,
+                                 reregInterval, runtimeLimit, quiet, (daemonPIDFile != NULL),
                                  (struct TagItem*)&tags);
    }
    else if(service == SERVICE_FRACTAL) {
@@ -421,9 +444,10 @@ start:
                                  maxThreads,
                                  FractalGeneratorServer::fractalGeneratorServerFactory,
                                  FractalGeneratorServer::fractalGeneratorPrintParameters,
-                                 NULL, NULL, NULL,
+                                 NULL, NULL, NULL, NULL,
                                  (void*)&settings,
-                                 reregInterval, runtimeLimit, quiet,
+                                 localAddressSet, localAddresses,
+                                 reregInterval, runtimeLimit, quiet, (daemonPIDFile != NULL),
                                  (struct TagItem*)&tags);
    }
    else if(service == SERVICE_CALCAPP) {
@@ -504,15 +528,19 @@ start:
       calcAppServer.poolElement("CalcApp Server - Version 1.0",
                                 (poolHandle != NULL) ? poolHandle : "CalcAppPool",
                                 &info, &loadInfo,
-                                reregInterval, runtimeLimit, quiet,
+                                localAddressSet, localAddresses,
+                                reregInterval, runtimeLimit, quiet, (daemonPIDFile != NULL),
                                 (struct TagItem*)&tags);
       resetStatistics = false;
    }
    else if(service == SERVICE_SCRIPTING) {
       size_t maxThreads = getNumberOfCPUs();
       ScriptingServer::ScriptingServerSettings settings;
-      settings.KeepTempDirs    = false;
-      settings.TransmitTimeout = 60000;
+      settings.KeepTempDirs      = false;
+      settings.VerboseMode       = false;
+      settings.TransmitTimeout   = 30000;
+      settings.KeepAliveInterval = 10000;
+      settings.KeepAliveTimeout  = 10000;
       for(int i = 1;i < argc;i++) {
          if(!(strncmp(argv[i], "-ssmaxthreads=", 14))) {
             maxThreads = atol((const char*)&argv[i][14]);
@@ -523,9 +551,18 @@ start:
          else if(!(strcmp(argv[i], "-sskeeptempdirs"))) {
             settings.KeepTempDirs = true;
          }
+         else if(!(strcmp(argv[i], "-ssverbose"))) {
+            settings.VerboseMode = true;
+         }
          else if(!(strncmp(argv[i], "-sstransmittimeout=", 19))) {
             settings.TransmitTimeout = atol((const char*)&argv[i][19]);
          }
+         else if(!(strncmp(argv[i], "-sskeepaliveinterval=", 21))) {
+            settings.KeepAliveInterval = atol((const char*)&argv[i][21]);
+         }
+         else if(!(strncmp(argv[i], "-sskeepalivetimeout=", 20))) {
+            settings.KeepAliveTimeout = atol((const char*)&argv[i][20]);
+         }
       }
       if(!policyChanged) {
          loadInfo.rli_load_degradation = (unsigned int)rint((1.0 / maxThreads) * (double)PPV_MAX_LOAD_DEGRADATION);
@@ -538,9 +575,11 @@ start:
                                  maxThreads,
                                  ScriptingServer::scriptingServerFactory,
                                  ScriptingServer::scriptingPrintParameters,
+                                 ScriptingServer::rejectNewSession,
                                  NULL, NULL, NULL,
                                  (void*)&settings,
-                                 reregInterval, runtimeLimit, quiet,
+                                 localAddressSet, localAddresses,
+                                 reregInterval, runtimeLimit, quiet, (daemonPIDFile != NULL),
                                  (struct TagItem*)&tags);
    }
 
