@@ -1,4 +1,4 @@
-/* $Id: sessioncontrol.c 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: sessioncontrol.c 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -431,7 +431,8 @@ static void handleShutdown(struct RSerPoolSocket*            rserpoolSocket,
 
 
 /* ###### Handle SCTP notification ####################################### */
-void handleNotification(struct RSerPoolSocket*         rserpoolSocket,
+/* Returns true, if association has been disconnected; false otherwise. */
+bool handleNotification(struct RSerPoolSocket*         rserpoolSocket,
                         const union sctp_notification* notification)
 {
 
@@ -450,6 +451,7 @@ void handleNotification(struct RSerPoolSocket*         rserpoolSocket,
             case SCTP_COMM_LOST:
             case SCTP_SHUTDOWN_COMP:
                handleCommLost(rserpoolSocket, &notification->sn_assoc_change);
+               return(true);
              break;
          }
       }
@@ -457,6 +459,14 @@ void handleNotification(struct RSerPoolSocket*         rserpoolSocket,
          handleShutdown(rserpoolSocket, &notification->sn_shutdown_event);
       }
    }
+   else {
+      if( (notification->sn_header.sn_type == SCTP_ASSOC_CHANGE) &&
+          ( (notification->sn_assoc_change.sac_state == SCTP_SHUTDOWN_COMP) ||
+            (notification->sn_assoc_change.sac_state == SCTP_COMM_LOST) ) ) {
+         return(true);
+      }
+   }
+   return(false);
 }
 
 
