@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: standardservices.h 2076 2008-12-28 15:52:34Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: standardservices.h 1391 2007-05-23 07:27:45Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef STANDARDSERVICES_H
@@ -83,7 +87,7 @@ class CharGenServer : public TCPLikeServer
    CharGenServer(int rserpoolSocketDescriptor);
    ~CharGenServer();
 
-   static TCPLikeServer* charGenServerFactory(int sd, void* userData);
+   static TCPLikeServer* charGenServerFactory(int sd, void* userData, const uint32_t peIdentifier);
 
    protected:
    virtual EventHandlingResult initializeSession();
@@ -102,7 +106,7 @@ class PingPongServer : public TCPLikeServer
                   PingPongServer::PingPongServerSettings* settings);
    ~PingPongServer();
 
-   static TCPLikeServer* pingPongServerFactory(int sd, void* userData);
+   static TCPLikeServer* pingPongServerFactory(int sd, void* userData, const uint32_t peIdentifier);
 
    protected:
    EventHandlingResult handleCookieEcho(const char* buffer, size_t bufferSize);
