@@ -1,4 +1,4 @@
-/* $Id: tcplikeserver.cc 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: tcplikeserver.cc 2258 2009-10-25 14:54:45Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -49,8 +49,9 @@ TCPLikeServer::TCPLikeServer(int rserpoolSocketDescriptor)
    Load                     = 0;
    SyncTimerTimeStamp       = 0;
    AsyncTimerTimeStamp      = 0;
-   printTimeStamp(stdout);
-   printf("New thread for RSerPool socket %d.\n", RSerPoolSocketDescriptor);
+   printTimeStamp(stdlog);
+   fprintf(stdlog, "New thread for RSerPool socket %d.\n", RSerPoolSocketDescriptor);
+   fflush(stdlog);
 }
 
 
@@ -58,8 +59,9 @@ TCPLikeServer::TCPLikeServer(int rserpoolSocketDescriptor)
 TCPLikeServer::~TCPLikeServer()
 {
    CHECK(ServerList == NULL);
-   printTimeStamp(stdout);
-   printf("Thread for RSerPool socket %d has been stopped.\n", RSerPoolSocketDescriptor);
+   printTimeStamp(stdlog);
+   fprintf(stdlog, "Thread for RSerPool socket %d has been stopped.\n", RSerPoolSocketDescriptor);
+   fflush(stdlog);
    if(RSerPoolSocketDescriptor >= 0) {
       rsp_close(RSerPoolSocketDescriptor);
       RSerPoolSocketDescriptor = -1;
@@ -83,7 +85,9 @@ void TCPLikeServer::finishSession(EventHandlingResult result)
 // ###### Shutdown thread ###################################################
 void TCPLikeServer::shutdown()
 {
+   lock();
    Shutdown = true;
+   unlock();
 }
 
 
@@ -97,26 +101,34 @@ double TCPLikeServer::getLoad() const
 // ##### Set load ###########################################################
 void TCPLikeServer::setLoad(double load)
 {
+   ServerList->lock();
+   
    CHECK(ServerList != NULL);
    CHECK(ServerList->LoadSum >= Load);
    if((load < 0.0) || (load > 1.0)) {
-      fputs("ERROR: Invalid load setting!\n", stderr);
+      fputs("ERROR: Invalid load setting!\n", stdlog);
+      fflush(stdlog);
+      ServerList->unlock();
       return;
    }
    const unsigned int newLoad = (unsigned int)floor(load * (double)PPV_MAX_LOAD);
    if((long long)ServerList->LoadSum - (long long)Load + (long long)newLoad > PPV_MAX_LOAD) {
-      fputs("ERROR: Something is wrong with load settings. Total load would exceed 100%!\n", stderr);
+      fputs("ERROR: Something is wrong with load settings. Total load would exceed 100%!\n", stdlog);
+      fflush(stdlog);
+      ServerList->unlock();
       return;
    }
 
-   ServerList->lock();
    const double oldTotalLoad = ServerList->getTotalLoad();
 
    ServerList->LoadSum -= Load;
+   lock();
    Load = newLoad;
+   unlock();
    ServerList->LoadSum += Load;
 
    const double newTotalLoad = ServerList->getTotalLoad();
+
    ServerList->unlock();
 
    if(oldTotalLoad != newTotalLoad) {
@@ -129,8 +141,9 @@ void TCPLikeServer::setLoad(double load)
 EventHandlingResult TCPLikeServer::handleCookieEcho(const char* buffer,
                                                     size_t      bufferSize)
 {
-   printTimeStamp(stdout);
-   puts("COOKIE ECHO");
+   printTimeStamp(stdlog);
+   fputs("COOKIE ECHO\n", stdlog);
+   fflush(stdout);
    return(EHR_Okay);
 }
 
@@ -139,10 +152,11 @@ EventHandlingResult TCPLikeServer::handleCookieEcho(const char* buffer,
 EventHandlingResult TCPLikeServer::handleNotification(
                        const union rserpool_notification* notification)
 {
-   printTimeStamp(stdout);
-   printf("NOTIFICATION: ");
-   rsp_print_notification(notification, stdout);
-   puts("");
+   printTimeStamp(stdlog);
+   fprintf(stdlog, "NOTIFICATION: ");
+   rsp_print_notification(notification, stdlog);
+   fputs("\n", stdlog);
+   fflush(stdout);
    return(EHR_Okay);
 }
 
@@ -150,8 +164,9 @@ EventHandlingResult TCPLikeServer::handleNotification(
 // ###### Handle synchronous timer ##########################################
 EventHandlingResult TCPLikeServer::syncTimerEvent(const unsigned long long now)
 {
-   printTimeStamp(stdout);
-   puts("SyncTimer");
+   printTimeStamp(stdlog);
+   fputs("SyncTimer\n", stdlog);
+   fflush(stdout);
    return(EHR_Okay);
 }
 
@@ -159,8 +174,9 @@ EventHandlingResult TCPLikeServer::syncTimerEvent(const unsigned long long now)
 // ###### Handle asynchronous timer #########################################
 void TCPLikeServer::asyncTimerEvent(const unsigned long long now)
 {
-   printTimeStamp(stdout);
-   puts("AsyncTimer");
+   printTimeStamp(stdlog);
+   fputs("AsyncTimer\n", stdlog);
+   fflush(stdout);
 }
 
 
@@ -186,7 +202,7 @@ void TCPLikeServer::run()
 
    eventHandlingResult = initializeSession();
    if(eventHandlingResult == EHR_Okay) {
-      while(!Shutdown) {
+      while(!isShuttingDown()) {
          flags    = 0;
          unsigned long long nextTimerEvent;
          if(SyncTimerTimeStamp > 0) {
@@ -217,8 +233,9 @@ void TCPLikeServer::run()
                   eventHandlingResult = handleCookieEcho((char*)&buffer, received);
                }
                else {
-                  printTimeStamp(stdout);
-                  puts("Dropped unexpected ASAP COOKIE_ECHO!");
+                  printTimeStamp(stdlog);
+                  fputs("Dropped unexpected ASAP COOKIE_ECHO!\n", stdlog);
+                  fflush(stdlog);
                   eventHandlingResult = EHR_Abort;
                }
             }
@@ -266,7 +283,9 @@ void TCPLikeServer::run()
                   (eventHandlingResult == EHR_Abort) ? SCTP_ABORT : SCTP_EOF,
                   0);
    }
+   lock();
    Finished = true;
+   unlock();
 }
 
 
@@ -278,232 +297,270 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                                 size_t               maxThreads,
                                 TCPLikeServer*       (*threadFactory)(int sd, void* userData, const uint32_t peIdentifier),
                                 void                 (*printParameters)(const void* userData),
+                                void                 (*rejectNewSession)(int sd),
                                 bool                 (*initializeService)(void* userData),
                                 void                 (*finishService)(void* userData),
                                 double               (*loadUpdateHook)(const double load),
                                 void*                userData,
+                                const sockaddr*      localAddressSet,
+                                const size_t         localAddresses,
                                 unsigned int         reregInterval,
                                 unsigned int         runtimeLimit,
                                 const bool           quiet,
+                                const bool           daemonMode, 
                                 struct TagItem*      tags)
 {
    if(rsp_initialize(info) < 0) {
-      fputs("ERROR: Unable to initialize rsplib Library!\n", stderr);
+      fputs("ERROR: Unable to initialize rsplib Library!\n", stdlog);
       return;
    }
 
    int rserpoolSocket = rsp_socket(0, SOCK_STREAM, IPPROTO_SCTP);
-   if( (rserpoolSocket >= 0) &&
-       (rsp_listen(rserpoolSocket, (int)maxThreads) == 0) ) {
-      // ====== Initialize notification pipe ================================
-      int systemNotificationPipe[2];
-      int rspNotificationPipe[2];
-      if(ext_pipe((int*)&systemNotificationPipe) == 0) {
-         rspNotificationPipe[0] = rsp_mapsocket(systemNotificationPipe[0], -1);
-         rspNotificationPipe[1] = rsp_mapsocket(systemNotificationPipe[1], -1);
-         if( (setNonBlocking(systemNotificationPipe[0])) && (rspNotificationPipe[0] > 0) &&
-             (setNonBlocking(systemNotificationPipe[1])) && (rspNotificationPipe[1] > 0) ) {
+   if(rserpoolSocket >= 0) {
+      if(rsp_bind(rserpoolSocket, localAddressSet, localAddresses) == 0) {
+         if(rsp_listen(rserpoolSocket, (int)maxThreads) == 0) {
+            // ====== Initialize notification pipe ================================
+            int systemNotificationPipe[2];
+            int rspNotificationPipe[2];
+            if(ext_pipe((int*)&systemNotificationPipe) == 0) {
+               rspNotificationPipe[0] = rsp_mapsocket(systemNotificationPipe[0], -1);
+               rspNotificationPipe[1] = rsp_mapsocket(systemNotificationPipe[1], -1);
+               if( (setNonBlocking(systemNotificationPipe[0])) && (rspNotificationPipe[0] > 0) &&
+                   (setNonBlocking(systemNotificationPipe[1])) && (rspNotificationPipe[1] > 0) ) {
 
-            // ====== Initialize PE settings ================================
-            struct rsp_loadinfo dummyLoadinfo;
-            if(loadinfo == NULL) {
-               memset(&dummyLoadinfo, 0, sizeof(dummyLoadinfo));
-               loadinfo = &dummyLoadinfo;
-               loadinfo->rli_policy = PPT_ROUNDROBIN;
-            }
-
-            // ====== Server startup ========================================
-            if((initializeService == NULL) || (initializeService(userData))) {
-               double oldLoad = 0.0;
-               if(PPT_IS_ADAPTIVE(loadinfo->rli_policy)) {
-                  if(loadUpdateHook) {
-                     double newLoad = loadUpdateHook(0.0);
-                     CHECK((newLoad >= 0.0) && (newLoad <= 1.0));
-                     loadinfo->rli_load = (unsigned int)rint(newLoad * (double)PPV_MAX_LOAD);
-                     oldLoad = newLoad;
-                  }
-               }
-
-               // ====== Print program title ================================
-               if(!quiet) {
-                  puts(programTitle);
-                  for(size_t i = 0;i < strlen(programTitle);i++) {
-                     printf("=");
-                  }
-                  const char* policyName = rsp_getpolicybytype(loadinfo->rli_policy);
-                  puts("\n\nGeneral Parameters:");
-                  printf("   Pool Handle             = %s\n", poolHandle);
-                  printf("   Reregistration Interval = %1.3fs\n", reregInterval / 1000.0);
-                  printf("   Runtime Limit           = ");
-                  if(runtimeLimit > 0) {
-                     printf("%1.3f [s]\n", runtimeLimit / 1000.0);
-                  }
-                  else {
-                     puts("off");
-                  }
-                  printf("   Max Threads             = %u\n", (unsigned int)maxThreads);
-                  puts("   Policy Settings");
-                  printf("      Policy Type          = %s\n", (policyName != NULL) ? policyName : "?");
-                  printf("      Load Degradation     = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_load_degradation / (double)PPV_MAX_LOAD_DEGRADATION));
-                  printf("      Load DPF             = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_load_dpf / (double)PPV_MAX_LOADDPF));
-                  printf("      Weight               = %u\n", loadinfo->rli_weight);
-                  printf("      Weight DPF           = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_weight_dpf / (double)PPV_MAX_WEIGHTDPF));
-                  if(printParameters) {
-                     printParameters(userData);
-                  }
-               }
-
-               // ====== Register PE ========================================
-               if(rsp_register_tags(rserpoolSocket,
-                                    (const unsigned char*)poolHandle, strlen(poolHandle),
-                                    loadinfo, reregInterval, 0, tags) == 0) {
-                  uint32_t identifier = 0;
-                  if(rsp_getsockname(rserpoolSocket, NULL, NULL, &identifier) == 0) {
-                     if(!quiet) {
-                        puts("Registration");
-                        printf("   Identifier              = $%08x\n\n", identifier);
-                     }
+                  // ====== Initialize PE settings ================================
+                  struct rsp_loadinfo dummyLoadinfo;
+                  if(loadinfo == NULL) {
+                     memset(&dummyLoadinfo, 0, sizeof(dummyLoadinfo));
+                     loadinfo = &dummyLoadinfo;
+                     loadinfo->rli_policy = PPT_ROUNDROBIN;
                   }
 
-                  // ====== Main loop =======================================
-                  TCPLikeServerList        serverSet(maxThreads, systemNotificationPipe[1]);
-                  const unsigned long long autoStopTimeStamp =
-                     (runtimeLimit > 0) ? (getMicroTime() + (1000ULL * runtimeLimit)) : 0;
-                  installBreakDetector();
-                  while(!breakDetected()) {
-                     // ====== Wait for actions =============================
-                     struct pollfd pollfds[2];
-                     pollfds[0].fd      = rserpoolSocket;
-                     pollfds[0].events  = POLLIN;
-                     pollfds[0].revents = 0;
-                     pollfds[1].fd      = rspNotificationPipe[0];
-                     pollfds[1].events  = POLLIN;
-                     pollfds[1].revents = 0;
-                     const int result = rsp_poll((struct pollfd*)&pollfds, 2, 2500);
-
-                     // ====== Clean-up session list ========================
-                     // Possible, some sessions have finished working during
-                     // waiting time. Therefore, cleaning them up is useful
-                     // before accepting new sessions.
-                     if(serverSet.handleRemovalsAndTimers() > 0) {
-                        int backlog = (int)(serverSet.getMaxThreads() - serverSet.getThreads());
-                        if(rsp_listen(rserpoolSocket, 1 + backlog) < 0) {
-                           perror("Unable to update backlog using rsp_listen()");
+                  // ====== Server startup ========================================
+                  if((initializeService == NULL) || (initializeService(userData))) {
+                     double oldLoad = 0.0;
+                     if(PPT_IS_ADAPTIVE(loadinfo->rli_policy)) {
+                        if(loadUpdateHook) {
+                           double newLoad = loadUpdateHook(0.0);
+                           CHECK((newLoad >= 0.0) && (newLoad <= 1.0));
+                           loadinfo->rli_load = (unsigned int)rint(newLoad * (double)PPV_MAX_LOAD);
+                           oldLoad = newLoad;
                         }
                      }
 
-                     // ====== Wait for incoming sessions ===================
-                     if(result > 0) {
-                        if(pollfds[0].revents & POLLIN) {
-                            int newRSerPoolSocket = rsp_accept(rserpoolSocket, 0);
-                            if(newRSerPoolSocket >= 0) {
-                               if(serverSet.hasCapacity()) {
-                                  TCPLikeServer* serviceThread = threadFactory(newRSerPoolSocket, userData, identifier);
-                                  if(serviceThread) {
-                                     if(serverSet.add(serviceThread)) {
-                                        if(serviceThread->start()) {
-                                           int backlog = (int)(serverSet.getMaxThreads() - serverSet.getThreads());
-                                           if(rsp_listen(rserpoolSocket, 1 + backlog) < 0) {
-                                              perror("Unable to update backlog using rsp_listen()");
+                     // ====== Print program title ================================
+                     if(!quiet) {
+                        puts(programTitle);
+                        for(size_t i = 0;i < strlen(programTitle);i++) {
+                           printf("=");
+                        }
+                        const char* policyName = rsp_getpolicybytype(loadinfo->rli_policy);
+                        puts("\n\nGeneral Parameters:");
+                        printf("   Pool Handle             = %s\n", poolHandle);
+                        printf("   Reregistration Interval = %1.3fs\n", reregInterval / 1000.0);
+                        printf("   Local Addresses         = { ");
+                        if(localAddresses == 0) {
+                           printf("all");
+                        }
+                        else {
+                           const sockaddr* addressPtr = localAddressSet;
+                           for(size_t i = 0;i < localAddresses;i++) {
+                              if(i > 0) {
+                                 printf(", ");
+                              }
+                              fputaddress(addressPtr, (i == 0), stdout);
+                              addressPtr = (const sockaddr*)((long)addressPtr + (long)getSocklen(addressPtr));
+                           }
+                        }
+                        printf(" }\n");
+                        printf("   Runtime Limit           = ");
+                        if(runtimeLimit > 0) {
+                           printf("%1.3f [s]\n", runtimeLimit / 1000.0);
+                        }
+                        else {
+                           puts("off");
+                        }
+                        printf("   Max Threads             = %u\n", (unsigned int)maxThreads);
+                        puts("   Policy Settings");
+                        printf("      Policy Type          = %s\n", (policyName != NULL) ? policyName : "?");
+                        printf("      Load Degradation     = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_load_degradation / (double)PPV_MAX_LOAD_DEGRADATION));
+                        printf("      Load DPF             = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_load_dpf / (double)PPV_MAX_LOADDPF));
+                        printf("      Weight               = %u\n", loadinfo->rli_weight);
+                        printf("      Weight DPF           = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_weight_dpf / (double)PPV_MAX_WEIGHTDPF));
+                        if(printParameters) {
+                           printParameters(userData);
+                        }
+                     }
+
+                     // ====== Register PE ========================================
+                     if(rsp_register_tags(rserpoolSocket,
+                                          (const unsigned char*)poolHandle, strlen(poolHandle),
+                                          loadinfo, reregInterval,
+                                          (daemonMode == true) ? REGF_DAEMONMODE : 0,
+                                          tags) == 0) {
+                        uint32_t identifier = 0;
+                        if(rsp_getsockname(rserpoolSocket, NULL, NULL, &identifier) == 0) {
+                           if(!quiet) {
+                              puts("Registration:");
+                              printf("   Identifier              = $%08x\n\n", identifier);
+                           }
+                        }
+
+                        // ====== Main loop =======================================
+                        TCPLikeServerList        serverSet(maxThreads, systemNotificationPipe[1]);
+                        const unsigned long long autoStopTimeStamp =
+                           (runtimeLimit > 0) ? (getMicroTime() + (1000ULL * runtimeLimit)) : 0;
+                        installBreakDetector();
+                        while(!breakDetected()) {
+                           // ====== Wait for actions =============================
+                           struct pollfd pollfds[2];
+                           pollfds[0].fd      = rserpoolSocket;
+                           pollfds[0].events  = POLLIN;
+                           pollfds[0].revents = 0;
+                           pollfds[1].fd      = rspNotificationPipe[0];
+                           pollfds[1].events  = POLLIN;
+                           pollfds[1].revents = 0;
+                           const int result = rsp_poll((struct pollfd*)&pollfds, 2, 2500);
+
+                           // ====== Clean-up session list ========================
+                           // Possible, some sessions have finished working during
+                           // waiting time. Therefore, cleaning them up is useful
+                           // before accepting new sessions.
+                           if(serverSet.handleRemovalsAndTimers() > 0) {
+                              int backlog = (int)(serverSet.getMaxThreads() - serverSet.getThreads());
+                              if(rsp_listen(rserpoolSocket, 1 + backlog) < 0) {
+                                 logerror("Unable to update backlog using rsp_listen()");                                 
+                              }
+                           }
+
+                           // ====== Wait for incoming sessions ===================
+                           if(result > 0) {
+                              if(pollfds[0].revents & POLLIN) {
+                                  int newRSerPoolSocket = rsp_accept(rserpoolSocket, 0);
+                                  if(newRSerPoolSocket >= 0) {
+                                     if(serverSet.hasCapacity()) {
+                                        TCPLikeServer* serviceThread = threadFactory(newRSerPoolSocket, userData, identifier);
+                                        if(serviceThread) {
+                                           if(serverSet.add(serviceThread)) {
+                                              if(serviceThread->start()) {
+                                                 int backlog = (int)(serverSet.getMaxThreads() - serverSet.getThreads());
+                                                 if(rsp_listen(rserpoolSocket, 1 + backlog) < 0) {
+                                                    logerror("Unable to update backlog using rsp_listen()");
+                                                    fflush(stdlog);
+                                                 }
+                                              }
+                                              else {
+                                                 printTimeStamp(stdlog);
+                                                 fputs("ERROR: Unable to start up service thread\n", stderr);
+                                                 fflush(stdlog);
+                                                 serverSet.remove(serviceThread);
+                                                 delete serviceThread;
+                                              }
+                                           }
+                                           else {
+                                              delete serviceThread;
+                                              printTimeStamp(stdlog);
+                                              fputs("Unable to add new service thread\n", stdlog);
+                                              fflush(stdlog);
                                            }
                                         }
                                         else {
-                                           printTimeStamp(stderr);
-                                           fputs("ERROR: Unable to start up service thread\n", stderr);
-                                           serverSet.remove(serviceThread);
-                                           delete serviceThread;
+                                           rsp_close(newRSerPoolSocket);
+                                           printTimeStamp(stdlog);
+                                           fputs("Unable to create new service thread\n", stdlog);
+                                           fflush(stdlog);
                                         }
                                      }
                                      else {
-                                        delete serviceThread;
-                                        printTimeStamp(stderr);
-                                        puts("Unable to add new service thread");
+                                        if(rejectNewSession) {
+                                           rejectNewSession(newRSerPoolSocket);
+                                        }
+                                        rsp_close(newRSerPoolSocket);
+                                        printTimeStamp(stdlog);
+                                        fputs("Rejected new session, since server is fully loaded\n", stdlog);
+                                        fflush(stdlog);
                                      }
                                   }
-                                  else {
-                                     rsp_close(newRSerPoolSocket);
-                                     printTimeStamp(stderr);
-                                     puts("Unable to create new service thread");
-                                  }
-                               }
-                               else {
-                                  rsp_close(newRSerPoolSocket);
-                                  printTimeStamp(stdout);
-                                  puts("Rejected new session, since server is fully loaded");
-                               }
-                            }
-                        }
+                              }
+
+                              // ====== Do reregistration on load changes =========
+                              if(PPT_IS_ADAPTIVE(loadinfo->rli_policy)) {
+                                 double newLoad = serverSet.getTotalLoad();
+                                 if(loadUpdateHook) {
+                                    newLoad = loadUpdateHook(newLoad);
+                                    CHECK((newLoad >= 0.0) && (newLoad <= 1.0));
+                                 }
+                                 if(fabs(newLoad - oldLoad) >= 0.01) {
+                                    // printf("NEW LOAD = %1.6f\n", newLoad);
+                                    oldLoad = newLoad;
+                                    loadinfo->rli_load = (unsigned int)rint(newLoad * (double)PPV_MAX_LOAD);
+                                    rsp_register_tags(rserpoolSocket,
+                                                      (const unsigned char*)poolHandle, strlen(poolHandle),
+                                                      loadinfo, reregInterval, REGF_DONTWAIT,
+                                                      tags);
+                                 }
+                              }
+
+                              // ====== Clear notification pipe ===================
+                              if(pollfds[1].revents & POLLIN) {
+                                 char buffer[1024];
+                                 ext_read(systemNotificationPipe[0], (char*)&buffer, sizeof(buffer));
+                              }
 
-                        // ====== Do reregistration on load changes =========
-                        if(PPT_IS_ADAPTIVE(loadinfo->rli_policy)) {
-                           double newLoad = serverSet.getTotalLoad();
-                           if(loadUpdateHook) {
-                              newLoad = loadUpdateHook(newLoad);
-                              CHECK((newLoad >= 0.0) && (newLoad <= 1.0));
                            }
-                           if(fabs(newLoad - oldLoad) >= 0.01) {
-                              // printf("NEW LOAD = %1.6f\n", newLoad);
-                              oldLoad = newLoad;
-                              loadinfo->rli_load = (unsigned int)rint(newLoad * (double)PPV_MAX_LOAD);
-                              rsp_register_tags(rserpoolSocket,
-                                                (const unsigned char*)poolHandle, strlen(poolHandle),
-                                                loadinfo, reregInterval, REGF_DONTWAIT,
-                                                tags);
+
+                           // ====== Auto-stop ====================================
+                           if((autoStopTimeStamp > 0) &&
+                              (getMicroTime() > autoStopTimeStamp)) {
+                              fputs("Auto-stop reached!\n", stdlog);
+                              break;
                            }
                         }
 
-                        // ====== Clear notification pipe ===================
-                        if(pollfds[1].revents & POLLIN) {
-                           char buffer[1024];
-                           ext_read(systemNotificationPipe[0], (char*)&buffer, sizeof(buffer));
-                        }
-
+                        // ====== Clean up ========================================
+                        serverSet.removeAll();
+                        rsp_deregister(rserpoolSocket, 0);
+                        uninstallBreakDetector();
                      }
-
-                     // ====== Auto-stop ====================================
-                     if((autoStopTimeStamp > 0) &&
-                        (getMicroTime() > autoStopTimeStamp)) {
-                        puts("Auto-stop reached!");
-                        break;
+                     if(finishService != NULL) {
+                        finishService(userData);
                      }
                   }
-
-                  // ====== Clean up ========================================
-                  serverSet.removeAll();
-                  rsp_deregister(rserpoolSocket, 0);
-                  uninstallBreakDetector();
+                  else {
+                     fprintf(stdlog, "ERROR: Failed to register PE to pool %s\n", poolHandle);
+                  }
+                  if(rspNotificationPipe[0] > 0) {
+                     rsp_unmapsocket(rspNotificationPipe[0]);
+                  }
+                  if(rspNotificationPipe[1] > 0) {
+                     rsp_unmapsocket(rspNotificationPipe[1]);
+                  }
                }
-               if(finishService != NULL) {
-                  finishService(userData);
+               else {
+                  logerror("Unable to map notification pipe FDs");
                }
+               ext_close(systemNotificationPipe[0]);
+               ext_close(systemNotificationPipe[1]);
             }
             else {
-               fprintf(stderr, "ERROR: Failed to register PE to pool %s\n", poolHandle);
-            }
-            if(rspNotificationPipe[0] > 0) {
-               rsp_unmapsocket(rspNotificationPipe[0]);
-            }
-            if(rspNotificationPipe[1] > 0) {
-               rsp_unmapsocket(rspNotificationPipe[1]);
+               logerror("Unable to create notification pipe");
             }
          }
          else {
-            perror("Unable to map notification pipe FDs");
+            logerror("Unable to put RSerPool socket into listening mode");
          }
-         ext_close(systemNotificationPipe[0]);
-         ext_close(systemNotificationPipe[1]);
       }
       else {
-         perror("Unable to create notification pipe");
+         logerror("Unable to bind RSerPool socket");
       }
       rsp_close(rserpoolSocket);
    }
    else {
-      perror("Unable to create RSerPool socket");
+      logerror("Unable to create RSerPool socket");
    }
 
    rsp_cleanup();
-   puts("\nTerminated!");
+   fputs("\nTerminated!\n", stdlog);
 }
 
 
