@@ -1,4 +1,4 @@
-/* $Id: tcplikeserver.h 2086 2009-01-07 10:29:41Z dreibh $
+/* $Id: tcplikeserver.h 2240 2009-10-21 08:11:57Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -79,17 +79,27 @@ class TCPLikeServer : public TDThread
    TCPLikeServer(int rserpoolSocketDescriptor);
    ~TCPLikeServer();
 
-   inline bool hasFinished() const {
-      return(Finished);
+   inline bool hasFinished() {
+      lock();
+      const bool finished = Finished;
+      unlock();
+      return(finished);
    }
-   inline bool isShuttingDown() const {
-      return(Shutdown);
+   inline bool isShuttingDown() {
+      lock();
+      const bool shutdown = Shutdown;
+      unlock();
+      return(shutdown);
    }
    inline void setSyncTimer(const unsigned long long timeStamp) {
+      lock();
       SyncTimerTimeStamp = timeStamp;
+      unlock();
    }
    inline void setAsyncTimer(const unsigned long long timeStamp) {
+      lock();
       AsyncTimerTimeStamp = timeStamp;
+      unlock();
    }
    inline TCPLikeServerList* getServerList() const {
       return(ServerList);
@@ -109,14 +119,18 @@ class TCPLikeServer : public TDThread
                            size_t               maxThreads,
                            TCPLikeServer*       (*threadFactory)(int sd, void* userData, uint32_t peIdentifier),
                            void                 (*printParameters)(const void* userData),
+                           void                 (*rejectNewSession)(int sd),
                            bool                 (*initializeService)(void* userData),
                            void                 (*finishService)(void* userData),
                            double               (*loadUpdateHook)(const double load),
                            void*                userData,
-                           unsigned int         reregInterval = 30000,
-                           unsigned int         runtimeLimit  = 0,
-                           const bool           quiet         = false,
-                           struct TagItem*      tags          = NULL);
+                           const sockaddr*      localAddressSet = NULL,
+                           const size_t         localAddresses  = 0,
+                           unsigned int         reregInterval   = 30000,
+                           unsigned int         runtimeLimit    = 0,
+                           const bool           quiet           = false,
+                           const bool           daemonMode      = false,
+                           struct TagItem*      tags            = NULL);
 
    protected:
    int                RSerPoolSocketDescriptor;
