@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: terminal.c 2076 2008-12-28 15:52:34Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: terminal.c 1288 2007-01-11 20:07:41Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "rserpool.h"
@@ -31,6 +35,11 @@
 #include "componentstatuspackets.h"
 #endif
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
+
 
 /* ###### Main program ################################################### */
 int main(int argc, char** argv)
@@ -39,10 +48,11 @@ int main(int argc, char** argv)
    struct rsp_sndrcvinfo        rinfo;
    union rserpool_notification* notification;
    char                         buffer[65536 + 4];
-   char*                        poolHandle = "EchoPool";
+   const char*                  poolHandle = "EchoPool";
    struct pollfd                ufds[2];
    ssize_t                      received;
    ssize_t                      sent;
+   size_t                       failovers = 0;
    int                          result;
    int                          flags;
    int                          sd;
@@ -83,7 +93,7 @@ int main(int argc, char** argv)
       perror("Unable to create RSerPool socket");
       exit(1);
    }
-   if(rsp_connect(sd, (unsigned char*)poolHandle, strlen(poolHandle)) < 0) {
+   if(rsp_connect(sd, (unsigned char*)poolHandle, strlen(poolHandle), 0) < 0) {
       perror("Unable to connect to pool element");
       exit(1);
    }
@@ -120,9 +130,12 @@ int main(int argc, char** argv)
                   puts("\x1b[0m");
                   if((notification->rn_header.rn_type == RSERPOOL_FAILOVER) &&
                      (notification->rn_failover.rf_state == RSERPOOL_FAILOVER_NECESSARY)) {
-                     puts("FAILOVER...");
-                     usleep(200000);
-                     rsp_forcefailover(sd);
+                     failovers++;
+                     printf("FAILOVER #%u ...\n", (unsigned int)failovers);
+/*static int yyy=0;yyy++;
+if(yyy>=2) exit(1);*/
+                     /* usleep(200000); */
+                     rsp_forcefailover(sd, FFF_NONE, 0);
                   }
                }
                else {
