@@ -1,4 +1,4 @@
-/* $Id: thread.h 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: thread.h 2135 2009-07-07 09:20:40Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -41,8 +41,20 @@ class TDThread : public TDMutex
    TDThread();
    virtual ~TDThread();
 
+   inline bool isRunning() const {
+      return(MyThread != 0);
+   }
+   inline bool isStopping() {
+      lock();
+      const bool stopping = Stopping;
+      unlock();
+      return(stopping);
+   }
+
    virtual bool start();
+   virtual void stop();
    void waitForFinish();
+
    static void delay(const unsigned int us);
 
    protected:
@@ -51,9 +63,10 @@ class TDThread : public TDMutex
    protected:
    pthread_t MyThread;
 
-   private:
-   static void* startRoutine(void* object);
+   private:      
+   static void* startRoutine(void* object);   
+
+   bool         Stopping;
 };
 
-
 #endif
