@@ -1,4 +1,4 @@
-/* $Id: timer.c 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: timer.c 2155 2009-07-11 14:52:04Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -81,6 +81,7 @@ void timerStart(struct Timer*            timer,
    result = simpleRedBlackTreeInsert(&timer->Master->TimerStorage,
                                      &timer->Node);
    CHECK(result == &timer->Node);
+   timer->Master->AddRemove = true;
    dispatcherUnlock(timer->Master);
 }
 
@@ -105,14 +106,15 @@ bool timerIsRunning(struct Timer* timer)
 void timerStop(struct Timer* timer)
 {
    struct SimpleRedBlackTreeNode* result;
+   dispatcherLock(timer->Master);
    if(simpleRedBlackTreeNodeIsLinked(&timer->Node)) {
-      dispatcherLock(timer->Master);
       result = simpleRedBlackTreeRemove(&timer->Master->TimerStorage,
                                         &timer->Node);
       CHECK(result == &timer->Node);
-      dispatcherUnlock(timer->Master);
       timer->TimeStamp = 0;
+      timer->Master->AddRemove = true;
    }
+   dispatcherUnlock(timer->Master);
 }
 
 
