@@ -1,4 +1,4 @@
-/* $Id: transportaddressblock.c 2086 2009-01-07 10:29:41Z dreibh $
+/* $Id: transportaddressblock.c 2223 2009-09-22 16:24:59Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //=====  //   //      //
@@ -230,12 +230,10 @@ struct TransportAddressBlock* transportAddressBlockDuplicate(const struct Transp
       duplicate = (struct TransportAddressBlock*)malloc(size);
       if(duplicate) {
          memcpy(duplicate, transportAddressBlock, size);
+         return(duplicate);
       }
    }
-   else {
-      duplicate = NULL;
-   }
-   return(duplicate);
+   return(NULL);
 }
 
 
